$nomod51
$symbols
$debug
$nolist
$include (\i\rg51fb.pdf)
$list
%set (hasgain,1)
;
;This is a test of the PWM output of the 8051fx family of parts.
;
;It will generate a sine wave of arbitrary frequency by DDS techniques.
;with a 16.000 Mhz clock, the sample frequency, Fs, will be 15.625 Khz.
; output frequency will be: Fo = Fs * ( Phase_inc / 65536 )
;
; This particular variation will generate 3 sine waves at a time, each with
;  independent amplitude control.  It uses 76 of 85 available
;  cycles per interrupt, for approximately 89%' processor time utilization.'
;
;  Phase_inc = (TableSize * 256 * Frequency)/Fs
;  phase_inc = 65536 / 15645 * Frequency  = 4.1943 * Freq.
; for standard DTMF tones the following table can be used:
;row	Phase_inc
;697 	2923 = 0b6bh	1 2 3  a
;770	3230 = 0c9eh	4 5 6  b
;852	3574 = 0df6h	7 8 9  c
;941	3947 = 0f6bh 	* 0 #  d
;col		     	| | |  |
;1209	5071 = 13cfh ---+ | |  |
;1336	5604 = 15e4h -----+ |  |
;1477	6195 = 1833h -------+  |
;1633   6849 = 1ac1h ----------+
;
;440    1845 = 0735h
;
; This technique allows synthesis of Sine waves from 7.8Khz to .24Hz with
; .24Hz resolution.
;
data_seg segment data
code_seg segment code
;
cseg at reset
	ljmp	start
;reserve interrupt vectors
;
cseg at exti0
	reti
;
cseg at timer0
	reti
;
cseg at exti1
	reti
;
cseg at timer1
	reti
;
cseg at sint
	reti
;
; the following interrupt vector is for the 8052
Timer2	equ	002bh
; the following interrupt vector is for the 8051Fx
pca	equ	0033h
;
cseg at timer2
	reti
;
cseg at pca
	ljmp	DDS_isr
;
dseg	at 18h
;
Phase_acc:	ds	2	; big endian
Phase_inc:	ds	2	; msb,lsb
NewVal:		ds	1		; new output value
Phase2acc:	ds	2
Phase2inc:	ds	2
Phase3acc:	ds	2
Phase3inc:	ds	2
Gain1:		ds	1	; gain multipliers for pre-emphasis
Gain2:		ds	1	; fractional value from 0/256 to 255/256
Gain3:		ds	1
delay:		ds	1
delay2:		ds	1
;
;
cseg
;
DDS_isr:
	clr	ccf3            ; clear interrupting flag
	inc	ccap3h		; next interrupt occurs 256 cycles later
	push	acc
	push	b
	push	psw
	push	dph
	push	dpl
	mov	dptr,#SinTab
;				; Phase_acc := Phase_acc + Phase_inc
	mov	a,phase_inc+1	;  LSB
	add	a,Phase_acc+1
	mov	phase_acc+1,a

	mov	a,phase_inc	;  MSB
	addc	a,Phase_acc
	mov	phase_acc,a
;
	movc	a,@a+dptr       ; PWM = sine(hi(Phase_Acc))
%IF (%hasgain) THEN (
	mov	b,Gain1		; multiply by gain factor
	mul	ab
	mov	newval,b
)ELSE(
      	mov	newval,a
)FI
; Calculate second sine wave	; Phase2acc += Phase2inc
	mov	a,phase2inc+1	;  LSB
	add	a,Phase2acc+1
	mov	phase2acc+1,a

	mov	a,phase2inc	;  MSB
	addc	a,Phase2acc
	mov	phase2acc,a
;				; PWM = sine(hi(Phase2Acc))
	movc	a,@a+dptr
%IF (%hasgain)THEN(
	mov	b,gain2		; multiply by gain factor
	mul	ab
	mov	a,b
)FI
	add	a,newval	; add the two values
	mov	newval,a
;
; Calculate third  sine wave	; Phase2acc += Phase2inc
	mov	a,phase3inc+1	;  LSB
	add	a,Phase3acc+1
	mov	phase3acc+1,a

	mov	a,phase3inc	;  MSB
	addc	a,Phase3acc
	mov	phase3acc,a
;				; PWM = sine(hi(Phase2Acc))
	movc	a,@a+dptr
%IF ( %hasgain ) THEN (
	mov	b,gain3		; multiply by gain factor
	mul	ab
	mov	a,b
)FI
	add	a,newval	; add the two values
	mov	ccap4h,a        ; output the new value
;
	pop	dpl
	pop	dph
	pop	psw
	pop	b
	pop	acc
dds_end:
	reti
;
;**********************************
;
Start:	mov	ie,#0
	mov	sp,#40h

	; initialize PCA
	mov	cmod,#02h	;select clock = 1/4 Fosc
	mov	cl,#0
	mov	ch,#0

 	mov	ccapm4,#43h	; module 4 in PWM mode
	mov	ccap4l,#00h
	mov	ccap4h,#80h	; start with .50 duty cycle

	; Use PCA channel 3 as interrupt source.
	mov	ccapm3,#49h	; module 3= interrupt on compare
	mov	ccap3l,#0
	mov	ccap3h,#0
	setb	ec		; enable pca INT

	;initialize DDS variables
	orl	psw,#18h	; sel rb3
	mov	phase_inc,#0bh
	mov	phase_inc+1,#6bh
	mov	phase2inc,#13h
	mov	phase2inc+1,#0cfh
	mov	phase3inc,#07h
	mov	phase3inc+1,#35h
	mov	gain1,#255
	mov	gain2,#255
	mov	gain3,#255

	anl	psw,# not 18h	; sel rb0

	setb	cr		; turn on PCA timer.
	setb	ea		; enable all interrupts

loop:	clr	a
	mov	dptr,#rowTab
	jb	p1.3,rowdown
	inc	a
	jb	p1.4,rowdown
	inc	a
	jb	p1.5,rowdown
	inc	a
	jb	p1.6,rowdown
	mov	phase_inc,#0
	mov	phase_inc+1,#0
	mov	delay,#040h
	mov	gain1,#0f8h
testcol:
	clr	a
	mov	dptr,#ColTab
	jb	p3.3,ColDown
	inc	a
	jb	p3.1,testc2
	inc	a
	sjmp	testc3
testc2:	jnb	p3.0,colDown	; if both buttons down, do col4
	mov	a,#3
	sjmp	colDown
testc3:	jb	p3.0,ColDown
	mov	phase2inc,#0
	mov	phase2inc+1,#0
testcr:	djnz	delay,nodecay
	mov	a,gain1
	mov	b,#0ffh
	mul	ab
	mov	gain1,b
nodecay:
	jmp	loop		;  hang in there waiting for interrupts
;
rowdown:
	clr	c
	rlc	a
	mov	b,a
	movc	a,@a+dptr
	mov	phase_inc,a
	mov	a,b
	inc	a
	movc	a,@a+dptr
	mov	phase_inc+1,a
	jmp	testcol
	nop
;
ColDown:
	clr	c
	rlc	a
	mov	b,a
	movc	a,@a+dptr
	mov	phase2inc,a
	mov	a,b
	inc	a
	movc	a,@a+dptr
	mov	phase2inc+1,a
	jmp	testcr

;
RowTab:
	dw	0b6bh ; 1 2 3  a  697 Hz
	dw	0c9eh ; 4 5 6  b  770
	dw	0df6h ;	7 8 9  c  852
	dw	0f6bh ;	* 0 #  d  941
ColTab:	   	      ;	| | |  |
	dw	13cfh ;-+ | |  |  1209Hz
	dw	15e4h ;---+ |  |  1336
	dw	1833h ;-----+  |  1477
	dw	1ac1h ;--------+  1633
;
sintab:
    DB  02AH,02BH,02CH,02DH,02EH,02FH,030H,031H
    DB  032H,033H,034H,035H,036H,037H,038H,039H
    DB  03AH,03BH,03CH,03DH,03EH,03FH,040H,040H
    DB  041H,042H,043H,044H,045H,045H,046H,047H
    DB  048H,048H,049H,04AH,04AH,04BH,04CH,04CH
    DB  04DH,04DH,04EH,04FH,04FH,050H,050H,050H
    DB  051H,051H,052H,052H,052H,052H,053H,053H
    DB  053H,053H,054H,054H,054H,054H,054H,054H
    DB  054H,054H,054H,054H,054H,054H,054H,053H
    DB  053H,053H,053H,052H,052H,052H,052H,051H
    DB  051H,050H,050H,050H,04FH,04FH,04EH,04DH
    DB  04DH,04CH,04CH,04BH,04AH,04AH,049H,048H
    DB  048H,047H,046H,045H,045H,044H,043H,042H
    DB  041H,040H,040H,03FH,03EH,03DH,03CH,03BH
    DB  03AH,039H,038H,037H,036H,035H,034H,033H
    DB  032H,031H,030H,02FH,02EH,02DH,02CH,02BH
    DB  02AH,029H,028H,027H,026H,025H,024H,023H
    DB  022H,021H,020H,01FH,01EH,01DH,01CH,01BH
    DB  01AH,019H,018H,017H,016H,015H,014H,014H
    DB  013H,012H,011H,010H,00FH,00FH,00EH,00DH
    DB  00CH,00CH,00BH,00AH,00AH,009H,008H,008H
    DB  007H,007H,006H,005H,005H,004H,004H,004H
    DB  003H,003H,002H,002H,002H,002H,001H,001H
    DB  001H,001H,000H,000H,000H,000H,000H,000H
    DB  000H,000H,000H,000H,000H,000H,000H,001H
    DB  001H,001H,001H,002H,002H,002H,002H,003H
    DB  003H,004H,004H,004H,005H,005H,006H,007H
    DB  007H,008H,008H,009H,00AH,00AH,00BH,00CH
    DB  00CH,00DH,00EH,00FH,00FH,010H,011H,012H
    DB  013H,014H,014H,015H,016H,017H,018H,019H
    DB  01AH,01BH,01CH,01DH,01EH,01FH,020H,021H
    DB  022H,023H,024H,025H,026H,027H,028H,029H


	end
