; Global definitions for Graphic Routines
; Some of these are duplicated in DRIVER.H for the C code

              .XLIST
;----------
; Galactic Constants

VINT          EQU      0AH                  ; video interrupt number
                                            ; IRQ 2 -> INT A

;----------
; Structures for things updated during vertical interrupts

motion        STRUC
current       DW       0                    ; current value
interval      DW       0                    ; rate downcounter
delay         DW       0                    ; startup delay downcounter
direction     DW       0                    ; increment at rate intervals
minimum       DW       0                    ; lowest value
maximum       DW       0                    ; highest value
rate          DW       0                    ; rate counter reload
motion        ENDS

;----------
; Vertical interrupt handler flag bit definitions

vint_bits     RECORD \
     vintsuppress:1, \
     vint_bx:2, \
     align:1, \
     vbounce:1,vrun:1,vstep:1, \
     hbounce:1,hrun:1,hstep:1, \
     sbounce:1,srun:1,sstep:1, \
     vscroll:1,hpan:1,bothvs:1

vint_mbits    RECORD \
     hpipeline:1, \
     isvga:1, \
     vint_mx:14

;----------
; EGA and VGA registers

;--- CRT Controller Registers

CRT_HTOTAL    EQU      0
CRT_HEND      EQU      1
CRT_HBLST     EQU      2
CRT_HBLEND    EQU      3
CRT_HRTST     EQU      4
CRT_HRTEND    EQU      5
CRT_VTOTAL    EQU      6
CRT_OFLOW     EQU      7
CRT_PRESET    EQU      8
CRT_MAXLINE   EQU      9
CRT_CURST     EQU      0ah
CRT_CUREND    EQU      0bh
CRT_STHIGH    EQU      0ch
CRT_STLOW     EQU      0dh
CRT_CURHI     EQU      0eh
CRT_CURLOW    EQU      0fh
CRT_VRTST     EQU      10h
CRT_LPHI      EQU      10h
CRT_VREND     EQU      11h
CRT_LPLOW     EQU      11h
CRT_VEND      EQU      12h
CRT_OFFSET    EQU      13h
CRT_ULINE     EQU      14h
CRT_VBLST     EQU      15h
CRT_VBLEND    EQU      16h
CRT_MODE      EQU      17h
CRT_COMPARE   EQU      18h

;--- Sequencer Registers

SQR_RESET     EQU      0
SQR_CLOCK     EQU      1
SQR_MASK      EQU      2
SQR_CHAR      EQU      3
SQR_MEM       EQU      4

;--- Graphic Controller register names

GCR_RESET     EQU      0
GCR_ENABLE    EQU      1
GCR_COMP      EQU      2
GCR_ROTATE    EQU      3
GCR_RDMAP     EQU      4
GCR_MODE      EQU      5
GCR_MISC      EQU      6
GCR_DC        EQU      7
GCR_MASK      EQU      8

GC_WRMODE0    EQU      00H                  ; write mode 0 (use CPU pel data)
GC_WRMODE2    EQU      02H                  ; write mode 2 (CPU sets plane)
GC_DRAW       EQU      00H                  ; replace latched data w/CPU data
GC_AND        EQU      08H                  ;   AND
GC_OR         EQU      10H                  ;    OR
GC_XOR        EQU      18H                  ;   XOR

;--- Attribute Controller register names
;    the odd 20h enables EGA access to palette regs during updates

ACR_PAL0      EQU      0                    ; 0-F are individual palette
ACR_MODE      EQU      10h
ACR_BORD      EQU      11h
ACR_PLANE     EQU      12h
ACR_PAN       EQU      13h+20h
ACR_SELECT    EQU      14h

;--- Port bit locations

ISR0_bits     RECORD   CRT_int:1,ISR0_1:2,sw_sense:1,ISR0_2:4
ISR1_bits     RECORD   ISR1_1:2,ISR1_diag:2,CRT_retr:1,LP_sw:1,LP_en:1,CRT_en:1
CRTC_bits09   RECORD   CRTC_09a:1,LCOMP9:1,CRTC_09b:6
CRTC_bits11   RECORD   CRTC_11:1,VINT_en:1,VINT_clr:1,VEND:4
CRTC_bits18   RECORD   CRTC_18:2,CLOC8:1,LCOMP8:1,VBL8:1,VRET8:1,VDE8:1,VTOT8:1
ACR_bits10    RECORD   ACR_10a:2,HPAN_en:1,ACR_10b:5

;--------------
; Interrupt controller stuff

I8259_1       EQU      20H                  ; master interrupt controller
EOI           EQU      20H                  ; non-specific end of interrupt

;----------
; Stack frame resulting from PUSHA, PUSH ES, PUSH DS, and MOV BP,SP

allpushed     STRUC
DSstk         DW       0                    ; pushed separately
ESstk         DW       0                    ; pushed separately
DIstk         DW       0
SIstk         DW       0
BPstk         DW       0
SPstk         DW       0                    ; before PUSHA started
BXstk         DW       0
DXstk         DW       0
CXstk         DW       0
AXstk         DW       0
allpushed     ENDS


              PAGE
;----------
; DOS functions

charout       EQU      02H                  ; show char from DL
showstr       EQU      09H                  ; show string from DX
setvect       EQU      25H                  ; set interrupt vector from DS:DX
getdate       EQU      2AH                  ; get date yr:mo:day, day of week
gettime       EQU      2CH                  ; get time of day hr:min:sec:10ms
getvect       EQU      35H                  ; get interrupt vector in ES:BX
malloc        EQU      48H                  ; allocate memory block
free          EQU      49H                  ; free memory block
termrc        EQU      4cH                  ; terminate with return code

;----------
; BIOS functions

BIOS_sserv    EQU      15H                  ; system services interrupt
BIOS_stime    EQU      1AH                  ; system timer services
syscfg        EQU      0C0H                 ;  return system configuration

sys_desc      STRUC                         ; system description structure
sys_bcnt      DW       0                    ; byte count of following data
sys_mod       DB       0                    ; system model number
sys_submod    DB       0                    ; system submodel number
sys_blev      DB       0                    ; BIOS revision level (0..)
sys_feat      DB       0                    ; system feature information
sys_desc      ENDS

;----------
; General constants

null          EQU      00H                  ; null character
tab           EQU      09H                  ; tab
lf            EQU      0aH                  ; line feed
cr            EQU      0dH                  ; carriage return
blank         EQU      ' '                  ; blank character
termin        EQU      '$'                  ; DOS end-of-string flag

firstprt      EQU      20H                  ; first printable ASCII char
lastprt       EQU      7fH                  ; last printable ASCII char

              .LIST
