;-----------
; The 8x8 byte multiply routine

; Copyright (c) 1988, Circuit Cellar Inc.
;   All Rights Reserved

;-----------
; Constants and externals

$SHOWMACS
%INCLUDE "defines.inc"
%INCLUDE "macros.inc"

           %FOR    N = 0 TO NUMLEN-1
           EXTRN   mpy&N(DATA)
           EXTRN   prod&N(DATA)
           %ENDFOR

           EXTRN   compute(BIT)
           EXTRN   done(BIT)
           EXTRN   squaring(BIT)
           EXTRN   mcand_sign(BIT)
           EXTRN   mpy_sign(BIT)
           EXTRN   prod_sign(BIT)
           EXTRN   R00(DATA)
           EXTRN   R01(DATA)
           EXTRN   R02(DATA)
           EXTRN   R03(DATA)

           EXTRN   clamp(CODE)
           EXTRN   long_abs(CODE)
           EXTRN   long_neg(CODE)
           EXTRN   restart(CODE)

;-----------
; Multiply two signed integers
; R0 -> multiplicand high byte
; R1 -> multiplier high byte
; Product returned in high order half of buffer
; Result is clamped to +3.999
; The multiplier core is unsigned, so we have to fiddle the signs around it
; Mashes A, B, R0, and R1
; Uses R2 and R3 to hold incoming R0 and R1

; Because this takes so long, we check to make sure it's not been cancelled
;  in mid flight... if so, we flush the stack and return to the main loop

; Remember that the numbers are stored with the high byte in the low address!
;  This makes 'em read correctly when displayed on the screen...

; Input numeric format is
;  sign 0 0 0 <point> 0 0 0 0  ...


long_mult  PROC
           PUBLIC  long_mult

           %IF     1                   ; shorten this a whole lot!

;-----------
; Fix the input values and figure out the sign of the product
; If R0 = R1, the result is positive & we don't change signs twice!
; If we're squaring a number, short-circuit if it's >= 2

           MOV     R2,R00              ; save incoming pointers
           MOV     R3,R01

           CLR     mcand_sign          ; assume all positive
           CLR     mpy_sign
           CLR     prod_sign

           CALL    long_abs            ; multiplicand at R0
           MOV     mcand_sign,C

           JNB     squaring,no_sqrin   ; if not squaring, figure other signs
           MOV     A,@R0
           ANL     A,#0E0h             ; >= 2.0?
           JZ      pplus               ; no, continue with multiply
           SETB    done                ; yes, signal done now!
           JMP     signfix             ;    and fix signs if needed

no_sqrin   EQU     $
           MOV     R0,R01              ; set up multiplier
           CALL    long_abs
           MOV     mpy_sign,C
           MOV     R0,R02              ; restore multiplicand pointer

;--- if both incoming signs are the same, the product is positive

           CLR     A
           MOV     ACC.0,C
           MOV     C,mcand_sign
           ADDC    A,#0
           CJNE    A,#1,pplus          ; if sum isn't 1, signs were same
           SETB    prod_sign           ; sum was 1, so product is -

pplus      EQU     $

;-----------
; Move multiplier into high half of buffer

           %FOR    I=NUMLEN-1 TO 0 BY -1
           MOV     mpy&I,@R1
           INC     R1
           %ENDFOR

           MOV     A,R1                ; reset input pointer
           ADD     A,#-NUMLEN
           MOV     R1,A

;-----------
; Special-case the first part of the product

           MOV     A,R0                ; point to low multiplicand byte
           ADD     A,#NUMLEN-1
           MOV     R0,A

           MOV     A,@R0               ; low multiplicand byte
           DEC     R0
           MOV     B,mpy0              ; low multiplier byte
           MUL     AB                  ; BA = A*B
           MOV     prod0,A             ; low product byte
           MOV     prod1,B             ; high product byte

;-----------
; Clunk through rest of multiplicand
; This special-cases the carry propagation, because product is mostly zero
; Remember that high bytes are stored in lower addresses than low bytes...

           MOV     DPL,mpy0            ; set up multiplier byte
           MOV     mpy0,#0             ; and clear for results

           %FOR    I = 1 TO NUMLEN-1   ; multiplicand index

           JB      compute,m0&I        ; check for cancel flag
           JMP     restart             ;  ... exit if off

m0&I       EQU     $

           MOV     A,@R0               ; next multiplicand byte
           DEC     R0
           MOV     B,DPL               ; low multiplier byte
           MUL     AB

           ADD     A,prod&I            ; combine low byte
           MOV     prod&I,A

           CLR     A                   ; set high byte
           ADDC    A,B                 ; this can't generate a carry
           MOV     prod&I-1,A

           %ENDFOR

;--- reset multiplicand pointer

           MOV     A,R0
           ADD     A,#NUMLEN
           MOV     R0,A

;-----------
; Rub remaining multiplier bytes across multiplicand

           %FOR    J = 1 TO NUMLEN-1   ; multiplier index

           MOV     DPL,mpy&J           ; set up multiplier byte
           MOV     mpy&J,#0            ; and clear for results

           JB      compute,m&J&top     ; check for cancel flag
           JMP     restart             ;  ... exit if off
m&J&top    EQU     $

           %FOR    I = 0 TO NUMLEN-1   ; multiplicand index

m&J&&I     EQU     $

           MOV     A,@R0               ; next multiplicand byte
           DEC     R0
           MOV     B,DPL               ; multiplier byte
           MUL     AB

           ADD     A,prod&J-&I         ; combine low byte
           MOV     prod&J-&I,A

           MOV     A,B                 ; combine high byte
           ADDC    A,prod&J-&I-1
           MOV     prod&J-&I-1,A

           %IF     &I LT NUMLEN-1      ; no carry past multiplier byte
           %FOR    N = 2 TO NUMLEN-&I  ;  ... but all remaining bytes

           JNC     noc&J&&I
           CLR     A
           ADDC    A,prod&J-&I-&N
           MOV     prod&J-&I-&N,A

           %ENDFOR
           %ENDIF

noc&J&&I   EQU     $

           %ENDFOR                     ; multiplicand loop

           %IF     &J NE NUMLEN-1      ; special reset for last byte
           MOV     A,R0                ; point to last byte
           ADD     A,#NUMLEN
           MOV     R0,A
           %ELSE
           INC     R0                  ; point to first byte
           %ENDIF

           %ENDFOR                     ; multiplier loop

haveprod   EQU     $

;-----------
; Round the product to NUMLEN bytes
; This won't cause an overflow because we have an extra
;  nibble on the high end

           MOV     A,prod0-(NUMLEN-1)
           JNB     ACC.3,noround       ; skip if not needed

           ADD     A,#10H              ; start rounding!
           MOV     prod0-(NUMLEN-1),A
           JNC     noround             ; break out if done

           %FOR    N=0 TO NUMLEN-1
           CLR     A
           ADDC    A,mpy&N
           MOV     mpy&N,A
           JNC     noround             ; break out if done
           %ENDFOR

noround    EQU     $

;-----------
; Align the high order NUMLEN bytes by shifting left 4 bits
; If the result is >= 4 it gets clamped to 3.9999

align      EQU     $

           MOV     A,mpy0-(NUMLEN-1)   ; handle first byte
           ANL     A,#0FCH             ; result >= 4?
           JZ      lm_OK
           CALL    clamp               ; yet, clamp it
           JMP     lm_aligned

lm_OK      EQU     $
           MOV     A,mpy0-(NUMLEN-1)   ; get first byte again
           SWAP    A
           ANL     A,#0F0H             ; remove low nibble
           MOV     mpy0-(NUMLEN-1),A   ; save for now

           %FOR    N = NUMLEN-2 TO 0 BY -1

           MOV     A,mpy&N             ; get new byte
           MOV     B,#10H
           MUL     AB                  ; shift and separate nibbles

           MOV     mpy&N,A             ; save low part for next pass

           MOV     A,mpy&N-1           ; combine with previous byte
           ORL     A,B
           MOV     mpy&N-1,A

           %ENDFOR

           MOV     A,prod0-(NUMLEN-1)  ; special-case last byte
           MOV     B,#10H
           MUL     AB

           MOV     A,mpy0
           ORL     A,B
           MOV     mpy0,A

lm_aligned EQU     $

;-----------
; Fix up the signs of all participants

signfix    EQU     $

           JNB     mcand_sign,mcandOK
           MOV     R0,R02
           CALL    long_neg
mcandOK    EQU     $

           JB      squaring,prodOK               ; finished if squaring

nosqr_out  EQU     $
           JNB     mpy_sign,mpyOK
           MOV     R0,R03
           CALL    long_neg
mpyOK      EQU     $

           JNB     prod_sign,prodOK
           MOV     R0,#mpy7
           CALL    long_neg

prodOK     EQU     $

           %ENDIF

           RET

long_mult  ENDPROC

           END
