              .XLIST

;----------
; Write a register number/value pair to a port pair
; Uses byte writes to ensure clone compatibility
; argument = register number
; AH = data value
; DX = index port (the lower-numbered one)
; returns AH and AL swapped
;         DX unchanged


regset        MACRO    reg

              MOV      AL,reg
              punt
              CLI
              OUT      DX,AL                ; set register number

              XCHG     AL,AH
              INC      DX                   ; point to data port

              punt
              OUT      DX,AL                ; set data
              STI

              DEC      DX                   ; reset to address port

              ENDM

;----------
; Write an attribute controller number/value pair to a port pair
; Uses byte writes to ensure clone compatibility
; argument = register number
; AH = data value
; returns AH and AL swapped


ACRset        MACRO    reg

              PUSH     DX                   ; save bystander

              MOV      DX,ISR1_port         ; reset address flipflop
              punt
              CLI
              IN       AL,DX

              MOV      DX,AC_port

              MOV      AL,reg
              punt
              OUT      DX,AL                ; set register number

              XCHG     AL,AH

              punt
              OUT      DX,AL                ; set data
              STI

              POP      DX                   ; restore bystander
              ENDM

;----------
; Read a register value from port/reg pair
; argument = register number
; AH = data value
; DX = index port (the lower-numbered one)
; returns register in AH
;         DX unchanged
; This is useful only on VGAs, but we don't check here!


regget        MACRO    reg

              MOV      AL,reg
              punt
              CLI
              OUT      DX,AL                ; set register number

              INC      DX                   ; point to read port
              punt
              IN       AL,DX                ; get data
              STI

              MOV      AH,AL                ; set up for return

              DEC      DX                   ; point to address port

              ENDM

;----------
; Read an Attribute Controller register value
; argument = register number
; returns data in AH
; This is useful only on VGAs, but we don't check here!


ACRget        MACRO    reg

              PUSH     DX                   ; save bystander

              MOV      DX,ISR1_port         ; reset address flipflop
              punt
              CLI
              IN       AL,DX

              MOV      DX,AC_port

              MOV      AL,reg
              punt
              OUT      DX,AL                ; set register number

              INC      DX                   ; point to read port
              punt
              IN       AL,DX                ; get data
              STI
              MOV      AH,AL                ; set up for return

              POP      DX                   ; restore bystander

              ENDM

;----------
; Ensure enough delay for I/O recovery

punt          MACRO    fn
              LOCAL    iowait

              JMP      SHORT iowait
iowait        LABEL    NEAR

              ENDM

;----------
; Invoke DOS functions

DOScall       MACRO    fn

              MOV      AH,fn
              INT      21H

              ENDM

;----------
; Invoke BIOS functions

BIOSint       MACRO    intnum,fn

              MOV      AH,fn
              INT      intnum

              ENDM

              .LIST
