; ----------------------------------------------------
; Adc1.asm    Last Update:10/27/91
;
; Simple Analog-to-Digital Converter Example
;
; Shows analogue input 0 (AN0) in hexadecimal
; on the 2 digital displays
;
; The voltage seen at the microcontroller pin
; is displayed in hexadecimal (0..ff), 00=0
; volts, and FF=5 volts.  The jumpers jp2
; (vgnd) and jp3 (vref) should be installed.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD Connections:
;  LOW DIGIT to Port 4 and HIGH DIGIT to Port 5.
;  80535 port AN0 to UIOD potentiometer AN0.
;
; ----------------------------------------------------
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 break     equ  010ch  ; a, (reads accumulator)
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 sdelay    equ  0142h  ; a
; ----------------------------------------------------

    org 8000h

begin:
    lcall getAN0       ; convert and get AN0 value
    push  acc          ; save Analog0 on stack
    anl   a, #0fh      ; isolate the low nibble
    lcall display      ; convert to display pattern
    mov   P4, a        ; display low nibble on LOW DIGIT
    pop   acc          ; recall Analog0
    swap  a            ; swap nibbles
    anl   a, #0fh      ; isolate low nibble
    lcall display      ; get display pattern
    mov   p5, a        ; display on HIGH DIGIT

    mov   a, #1        ; wait for...
    lcall sdelay       ; 1 second

    lcall inkey        ; see if there is a CTL-C pending
    lcall break        ; if so quit
    sjmp  begin        ; else repeat

; ====================================================
;  subroutine getAN0 - get analog 0
;  input    : none
;  output   : returns the digital value of analog
;             channel 0 in the accumulator
;  destroys : a, psw
; ----------------------------------------------------
getAN0:
        jb    BSY, getAN0 ; wait here if ADC in progress
        mov   a, adcon    ; read adc control register
        anl   a, #0d0h    ; select channel 0, single shot
        mov   adcon, a
        mov   dapr, #0    ; start conversion
here:
        jb    BSY, here   ; wait here if ADC in progress
        mov   a, ADDAT    ; read the digital value
        ret

; ====================================================
;
; end of program
