; ----------------------------------------------------
; Adc4.asm    Last Update:10/27/91
;
; Simple Interrupt-Driven Analog-to-Digital
; Converter Example
;
; Shows analogue input 0 (AN0) in hexadecimal
; on the 2 digital displays
;
; The voltage seen at the microcontroller pin
; is displayed in hexadecimal (0..ff), 00=0
; volts, and FF=5 volts.  The jumpers jp2
; (vgnd) and jp3 (vref) should be installed.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD Connections:
;  LOW DIGIT to Port 4 and HIGH DIGIT to Port 5.
;  80535 port AN0 to UIOD potentiometer AN0.
;
; internal memory use
; bytes:
   analog0 equ 0c0h    ;  0-th adc input
; ----------------------------------------------------
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 break     equ  010ch  ; a, (reads accumulator)
 crlf      equ  0115h  ; a
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 print     equ  0136h  ; a, dptr
 prthex    equ  013fh  ; a, r2
 setintvec equ  0145h  ; a, dptr
; ----------------------------------------------------

    org 8000h

    lcall initadc  ; initialize ADC
    setb  eal      ; master interrupt enable flag
begin:
    mov   r0, #Analog0 ; get Analog0 address
    mov   a, @r0       ; read Analog0
    push  acc          ; save Analog0 on stack
    anl   a, #0fh      ; isolate the low nibble
    lcall display      ; convert to display pattern
    mov   P4, a        ; display low nibble on LOW DIGIT
    pop   acc          ; recall Analog0
    swap  a            ; swap nibbles
    anl   a, #0fh      ; isolate low nibble
    lcall display      ; get display pattern
    mov   p5, a        ; display on HIGH DIGIT

    mov   a, #0        ; wait for...
    lcall mdelay       ; 256 milliseconds

    lcall inkey        ; see if there is a CTL-C pending
    cjne  a, #3h, begin; if not repeat
    clr   eal          ; if CTL-C, clear master interrupt
                       ; enable flag
    lcall break        ; quit

; ====================================================
;  subroutine initadc - initialize adc
;  input    : none
;  output   : initializes adc interrupt
;             to continuously read analog 0
;  destroys : a, dptr
; ----------------------------------------------------
initadc:
        clr   eadc        ; disable adc interrupts
        clr   iadc        ; reset adc interrupt flag
        mov   a, #6       ; sixth interrupt source
        mov   dptr, #adcisr ; setup interrupt vector
        lcall setintvec
        setb  eadc        ; adc interrupt mask bit

        mov   a, adcon    ; read adc control register
        anl   a, #0d0h    ; select channel 0, single shot
        mov   adcon, a
        mov   dapr, #0    ; start conversion
        ret

; ====================================================
;  subroutine adcisr
;  input    : adc interrupt
;  output   : continuously obtains and stores analog 0
;             value in internal ram.
;  destroys : nothing - uses a, r0, and r1
; ----------------------------------------------------
adcisr:
    push  psw
    mov   psw, #0        ; select register bank 0
    push  acc
    push  1              ; push register 1

    mov   r0,  #analog0  ; analog 0 store address
    mov   a,   addat     ; read analog value
    mov   @r0, a         ; store analog value
    mov   a, adcon       ; adc control register
    anl   a, #0d0h
    mov   adcon, a       ; select next channel
    clr   iadc           ; reset adc interrupt flag
    mov   dapr, #0       ; start next conversion

done:
    pop   1
    pop   acc
    pop   psw
    reti

; ====================================================
;
; end of program
