$mod51
$pagelength(56)
$nosymbols
$nolist
$include(z80.inc)
$include(mon51.inc)
$list

noq	equ	8
qlink	equ	0
qtick	equ	qlink+2
qjobn	equ	qlink+4
qcomp	equ	qlink+6
soq	equ	qlink+8

	cseg at	9E00h
	jmp	inque
slicer:	jmp	sliced		; slice
execut:	jmp	exeque		; exeq
insque:	jmp	insrt		; addque

; initialize queue, called BEFORE starting timer
INIQUE:	mov	dptr,#0
	shld	exec		; none to execute
	shld	wait		; none waiting
	mov	dptr,#queue	; current node
	shld	free		; all are free
	mov	b,#noq-1	; do all but last
iql:	mov	a,dpl
	add	a,#soq
	mov	Ereg,a
	mov	a,dph
	addc	a,#0
	mov	Dreg,a		; DE = next node
	mov	a,Dreg
	movx	@dptr,a
	mov	a,Ereg
	inc	dptr
	movx	@dptr,a
	mov	dpl,Ereg	; DPTR = next, becomes current
	mov	dph,Dreg
	djnz	b,iql
	clr	a		; mark end of linked list
	movx	@dptr,a
	inc	dptr
	movx	@dptr,a
	ret
;=====================================================================
; on stack:
; Completion routine
; job number
; tick count
; Get the data off the stack and into AREA.
; This may seem redundant but the stack is limited
; and this makes for an easy block move.
; In keeping with the 8051 convention, data is 
; stored high then low except for tick which is 
; easier to manipulate if left low byte first.
; See mon51.inc for MRKT macro to insert a job.

insrt:	popbc			; get return address
	mov	dptr,#area	; PDP's used arg areas
	pop	acc
	movx	@dptr,a		; store tick low byte first
	inc	dptr
	pop	acc		; hi tick
	movx	@dptr,a
	inc	dptr
	pop	acc
	movx	@dptr,A		; store job #
	inc	dptr
	clr	a
	movx	@dptr,a		; null high job
	inc	dptr
	pop	acc
	movx	@dptr,a		; store completion hi
	inc	dptr
	pop	acc		; comp lo
	movx	@dptr,a
	pushbc			; return address back on stack
	call	addq
	ret	
;=====================================================================
; support routines put here so there are no forward references.
;=====================================================================
; entry:
; DE = Current list
; exit:
; BC = next node
;
nxtnode:
	mov	dph,Dreg
	mov	dpl,Ereg
	movx	a,@dptr
	mov	Breg,a
	inc	dptr
	movx	a,@dptr
	mov	Creg,a
	ret
;
getnode:
	movx	a,@dptr
	mov	Dreg,a
	inc	dptr
	movx	a,@dptr
	mov	Ereg,a
	ret
;
putnode:
	mov	a,Breg
	movx	@dptr,a
	inc	dptr
	mov	a,Creg
	movx	@dptr,a
	ret

cmpde:	mov	a,Dreg
	orl	a,Ereg
	clr	c
	jnz	cmpde1
	cpl	c
cmpde1:	ret
;
qadd:	mov	a,dpl
	add	a,Ereg
	mov	dpl,a
	mov	a,dph
	addc	a,Dreg
	mov	dph,a
	ret
;=====================================================================
; get a node from the EXEC list and add to the FREE list
; and go to the completion routine
exeque:	mov	dptr,#EXEC
	call	getnode		; DE = current node
	call	cmpde
	jc	qxit
	call	nxtnode		; BC = next node
	mov	dptr,#EXEC
	call	putnode		; BC has next node
	mov	dptr,#QCOMP
	call	qadd
	movx	a,@dptr
	mov	Breg,a
	inc	dptr
	movx	a,@dptr
	mov	Creg,a
	pushbc			; completion routine to stack
	pushde			; save free node
	mov	dptr,#FREE	; add current to free list

;=====================================================================
; find end of FREE, WAIT or EXEC list pointed to by DPTR 
; and insert the node currently on the stack, making that 
; node the end of the list with a null link pointer.
; The high byte address of nodes is constant.

endq:	movx	a,@dptr		; get hi addr
	jz	geol		; never a node on page 0
	inc	dptr
	movx	a,@dptr
	mov	dpl,a		; get lo addr
	jmp	endq		; keep looking
geol:	popde			; get link freed
	mov	a,Dreg
	movx	@dptr,a
	inc	dptr
	mov	a,Ereg
	movx	@dptr,a
	mov	dpl,Ereg	; next node
	mov	dph,Dreg	; becomes
	clr	a		; the new
	movx	@dptr,a		; end of
	inc	dptr		; the queue
	movx	@dptr,a		; list
qxit:	ret			; done
;=====================================================================
; get a node from the FREE list and add to the WAIT list.
; Initialize job information from data in AREA.

addq:	mov	dptr,#FREE
	call	getnode		; DE = free node
	call	cmpde		; free list empty ?
	jnc	adq1		; nope
	lcall	ilprt
	db	'Queue full.',0
	jmp	qxit		; oops
adq1:	pushde			; save node address for endq
	call	nxtnode		; BC = next node
	mov	dptr,#FREE
	call	putnode		; store next as free
	mov	dpl,Ereg	; dptr = current node
	mov	dph,Dreg
	inc	dptr
	inc	dptr		; point to qtick
	mov	p2,#high AREA	; source info
	mov	r0,#low AREA
	mov	b,#soq-2	; dont count qlink
	ldir			; block move data from area to node
	mov	dptr,#WAIT	; put node on wait list
	jmp	endq
;=====================================================================
; get a node from the WAIT list
; decrement ticks, if ticks = 0
; then add to EXEC list

sliced:	mov	dptr,#WAIT
sl1:	mov	r0,dpl			; r0 = previous node low
	movx	a,@dptr			; next node high
	jnz	sl2			; end of list ?
	ret				; yes, return to clock
sl2:	mov	Dreg,a
	inc	dptr
	movx	a,@dptr
	mov	Ereg,a			; DE = current node
	mov	dptr,#QTICK
	call	qadd			; point to qtick
	clr	c
	movx	a,@dptr
	subb	a,#1			; tick = tick - 1
	movx	@dptr,a
	mov	b,a			; save for zero check
	inc	dptr
	movx	a,@dptr
	subb	a,#0			; word extend
	movx	@dptr,a
	orl	a,b			; = 0 ?
	jz	sl3			; yes, execute it
	mov	dpl,Ereg		; DPTR = current node
	jmp	sl1
sl3:	mov	p2,dph			; p2/r0 = previous node
	mov	dpl,Ereg		; dptr  = next node
	movx	a,@dptr			; next hi
	movx	@r0,a			; previous hi
	inc	dptr
	inc	r0
	movx	a,@dptr			; next lo
	movx	@r0,a			; previous lo
	mov	dptr,#sliced
	push	dpl
	push	dph			; return back to here
	pushde				; current node to stack	
	mov	dptr,#EXEC
	jmp	endq			; add job to execute list
;=====================================================================
lastcode	set $
	dseg at	(lastcode and 0fff0h) + 10h
; all this data MUST be on the same page else tilt
; a small sacrafice for effenciency

FREE:	ds	2	; available job entries
WAIT:	ds	2	; jobs waiting to time out
EXEC:	ds	2	; jobs ready to be run
AREA:	ds	6	; qtick,qjobn,qcomp
	ds	4
queue:	ds	soq
	ds	soq
	ds	soq
	ds	soq
	ds	soq
	ds	soq
	ds	soq
	ds	soq
	end
	
