; ----------------------------------------------------
; Counter2.asm      Last Update:10/27/91
;
; Simple Counter Example
; Uses Counter/Timer 0 in Mode 2
;
; In mode 2, whenever the counter low byte rolls
; over (from FF to 0), the contents of the high
; byte (TH0) are copied to the low byte (TL0).
;
; ----------------------------------------------
; system calls              registers used
; ----------------------------------------------

 break     equ  010ch  ; a, (reads accumulator)
 crlf      equ  0115h  ; a
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prthex    equ  013fh  ; a, r2

; ====================================================
;       m a i n    p r o g r a m
; UIOD connections
;
; LOW DIGIT    P1     : output (segment a to P1.0, ...
;                               segment g to P1.6)
; PB0          P3.4 (T0) : counter input
; ----------------------------------------------------

      org   8000h         ; run using the monitor
                          ;  G8000 command
      setb  T0            ; prepare T0 for input
      anl   TMOD, #0F0h   ; mask off low nibble - keep timer 1 mode
      orl   TMOD, #6      ; configure timer/counter 0
                          ;  counter in mode 0
      mov   TH0, #0       ; reload value (try other values)
      setb  TR0           ; turn timer/counter 0 on
      mov   b, #0         ; current count

      mov   a, b          ; load a with current count
      lcall display       ; get display pattern
      mov   P1, a         ; display the digit
      mov   a, b          ; get current count
      lcall prthex        ; echo count to host
      lcall crlf          ; new line on host

loop:
      lcall inkey         ; check serial port for character
      lcall break         ; see if character is CTL-C
                          ; terminate program if CTL-C
      mov   a, TL0        ; get counter value
      anl   a, #0fh       ; modulus 0Fh
      xrl   a, b          ; compare with current count
      jz    loop          ; repeat if equal

      inc   b             ; else increment current count
      anl   b, #0Fh       ; modulus 0Fh
      mov   a, b          ; load a with current count
      lcall display       ; get display pattern
      mov   P1, a         ; display the digit
      mov   a, b          ; get current count
      lcall prthex        ; echo count to host

      lcall print         ; print a few spaces
      db "   ",0
      mov   a, TH0        ; load a with counter high byte
      lcall prsphx        ; display on host
      mov   a, TL0        ; load a with counter low byte
      lcall prsphx        ; display on host

      lcall crlf          ; new line on host

      sjmp loop           ; repeat forever...

; ====================================================
; end of program
