; ----------------------------------------------------
; DAC2.asm    Last Update:10/27/91
;
; Simple Function Generator with a Digital-to-Analog
; Converter
;
; A counter is incremeted periodically.  The digital
; value is converted to analog, producing a saw tooth
; wave signal.  The rate at which the counter is
; incremented determines the frequency.  A simple
; delay loop which decrements r7 to zero is used to
; introduce a delay and thus determine the frequency.
; Try different values for the constant Dvalue to
; change the frequency.
;
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD Connections:
;  Port 4 to the Digital-to-Analog Converter
;  Port 5 bits:
;   P5.0 to DAC WR1\  (DAC0832 pin  2)
;   P5.1 to DAC WR2\  (DAC0832 pin 18)
;
    DACdata equ 0E8h  ; P4
    WR1     equ 0F8h  ; P5.0
    WR2     equ 0F9h  ; P5.1
;
; internal memory use
; bytes:
   Counter equ 7Fh     ; store digital value
; constants:
   Dvalue  equ 10h     ; delay value
; ----------------------------------------------------
    org 8000h

    mov   p4, #0       ; start with 0 volts
    lcall DAC
    mov   Counter, #0  ; initialize counter
begin:
    mov   P4, Counter  ; output value to DAC
    lcall DAC          ; convert to analog
    inc   Counter      ; update counter

    mov   r7, #Dvalue  ; set up delay counter r7
here:
    djnz  r7, here     ; waste a little time

    sjmp  begin        ; repeat

; ====================================================
;  subroutine DAC - digital-to-analog conversion
;  input    : none
;  output   : uses WR1 and WR2 to execute a DAC.
;              DAC WR1\  (DAC0832 pin  2)
;              DAC WR2\  (DAC0832 pin 18)
;
;  destroys : Port bits connected to WR1 and WR2
; ----------------------------------------------------
DAC:
        setb  WR1   ; disable DAC
        setb  WR2   ; disable DAC
        clr   WR1   ; input latch
        setb  WR1   ;
        clr   WR2   ; output latch
        setb  WR2   ;
        ret

; ====================================================
;
; end of program
