; ----------------------------------------------------
; DAC3.asm    Last Update:10/27/91
;
; Simple Function Generator with a Digital-to-Analog
; Converter
;
; A counter is incremeted periodically.  The digital
; value is converted to analog, producing a saw tooth
; wave signal.  The rate at which the counter is
; incremented determines the frequency.  A simple
; delay loop which decrements r7 to zero is used to
; introduce a delay and thus determine the frequency.
; Try different values for the constant Dvalue to
; change the frequency.
;
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD Connections:
;  Port 4 to the Digital-to-Analog Converter
;  Port 5 bits:
;   P5.0 to DAC WR1\  (DAC0832 pin  2)
;   P5.1 to DAC WR2\  (DAC0832 pin 18)
;
    DACdata equ 0E8h  ; P4
    WR1     equ 0F8h  ; P5.0
    WR2     equ 0F9h  ; P5.1
;
; constants:
   Dvalue  equ 0FFh    ; delay value
   segment equ  90h    ; waveform starts at 9000h
; ----------------------------------------------
    org 8000h

    lcall WaveInit

    mov   dph, #segment
    mov   dpl, #0
begin:
    movx  a, @dptr     ; read value from external memory
    mov   P4, a        ; send to DAC data inputs
    lcall DAC          ; convert to analog
    inc   dpl          ; update data pointer

    mov   r7, #Dvalue
here:
    djnz r7, here

    sjmp  begin        ; repeat

; ====================================================
;  subroutine DAC - digital-to-analog conversion
;  input    : none
;  output   : uses WR1 and WR2 to execute a DAC.
;              DAC WR1\  (DAC0832 pin  2)
;              DAC WR2\  (DAC0832 pin 18)
;
;  destroys : Port bits connected to WR1 and WR2
; ----------------------------------------------------
DAC:
        setb  WR1   ; disable DAC
        setb  WR2   ; disable DAC
        clr   WR1   ; input latch
        setb  WR1   ;
        clr   WR2   ; output latch
        setb  WR2   ;
        ret

; ====================================================
;  subroutine WaveInit - initialize waveform
;   This implementation creates a triangular waveform.
;   There are two loops, each placing 128 bytes in
;   external memory.  Each loop increments the data
;   pointer by incrementing dpl.  The first loop
;   multiplies dpl by 2 and places in memory.  This
;   loop will place 2*dpl at dptr for dpl=0,1,...,7F.
;   Thus, offset 0 contains 0, 1 contains 2, etc.
;   The second complements dpl and then multiplies by
;   2.  It places 2*(FF-dpl) at dptr for dpl=80h,81h,
;   ...,FFh.  Thus, offset 80h contains FEh, 81h
;   contains FCh, etc.
;
;  input    : none
;  output   : places 256 bytes in external memory
;             starting at address high byte segment
;             and low byte 0.
;  destroys :
; ----------------------------------------------------
WaveInit:
        mov   dph, #segment ; dptr points to segment
        mov   dpl, #0
W1:
        mov   a, dpl        ; get dpl
        rl    a             ; multiply by 2
        movx  @dptr, a      ; place in external mem
        inc   dpl           ; increment dptr
        mov   a, dpl        ; repeat for 128 bytes
        cjne  a, #80h, W1
W2:
        mov   a, dpl        ; get dpl
        cpl   a             ; complement
        rl    a             ; multiply by 2
        movx  @dptr, a      ; place in external mem
        inc   dpl           ; increment dptr
        mov   a, dpl        ; repet for another 128...
        cjne  a, #0FFh, W2  ; bytes

        ret

; ====================================================
;
; end of program
