; ----------------------------------------------------
; DAC4.asm    Last Update:10/27/91
;
; Simple Function Generator with a Digital-to-Analog
; Converter
;
; A counter is incremeted periodically.  The digital
; value is converted to analog, producing a saw tooth
; wave signal.  The rate at which the counter is
; incremented determines the frequency.  A simple
; delay loop which decrements r7 to zero is used to
; introduce a delay and thus determine the frequency.
; Try different values for the constant Dvalue to
; change the frequency.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD Connections:
;  Port 4 to the Digital-to-Analog Converter
;  Port 5 bits:
;   P5.0 to DAC WR1\  (DAC0832 pin  2)
;   P5.1 to DAC WR2\  (DAC0832 pin 18)
;
    DACdata equ 0E8h  ; P4
    WR1     equ 0F8h  ; P5.0
    WR2     equ 0F9h  ; P5.1
;
; constants:
   Dvalue  equ 0FFh    ; delay value
; ----------------------------------------------
; system calls              registers used
; ----------------------------------------------
 mdelay    equ  012dh  ; a
 sdelay    equ  0142h  ; a
; ----------------------------------------------
    org 8000h
    mov   r0, #0       ; waveform loop count index
begin:
    mov   a, r0        ; get loop count index
    inc   r0           ; increment index
    lcall WaveLU       ; get waveform amplitude
    mov   DACdata, a   ; send data to DAC
    lcall DAC          ; convert to analog

    mov   a, #Dvalue   ; waste a little time
    lcall mdelay       ; try sdelay for longer
                       ; delays

    sjmp  begin        ; repeat

; ====================================================
;  subroutine DAC - digital-to-analog conversion
;  input    : none
;  output   : uses WR1 and WR2 to execute a DAC.
;              DAC WR1\  (DAC0832 pin  2)
;              DAC WR2\  (DAC0832 pin 18)
;
;  destroys : Port bits connected to WR1 and WR2
; ----------------------------------------------------
DAC:
        setb  WR1   ; disable DAC
        setb  WR2   ; disable DAC
        clr   WR1   ; input latch
        setb  WR1   ;
        clr   WR2   ; output latch
        setb  WR2   ;
        ret

; ====================================================
; subroutine WaveLU
;  A look up table to generate analog waveforms.
;  One period of the waveform is divided into 256
;  steps.  The wave amplitude at each step is an entry
;  in the table.  The subroutine returns the n-th
;  entry, where n is the contents of the accumulator.
;
;  input    : step n (n=0,1,...,FF)
;  output   : Waveform amplitude at step n
;  destroys : a
; ----------------------------------------------------
WaveLU:
        cjne  a, #0FFh, AnotFF
        mov  a, #0h        ; amplitude at step FFh
        ret
AnotFF:
        inc  a
        movc a, @a+pc
        ret
        db  99h, 99h, 99h, 99h, 99h ;   0-  4
        db  99h, 99h, 99h, 99h, 99h ;   5-  9
        db  99h, 99h, 99h, 99h, 99h ;  10- 14
        db  99h, 99h, 99h, 99h, 99h ;  14- 19
        db  99h, 99h, 99h, 99h, 98h ;  20- 24
        db  97h, 95h, 94h, 93h, 92h ;  25- 29
        db  91h, 90h, 87h, 85h, 84h ;  30- 34
        db  82h, 80h, 79h, 78h, 77h ;  35- 39
        db  76h, 75h, 75h, 74h, 74h ;  40- 44
        db  73h, 72h, 71h, 70h, 69h ;  45- 49
        db  68h, 67h, 66h, 65h, 64h ;  50- 54
        db  63h, 62h, 61h, 60h, 60h ;  55- 59
        db  59h, 58h, 57h, 57h, 56h ;  60- 64
        db  55h, 55h, 54h, 53h, 53h ;  65- 69
        db  53h, 52h, 51h, 51h, 50h ;  70- 74
        db  50h, 49h, 48h, 48h, 47h ;  75- 79
        db  47h, 46h, 46h, 45h, 45h ;  80- 84
        db  44h, 44h, 43h, 43h, 42h ;  85- 89
        db  42h, 41h, 41h, 40h, 40h ;  90- 94
        db  39h, 39h, 38h, 38h, 37h ;  95- 99
        db  37h, 36h, 36h, 35h, 35h ; 100-104
        db  34h, 34h, 33h, 33h, 32h ; 105-109
        db  32h, 32h, 31h, 31h, 30h ; 110-114
        db  30h, 30h, 29h, 29h, 28h ; 115-119
        db  28h, 28h, 27h, 27h, 27h ; 120-124
        db  26h, 26h, 25h, 25h, 25h ; 125-129
        db  25h, 24h, 24h, 23h, 22h ; 130-134
        db  22h, 21h, 21h, 20h, 20h ; 135-139
        db  19h, 19h, 19h, 18h, 18h ; 140-144
        db  18h, 17h, 17h, 17h, 16h ; 145-149
        db  16h, 15h, 15h, 14h, 14h ; 150-154
        db  13h, 13h, 12h, 12h, 11h ; 155-159
        db  11h, 11h, 10h, 10h, 10h ; 160-164
        db  09h, 08h, 08h, 07h, 07h ; 165-169
        db  07h, 06h, 06h, 06h, 05h ; 170-174
        db  05h, 05h, 05h, 04h, 04h ; 175-179
        db  03h, 03h, 02h, 02h, 01h ; 180-184
        db  01h, 00h, 00h, 00h, 00h ; 185-189
        db  00h, 00h, 00h, 00h, 00h ; 190-194
        db  00h, 00h, 00h, 00h, 00h ; 195-199
        db  00h, 00h, 00h, 00h, 00h ; 200-204
        db  00h, 00h, 00h, 00h, 00h ; 205-209
        db  00h, 00h, 00h, 00h, 00h ; 210-214
        db  00h, 00h, 00h, 00h, 00h ; 215-219
        db  00h, 00h, 00h, 00h, 00h ; 220-224
        db  00h, 00h, 00h, 00h, 00h ; 225-229
        db  00h, 00h, 00h, 00h, 00h ; 230-234
        db  00h, 00h, 00h, 00h, 00h ; 235-239
        db  00h, 00h, 00h, 00h, 00h ; 240-244
        db  00h, 00h, 00h, 00h, 00h ; 245-249
        db  00h, 00h, 00h, 00h, 00h ; 250-254

; ====================================================
; end of program
