$nomod51
$symbols
$debug
$nolist
$include (\i\rg51fb.pdf)
$list
;
;This is a test of the PWM output of the 8051fx family of parts.
;	This particular file uses a half amplitude sine table.
;
;
;It will generate a sine wave of arbitrary frequency by DDS techniques.
;with a 16.000 Mhz clock, the sample frequency, Fs, will be 15.625 Khz.
; output frequency will be: Fo = Fs * ( Phase_inc / 65536 )
;
;  Phase_inc = (TableSize * 256 * Frequency)/Fs
;  phase_inc = 65536 / 15645 * Frequency  = 4.1943 * Freq.
; for standard DTMF tones the following table can be used:
;row	Phase_inc
;697 	2923 = 0b6bh	1 2 3  a
;770	3230 = 0c9eh	4 5 6  b
;852	3574 = 0df6h	7 8 9  c
;941	3947 = 0f6bh 	* 0 #  d
;col		     	| | |  |
;1209	5071 = 13cfh ---+ | |  |
;1336	5604 = 15e4h -----+ |  |
;1477	6195 = 1833h -------+  |
;1633   6849 = 1ac1h ----------+
;
; This technique allows synthesis of Sine waves from 7.8Khz to .24Hz with
; .24Hz resolution.
;
data_seg segment data
code_seg segment code
;
cseg at reset
	ljmp	start
;reserve interrupt vectors
;
cseg at exti0
	reti
;
cseg at timer0
	reti
;
cseg at exti1
	reti
;
cseg at timer1
;	ljmp	DDS_isr
	reti
;
cseg at sint
	reti
;
; the following interrupt vector is for the 8052
timer2	equ	002bh
; the following interrupt vector is for the 8051Fx
pca	equ	0033h
;
cseg at timer2
	reti
;
cseg at pca
	ljmp	DDS_isr
;
dseg	at 18h
;
Phase_acc:	ds	2	; big endian
Phase_inc:	ds	2	; msb,lsb
NewVal:	ds	1
Phase2acc:	ds	2
Phase2inc:	ds	2
;
;
cseg
;
DDS_isr:
	mov	ccap4h,newval	; put out new value, pronto

	clr	ccf3
	inc	ccap3h		; next interrupt occurs 256 cycles later
	clr	ea
	push	acc
	push	psw
	push	dph
	push	dpl
;				; Phase_acc := Phase_acc + Phase_inc
	mov	a,phase_inc+1	;  LSB
	add	a,Phase_acc+1
	mov	phase_acc+1,a

	mov	a,phase_inc	;  MSB
	addc	a,Phase_acc
	mov	phase_acc,a
;
	mov	dptr,#SinTab	; PWM = sine(hi(Phase_Acc))
	movc	a,@a+dptr
	mov	newval,a
;				; Phase2acc += Phase2inc
	mov	a,phase2inc+1	;  LSB
	add	a,Phase2acc+1
	mov	phase2acc+1,a

	mov	a,phase2inc	;  MSB
	addc	a,Phase2acc
	mov	phase2acc,a
;				; PWM = sine(hi(Phase2Acc))
	movc	a,@a+dptr
	add	a,newval	; add the two values
;	rrc	a		; and divide by 2
	mov	newval,a
;
	pop	dpl
	pop	dph
	pop	psw
	pop	acc
	setb	ea
DDS_END:
	reti
;
;*****************************
;
Start:	mov	ie,#0
	mov	sp,#20h

	; initialize PCA
	mov	cmod,#02h	;select clock = 1/4 Fosc
	mov	cl,#0
	mov	ch,#0

 	mov	ccapm4,#43h	; module 4 in PWM mode
	mov	ccap4l,#00h
	mov	ccap4h,#80h	; start with .50 duty cycle

	; Use PCA channel 3 as interrupt source.
	mov	ccapm3,#49h	; module 3= interrupt on compare
	mov	ccap3l,#0
	mov	ccap3h,#0
	setb	ec		; enable pca INT

	;initialize DDS variables
	orl	psw,#18h	; sel rb3
	mov	phase_inc,#0bh
	mov	phase_inc+1,#6bh
	mov	phase2inc,#13h
	mov	phase2inc+1,#0cfh
	anl	psw,# not 18h	; sel rb0

	setb	cr		; turn on PCA timer.
	setb	ea		; enable all interrupts

loop:	clr	a
	mov	dptr,#rowTab
	jb	p1.3,rowdown
	inc	a
	jb	p1.4,rowdown
	inc	a
	jb	p1.5,rowdown
	inc	a
	jb	p1.6,rowdown
	mov	phase_inc,#0
	mov	phase_inc+1,#0
testcol:
	clr	a
	mov	dptr,#ColTab
	jb	p3.3,ColDown
	inc	a
	jb	p3.1,ColDown
	inc	a
	jb	p3.0,ColDown
	mov	phase2inc,#0
	mov	phase2inc+1,#0

	jmp	loop		;  hang in there waiting for interrupts
;
rowdown:
	clr	c
	rlc	a
	mov	b,a
	movc	a,@a+dptr
	mov	phase_inc,a
	mov	a,b
	inc	a
	movc	a,@a+dptr
	mov	phase_inc+1,a
	jmp	testcol
	nop
;
ColDown:
	clr	c
	rlc	a
	mov	b,a
	movc	a,@a+dptr
	mov	phase2inc,a
	mov	a,b
	inc	a
	movc	a,@a+dptr
	mov	phase2inc+1,a
	jmp	loop
;
RowTab:
	dw	0b6bh ; 1 2 3  a  697 Hz
	dw	0c9eh ; 4 5 6  b  770
	dw	0df6h ;	7 8 9  c  852
	dw	0f6bh ;	* 0 #  d  941
ColTab:	   	      ;	| | |  |
	dw	13cfh ;-+ | |  |  1209Hz
	dw	15e4h ;---+ |  |  1336
	dw	1833h ;-----+  |  1477
	dw	1ac1h ;--------+  1633
;
sintab:

    DB  040H,042H,043H,045H,046H,048H,049H,04BH
    DB  04CH,04EH,04FH,051H,052H,054H,055H,057H
    DB  058H,05AH,05BH,05CH,05EH,05FH,060H,062H
    DB  063H,064H,066H,067H,068H,069H,06AH,06BH
    DB  06DH,06EH,06FH,070H,071H,072H,073H,074H
    DB  074H,075H,076H,077H,078H,078H,079H,07AH
    DB  07AH,07BH,07BH,07CH,07CH,07DH,07DH,07DH
    DB  07EH,07EH,07EH,07FH,07FH,07FH,07FH,07FH
    DB  07FH,07FH,07FH,07FH,07FH,07FH,07EH,07EH
    DB  07EH,07DH,07DH,07DH,07CH,07CH,07BH,07BH
    DB  07AH,07AH,079H,078H,078H,077H,076H,075H
    DB  074H,074H,073H,072H,071H,070H,06FH,06EH
    DB  06DH,06BH,06AH,069H,068H,067H,066H,064H
    DB  063H,062H,060H,05FH,05EH,05CH,05BH,05AH
    DB  058H,057H,055H,054H,052H,051H,04FH,04EH
    DB  04CH,04BH,049H,048H,046H,045H,043H,042H
    DB  040H,03EH,03DH,03BH,03AH,038H,037H,035H
    DB  034H,032H,031H,02FH,02EH,02CH,02BH,029H
    DB  028H,026H,025H,024H,022H,021H,020H,01EH
    DB  01DH,01CH,01AH,019H,018H,017H,016H,015H
    DB  013H,012H,011H,010H,00FH,00EH,00DH,00CH
    DB  00CH,00BH,00AH,009H,008H,008H,007H,006H
    DB  006H,005H,005H,004H,004H,003H,003H,003H
    DB  002H,002H,002H,001H,001H,001H,001H,001H
    DB  001H,001H,001H,001H,001H,001H,002H,002H
    DB  002H,003H,003H,003H,004H,004H,005H,005H
    DB  006H,006H,007H,008H,008H,009H,00AH,00BH
    DB  00CH,00CH,00DH,00EH,00FH,010H,011H,012H
    DB  013H,015H,016H,017H,018H,019H,01AH,01CH
    DB  01DH,01EH,020H,021H,022H,024H,025H,026H
    DB  028H,029H,02BH,02CH,02EH,02FH,031H,032H
    DB  034H,035H,037H,038H,03AH,03BH,03DH,03EH


	end
