dumpreq	equ	11h
sendreq	equ	12h
playreq	equ	13h
dumpall	equ	14h
looper	equ	15h
reset	equ	16h

; The status command "F0" has been detected

system_exclusive:
	call	inputt			; get manufacture ID
	cjne	a,#0,nouse
	call	inputt			; get device ID
	cjne	a,#0,nouse
	call	inputt			; get model ID
	cjne	a,#0,nouse
	call	inputt			; get command ID
;
; DUMP request #11h
	cjne	a,#dumpreq,chk12	; Dump request
	clr	runlite
	call	inputt			; get bank #
	mov	bank,a
	call	sysex_write
	setb	runlite
	sjmp	nouse
;
; SEND request #12
chk12:	cjne	a,#sendreq,chk13	; Send request
	clr	runlite
	call	inputt			; get bank #
	mov	bank,a
	mov	dpl,nxtmem_lo
	mov	dph,nxtmem_hi		; get load address
	mov	nbytes,#0		; no bytes loaded yet
	call	sysex_read		; now load the code
	call	insert_bank		; update bank
	setb	runlite
	sjmp	nouse
;
; RUN request #13h
chk13:	cjne	a,#playreq,chk14	; Run Request
	acall	gettwo			; hi byte load address
	mov	dph,a
	acall	gettwo			; lo byte load address
	mov	dpl,a
	acall	gettwo			; command byte
	clr	a
	jmp	@a+dptr			; goto it
;
; Dump all request #14h
chk14:	cjne	a,#dumpall,chk15
	acall	gettwo			; hi byte load address
	mov	dph,a
	acall	gettwo			; lo byte load address
	mov	dpl,a
	call	dump_all
	sjmp	nouse
;
; loop through request #15h
chk15:	cjne	a,#looper,chk16
	call	gettwo			; get on=1 or off=0
	jz	loopoff
	setb	loopthru
	sjmp	nouse
loopoff:
	clr	loopthru
	sjmp	nouse
;
; RESET request #16h
chk16:	cjne	a,#reset,nouse
	ljmp	0
; No use for this command
nouse:	call	inputt
	cjne	a,#0f7h,nouse		; ignore rest of sysex
	ret

sysex_read:
	acall	gettwo			; get a byte
	cjne	A,#':',sysex_read	; wait for a colon
	mov	checksum,#0		; zero checksum
	acall	bhex			; get # of bytes to load
	jz	endfl			; none to load
	mov	R2,A			; save byte count
	acall	tapehl			; get load address & command
rloop:	acall	bhex			; get a byte of data
	movx	@dptr,A			; store data
	inc	nbytes			; count bytes loaded
	mov	00h,A			; save data
	movx	A,@DPTR			; get data just placed
	cjne	A,00h,error		; error if not equal
	inc	DPTR			; next memory location
	djnz	R2,rloop		; do byte count
	acall	check			; check
	sjmp	sysex_read		; next line of data
;
endfl:	acall	tapehl			; get start address
	acall	bhex			; get last checksum byte
error:	ret
;
; normal loader would use this address.
tapehl:	acall	bhex			; hi byte load address ignored
	acall	bhex			; lo byte load address ignored
	acall	bhex			; command byte
	ret
;
check:	acall	bhex			; get last byte which is checksum
	mov	a,checksum
	jz	ckok
	pop	acc
	pop	acc
ckok:	ret				; ok
;	
;==========================================================================
;
nib:	clr	c			; clear carry
	subb	A,#'0'			; subtract bias
	jnb	ACC.4,nib3		; if < 10
	subb	A,#7			; make A-F
nib3:	anl	a,#0fh			; to be sure
	ret
; all that changed from a normal papertape loader is this call.
gettwo:	acall	inputt
	acall	nib
	swap	A
	mov	r4,A			; save
	acall	inputt
	acall	nib
	orl	A,r4			; combine
	ret
	
; get a hex value to A	
hex:	acall	gettwo			; normally called inputt
	acall	nib
	ret
;
bhex:	acall	hex			; get high nibble
	swap	A			; put to high nibble
	mov	R3,A			; save
	acall	hex			; get low nibble
	orl	A,R3			; combine high and low nibble
	mov	R3,A			; save
	add	A,checksum		; add to checksum
	mov	checksum,A		; save checksum
	mov	A,R3			; restore byte
	ret
;
sysex_write:
	mov	a,#0f0h		; sysex status
	call	outt
	mov	a,#0		; manufacture
	call	outt
	mov	a,#0		; device
	call	outt
	mov	a,#0		; model
	call	outt
	mov	a,#dumpreq
	call	outt
	mov	a,bank
	call	outt
	call	get_instrument
	movx	a,@r0		; get high address
	call	prthex
	inc	r0
	movx	a,@r0		; get low address
	call	prthex
	inc	r0
	movx	a,@r0		; get byte count
	call	prthex
	mov	a,#0f7h		; eox
	call	OUTT
	ret
;
PRTHEX:	push	2
	acall	BINHEX
	acall	OUTT
	mov	a,r2
	acall	OUTT
	pop	2
	RET
;
prtaddr:
	mov	a,dph
	call	prthex
	mov	a,dpl
	call	prthex
	ret
;
BINHEX:	mov	r2,a
	anl	a,#0fh
	add	a,#0f6h
	jnc	X029a
	add	a,#7
X029a:	add	a,#3ah
	xch	a,r2
	swap	a
	anl	a,#0fh
	add	a,#0f6h
	jnc	X02a6
	add	a,#7
X02a6:	add	a,#3ah
	ret
;
insert_bank:
	call	get_instrument		; address in P2:R0
	mov	a,nxtmem_hi
	movx	@r0,a
	inc	r0			; point to low address
	mov	a,nxtmem_lo
	movx	@r0,a
	mov	nxtmem_lo,dpl		; new next memory location
	mov	nxtmem_hi,dph
	inc	r0			; point to byte count
	mov	a,nbytes
	movx	@r0,a
	ret

get_instrument:
	mov	p2,#high instab
	mov	a,bank			; get 0 - 7
	anl	a,#07h			; enforce it
	clr	c
	rlc	a			; * 2
	rlc	a			; * 4
	inc	a			; skip ljmp instruction
	mov	r0,a
	ret
;
; with just a litte more effort this could be a loadable hex file
; DPTR has dump address

dump_all:
	mov	r1,#16
onepag:	mov	a,#0f0h		; sysex status
	acall	outt
	mov	a,#0		; manufacture
	acall	outt
	mov	a,#0		; device
	acall	outt
	mov	a,#0		; model
	acall	outt
	mov	a,#dumpreq
	acall	outt
	mov	r6,#0		; checksum = 0
	mov	r0,#16
	mov	a,#':'
	acall	outt
	mov	a,r0		; data length
	acall	prthex	
	acall	prtaddr
	clr	a
	acall	prthex
onelin:	movx	a,@dptr
	xch	a,r6		; a=checksum, r6=data
	add	a,r6		; add to checksum
	xch	a,r6		; r6=checksum, a=data
	acall	prthex
	inc	dptr
	djnz	r0,onelin
	mov	a,r6
	cpl	a
	inc	a		; -checksum
	acall	prthex		; output checksum
	mov	a,#0f7h		; eox
	acall	OUTT
	djnz	r1,onepag
	ret

