; This routine is called from RTC every time the
; slice time (SLIT) has been decremented to 0
; from the initial count of (QUANTIM)
; stack depth from rtcisr contains
; IPL_SYNCH,call to here
max_stack	equ	48	; stack is from 50h through 7fh
;==========================================================================
; 1) check execution list
;    if there are no jobs on the execution list
;    no need to swap the monitor off/on
; 2) save process context
; 3) get a free node
;    if none available, just continue with 
;    current process until one becomes free
; 4) initialize node with sp, job #, reg bank, ticks
; 5) insert job on execution list
; 6) schedule another task

reschedule:
	clr	ea		; don't bother me
	mov	dptr,#EXEC	; this sets dph to kernal page
	movc	a,@a+dptr	; get node from execution list
	jnz	rescd1		; there are jobs ready to run
	set_error	NEXERES		; set no job on exec list error
	ret			; no jobs, exit to isr
;
rescd1:	mov	a,#CURJOB
	movc	a,@a+dptr	; get current job number
	acall	svpctx		; save process context from job in A
	mov	dpl,#FREE	; point to free list
	acall	remque		; get a node from the free list
	jnz	rescd2		; ok
	set_error	NNODEXE		; set no free nodes error
	ret			; no free nodes, exit to isr
;
rescd2:	mov	Node,a		; B = node
	mov	a,#0ffh
	mov	Rtich,a		; R3:R1 = ticks
	mov	Rticl,a		; ticks (only for completness)
	mov	a,#CURJOB
	movc	a,@a+dptr
	mov	r4,a		; R4 = Job #
	mov	a,Node		; A = node
	mov	r2,sp		; R2 = stack pointer
	acall	initialize_node	; 
	mov	dpl,#EXEC
	mov	a,Node		; A = node
	acall	insque		; insert at tail of exec queue

$eject
;=============================================================================
; we have saved all evidence of current process...
; we now are in "kernal" mode - we can't get any lower than this
; there is ALWAYS a job to do because the monitor would have been
; queued if another job was running, otherwise TILT
; get a node from the EXEC list and put it back on the FREE list
; all registers are available for use
scheduler:
	mov	dptr,#EXEC	; this sets dph to kernal page
	acall	remque		; (getnode)
	jnz	xq2		; we got a computable job node in A !
bugcheck:
	set_error	nexesch		; set no jobs error
	call	querr
	db	'BugCheck',0
bchk2:	sjmp	bchk2		; tilt

xq2:	mov	Node,a		; save node in B
	mov	dpl,#FREE	; free list
	acall	insque		; add current to end of free list

	mov	a,Node		; node offset

; set new Stack Pointer
	add	a,#QSP		; + stack pointer offset
	mov	dpl,a
	movx	a,@dptr		; new SP
	mov	sp,a		; set new stack

; get new job number
	inc	dptr		; point to QJOBN
	movx	a,@dptr
	mov	b,a		; Store job #
	jz	ismon
	setb	p3.4
	sjmp	togle
ismon:	clr	p3.5
togle:

; new job becomes current job
	mov	a,b		; a = JOB #
	mov	dpl,#curjob
	movx	@dptr,a		; set new job number

; give new job a full quantim
	mov	dpl,#slit
	mov	a,#quantim
	movx	@dptr,a
	
	mov	a,b		; A = job #
	jmp	ldpctx		; load process context from job in A
;
; when a job completes, it comes here to go on to next job.
; make job number available, prepare next job and go to it.
; the information in the array is history. A new job will
; overwrite the array.
post_process:
	clr	ea		; can't have rtc try a resched
	mov	dptr,#CURJOB	; this sets dph to kernal page
	movx	a,@dptr
	inc	a		; job 0 is bit 0, job 1 is bit 1...
	mov	b,a
	setb	c
	clr	a
fbp:	rlc	a
	djnz	b,fbp
	mov	b,a		; save bit position
	inc	dptr		; point to AJOBN
	movx	a,@dptr
	orl	a,b		; set job number bit
	movx	@dptr,a
	ajmp	scheduler	; prepare next job
;
$eject

