; ----------------------------------------------------
; EXP-IO1.ASM          Last Update:10/27/91
; Simple Boolean Expressions
;
;  This program extends EXP-IO1.ASM.  It implements
;  the same simple combinational logic  system with
;  three inputs (X0, X1, and X2) and two outputs
;  (Y0 and Y1).  The system is described by two Boolean
;  equations, one for each output:
;
;  Y0 = (X0 AND X1) OR X2
;  Y1 = ((X0 OR X1) AND (X0 OR X2)
;
;  The inputs are connected to the push buttons and
;  the outputs to the LEDs.
;
;  This version of EXP-IO demonstrates masking and
;  isolating individual bits, and some useful system
;  calls to report the states to the host and receive
;  an abort (Control C) signal from the host.  It
;  also demonstrates the use of stack for temporary
;  storage.
;
; ====================================================
; UIOD connections
;
;  PB0 (push button 0) to P1.0
;  PB1 (push button 1) to P1.1
;  PB2 (push button 2) to P1.2
;  LED0 to P1.3
;  LED1 to P1.4
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 crlf      equ  0115h  ; a
 inkey     equ  012ah  ; a
 os_return equ  0130h
 print     equ  0136h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------
; --- port definitions -------------------------------
; input ports
X0   equ   90h ; P1.0
X1   equ   91h ; P1.1
X2   equ   92h ; P1.2
; output ports
Y0   equ   93h ; P1.3
Y1   equ   94h ; P1.4
; --- internal register definitions ------------------
; bits
temp equ   40h ; used to store intermediate results
; ----------------------------------------------------
; ----------------------------------------------------
;Y0 = (X0 AND X1) OR X2
;Y1 = ((X0 OR X1) AND (X0 OR X2)
;
; --- origin at the beginning of RAM -----------------
   org   8000h
; --- program starts here ----------------------------
   mov   P1, 0FFh ; prepare bits 0 to 2 as input
; --- now compute Y0 ---------------------------------
   clr   a     ; save false states on stack to force a
   push acc    ; show of the states
loop:
   mov   C, X0 ; place X0 in carry flag
   anl   C, X1 ; and X0 and X1, result in carry flag
   orl   C, X2 ; OR (X0 AND X1) with X2
; --- the result in the carry flag is Y0 -------------
   mov   Y0, C
; --- now compute Y1 ---------------------------------
next:
   mov C, X0   ; place X0 in carry flag
   orl C, X1   ; and X0 and X1, result in carry flag
   mov temp, C ; save intermediate result
   mov C, X0   ; similarly compute (X0 OR X2)...
   orl C, X2
   anl C, temp ; and AND it with temp
; --- the result in the carry flag is Y1 -------------
   mov Y1, C
; --- see if any of the inputs or outputs have changed
   pop   acc    ; recall old states from stack
   mov   b, P1  ; read input/output states
   push  b      ; save the new input/output states
   xrl   a, b   ; have ports changed ?
   jnz   show   ; if so show the new states
; --- if no changes, check for an abort from the host
   lcall inkey         ; see if a CONTROL-C is received
   cjne  a, #3, loop   ; if not repeat the process
                       ; if so jump back to the monitor
   ljmp  os_return     ; by a long jump to label os_return

; --- come here if a change in states is detected ----
show:
   lcall print  ; otherwise print message
   db    "inputs X0:", 0
   mov   a, P1  ; read inputs
   anl   a, #1  ; mask input X0
   add   a, #'0'; add ASCII '0'
   lcall sndchr ; display bit

   lcall print  ; repeat for inputs X1
   db    "  X1:", 0
   mov   a, P1  ; read inputs
   anl   a, #2  ; mask input X1
   rr    a      ; rotate right
   add   a, #'0'; add ASCII '0'
   lcall sndchr ; display bit

   lcall print  ; repeat for input X2
   db    "  X2:", 0
   mov   a, P1  ; read inputs
   anl   a, #4  ; mask input X2
   rr    a      ; rotate right...
   rr    a      ; ... twice
   add   a, #'0'; add ASCII '0'
   lcall sndchr ; display bit

   lcall print  ; repeat for output Y0
   db    "  outputs  Y0:", 0
   mov   a, P1  ; read inputs
   anl   a, #8  ; mask output Y0
   rr    a      ; rotate right...
   rr    a      ;
   rr    a      ; ... three times
   add   a, #'0'; add ASCII '0'
   lcall sndchr ; display bit

   lcall print  ; repeat for output Y1
   db    "  Y1:", 0
   mov   a, P1  ; read inputs
   anl   a, #10h; mask output Y1
   rr    a      ; rotate right...
   rr    a      ;
   rr    a      ;
   rr    a      ; ... four times
   add   a, #'0'; add ASCII '0'
   lcall sndchr ; display bit

   lcall crlf   ; insert blank line

; --- this ends the show routine ---------------------
   ljmp loop    ; repeat ... note the long jump
