; ----------------------------------------------------
; EXP-IO2.ASM          Last Update:10/27/91
; Simple Boolean Operatins
;
;  This program implements a simple sequential logic
;  system with two inputs (X0 and X1) and two outputs
;  (Y0 and Y1).  The system is described by two Boolean
;  equations, one for each output:
;
;  Y0 = (X0 AND Y1)'
;  Y1 = (X1 AND Y0)'
;
;  The inputs are connected to the push buttons and
;  the outputs to the LEDs.
;
;  This version of EXP-IO demonstrates masking and
;  isolating individual bits, and some useful system
;  calls to report the states to the host and receive
;  an abort (Control C) signal from the host.  It
;  also demonstrates the use of stack for temporary
;  storage.
;
; ====================================================
; UIOD connections
;
;  PB0 (push button 0) to P1.0
;  PB1 (push button 1) to P1.1
;  LED0 to P1.3
;  LED1 to P1.4
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 crlf      equ  0115h  ; a
 inkey     equ  012ah  ; a
 os_return equ  0130h
 print     equ  0136h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------
; --- port definitions -------------------------------
; input ports
X0   equ   90h ; P1.0
X1   equ   91h ; P1.1
; output ports
Y0   equ   93h ; P1.3
Y1   equ   94h ; P1.4
; --- internal register definitions ------------------
; none
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
   org 8000h
; --- program starts here ----------------------------
   mov   P1, 0FFh ; prepare bits 0 to 2 as input
   clr   a        ; save false states to force a show
   push  acc
loop:
; --- now compute Y0 ---------------------------------
   mov C, X0 ; place X0 in carry flag
   anl C, Y1 ; and X0 and Y1, result in carry flag
   cpl C     ; complement the result
; --- the result in the carry flag is Y0 -------------
   mov Y0, C
; --- now compute Y1 ---------------------------------
   mov C, X1   ; place X0 in carry flag
   anl C, Y0   ; and X1 and Y0, result in carry flag
   cpl C       ; complement the result
; --- the result in the carry flag is Y1 -------------
   mov Y1, C
; --- this ends the logic computations ---------------
   mov   b, P1 ; input output states
   pop   acc   ; recall previous input output states
   push  b     ; save new states on stack
   xrl   a, b  ; compare old and new states
   jnz   show  ; show states if they have changed
; --- else check for a Control-C ---------------------
   lcall inkey
   cjne  a, #3, loop ; if no CRL-C the repeat loop
   ljmp  os_return   ; if CTL-C, return to OS

; --- come here if the states have changed -----------
show:
   lcall showIO
   sjmp  loop  ; repeat forever...

; ::::::::::::::::::::::::::::::::::::::::::::::::::::
; --- subroutines ------------------------------------
; ====================================================
;  subroutine showIO
;  input    : none
;  output   : sends a message to the host displaying
;             states of X0, X1, Y0, and Y1.
;  destroys : psw, a, dptr
; ----------------------------------------------------
showIO:
   lcall print  ;  print message
   db    "inputs X0:", 0
   mov   a, P1  ; read inputs
   anl   a, #1  ; mask input X0
   add   a, #'0'; add ASCII '0'
   lcall sndchr ; display bit

   lcall print  ; repeat for inputs X1
   db    "  X1:", 0
   mov   a, P1  ; read inputs
   anl   a, #2  ; mask input X1
   rr    a      ; rotate right
   add   a, #'0'; add ASCII '0'
   lcall sndchr ; display bit

   lcall print  ; repeat for output Y0
   db    "  outputs  Y0:", 0
   mov   a, P1  ; read inputs
   anl   a, #8  ; mask output Y0
   rr    a      ; rotate right...
   rr    a      ;
   rr    a      ; ... three times
   add   a, #'0'; add ASCII '0'
   lcall sndchr ; display bit

   lcall print  ; repeat for output Y1
   db    "  Y1:", 0
   mov   a, P1  ; read inputs
   anl   a, #10h; mask output Y1
   rr    a      ; rotate right...
   rr    a      ;
   rr    a      ;
   rr    a      ; ... four times
   add   a, #'0'; add ASCII '0'
   lcall sndchr ; display bit

   lcall crlf   ; insert blank line
   ret          ; return to calling program
; --- end of subroutine showIO -----------------------

; ====================================================
; end of program
