; ----------------------------------------------------
; EXP-STP1.ASM          Last Update:10/27/91
; Stepper Motor Control Experiment 1
;
;  This program illustrates the stepper motor driver
;  explained in the experiment guide.  It uses the
;  lower nibble of Port 1 to drive the switching
;  transistors.  The program implements the half-step
;  mode.  The stepping sequence is defined by 8
;  symbols st1 through st8.
;
; ====================================================
; UIOD connections
;
;  Connect P1.0 to P1.3 to the switching transistors
;  as given in the accompanying circuit diagram.
;    P1.0  to base of Q1 via a 1K resistor
;    P1.1  to base of Q2 via a 1K resistor
;    P1.2  to base of Q3 via a 1K resistor
;    P1.3  to base of Q4 via a 1K resistor
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; --- stepping sequence ------------------------------
 st1   equ   0FAh   ; keep the high nibble all ones
 st2   equ   0F8h
 st3   equ   0F9h
 st4   equ   0F1h
 st5   equ   0F5h
 st6   equ   0F4h
 st7   equ   0F6h
 st8   equ   0F2h

 COUNT equ    20h  ; sets the delay between steps
                   ; try other values to change speed
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
   org   8000h
; --- program starts here ----------------------------
loop:
   mov   b, #COUNT ; reload value
   mov   a, #st1   ; first step
   lcall wait      ; waste a little time
   mov   a, #st2   ; repeat remaining steps
   lcall wait
   mov   a, #st3
   lcall wait
   mov   a, #st4
   lcall wait
   mov   a, #st5
   lcall wait
   mov   a, #st6
   lcall wait
   mov   a, #st7
   lcall wait
   mov   a, #st8
   lcall wait
; --- this ends the stepping sequence ----------------
   sjmp  loop     ; repeat forever...
; ::::::::::::::::::::::::::::::::::::::::::::::::::::
; --- subroutines ------------------------------------
; ====================================================
;  subroutine wait
;  executes a nested pair of dummy loops to wait for
;  the stepper motor to move to the new position.
;  The delay time may be changed by modifying the reload
;  value in the b register.  The reload value is the
;  number of times the inner loop is executed.  Reducing
;  the reload value thus shortens the delay time.
;
;  Note that the inner loop continuously refreshes the
;  stepping pattern.  This not only extends the inner
;  loop execution time but is also a good precaution
;  when driving inductive loads.
;
;  input    : stepping sequence pattern in acc
;             reload value in b register
;  output   : sends the stepattern to port P1
;  destroys : r0, r1
; ----------------------------------------------------
wait:
   mov   r1, b
   mov   r0, #0
loop1:
   mov   P1, a
   djnz  r0, loop1
   djnz  r1, loop1
   ret
; --- end of subroutine wait -------------------------
; ====================================================
;
; end of program
