; ----------------------------------------------------
; EXP-STP2.ASM          Last Update:10/27/91
; Stepper Motor Control Experiment 7
;
;  This program illustrates the stepper motor driver
;  explained in the experiment guide.  It uses the
;  lower nibble of Port 1 to drive the switching
;  transistors.  The program implements the half-step
;  mode.
;
;  This version differs from EXP-STP1.ASM in that the
;  stepping sequence is provided by the subroutine
;  GETSTEP.
;
; ====================================================
; UIOD connections
;
;  Connect P1.0 to P1.3 to the switching transistors
;  as given in the accompanying circuit diagram.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; --- internal register use --------------------------
  STEP    equ   07fh   ; store current step
; constants
  COUNT   equ 20h      ; sets delay time - try other
                       ; values for different speeds
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
   org   8000h
; --- program starts here ----------------------------
loop:
   mov   b, #COUNT  ; reload value
   lcall getpattern ; get next step pattern
   lcall wait       ; waste a little time
   sjmp  loop       ; repeat forever...

; ::::::::::::::::::::::::::::::::::::::::::::::::::::
; --- subroutines ------------------------------------
; ====================================================
;  subroutine wait
;  executes a nested pair of dummy loops to wait for
;  the stepper motor to move to the new position.
;  The delay time may be changed by modifying the reload
;  value in the b register.  The reload value is the
;  number of times the inner loop is executed.  Reducing
;  the reload value thus shortens the delay time.
;
;  Note that the inner loop continuously refreshes the
;  stepping pattern.  This not only extends the inner
;  loop execution time but is also a good precaution
;  when driving inductive loads.
;
;  input    : stepping sequence pattern in acc
;             reload value in b register
;  output   : sends the stepattern to port P1
;  destroys : r0, r1
; ----------------------------------------------------
wait:
   mov   r1, b
   mov   r0, #0
loop1:
   mov   P1, a
   djnz  r0, loop1
   djnz  r1, loop1
   ret
; --- end of subroutine wait -------------------------

; ====================================================
;  subroutine getpattern
;  This routine keeps track of the current step [0..7]
;  in internal register STEP.
;
;  Each time getpattern is called it returns the next
;  step pattern in the accumulator.
;
;  input    : none
;  output   : the next step pattern in accumulator
;  destroys : psw, a, b, r0, r1
; ----------------------------------------------------
getpattern:
         mov a, STEP  ; get current step
         inc a        ; increment step
         anl a, #7    ; modulus 8
         mov STEP, a  ; store new step

         inc  a       ; to account for the ret instruction
         movc a, @a+pc
         ret
;--- table of bit patterns for steps ---
         db 0Ah, 08h, 09h, 01h, 05h, 04h, 06h, 02h

; ====================================================
;
; end of program
