; ----------------------------------------------------
; EXP-TMP1.ASM          Last Update:10/27/91
;
; Temperature Measurement
;
;  Shows the temperature in Celsius on the 2
;  digital displays.
;
; ====================================================
; UIOD connections
;  LOW DIGIT to Port 4 and HIGH DIGIT to Port 5.
;  Analog input channel 0 (AN0) to the thermistor-
;  resistor network.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; internal memory use
; bytes:
   analog0 equ 07Fh    ; 0-th adc input
;
;  constants
   WAIT equ    100     ; delay duration
; ----------------------------------------------------
; Using a 10K series resistor, the voltage seen at
; the microcontroller pin is 5*RT/(10K + RT) where
; RT is the resistance of the thermistor.
; The analog value [0..FF] is proportional to the
; input voltage.  The subroutine XLATE translates
; the input voltage to temperature in Celsius by a
; table look up procedure.
; The temperature is displayed on the two seven-
; segment displays.
;
; jumpers jp2 and jp3 must be installed.
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 os_return equ  0130h
 setintvec equ  0145h  ; a, dptr
; ----------------------------------------------------

        org 8000h

        lcall initadc
        setb  eal
begin:
        mov   a, analog0 ; read most recent analog value
        lcall Xlate      ; translate count to temperature
        lcall shwbcd     ; display it
        mov   a, #WAIT
        lcall delay      ; show for a while

        lcall inkey      ; see if CONTROL-C pending
        cjne  a, #3h, begin ; if not repeat
        clr   eal        ; if so disable all interrupts
        ljmp  os_return  ; back to the monitor

; ====================================================
;  subroutine initadc - initialize adc
;  input    : none
;  output   : initializes adc interrupt
;             to continuously read analog 0
;  destroys : a, dptr
; ----------------------------------------------------
initadc:
        clr   eadc        ; disable adc interrupts
        clr   iadc        ; reset adc interrupt flag
        mov   a, #6       ; sixth interrupt source
        mov   dptr, #adcisr ; setup interrupt vector
        lcall setintvec
        setb  eadc        ; adc interrupt mask bit

        mov   a, adcon    ; read adc control register
        anl   a, #0d0h    ; select channel 0, single shot
        mov   adcon, a
        mov   dapr, #0    ; start conversion
        ret

; ====================================================
;  subroutine adcisr
;  input    : adc interrupt
;  output   : continuously obtains and stores analog 0
;             value in internal ram.
;  destroys : nothing - uses a, r0, and r1
; ----------------------------------------------------
adcisr:
        push  acc

        mov   a,   addat     ; read analog value
        mov   analog0, a     ; store analog value
        mov   a, adcon       ; adc control register
        anl   a, #0d0h
        mov   adcon, a       ; next select channel 0
        clr   iadc           ; reset adc interrupt flag
        mov   dapr, #0       ; start next conversion
done:
        pop   acc
        reti

; ====================================================
; subroutine shwbcd - shows a bcd number on the
;                        2 digital displays
;  input    : bcd number in the accumulator
;  output   : ports 4 and 5, connected to low digit
;             and high digit respectively.
;  destroys : a
; ----------------------------------------------------
shwbcd:
        push  acc     ; temporarily store byte
        anl   a, #0fh ; isolate low digit
        lcall display ; get 7-segment pattern
        mov   p4, a   ; send to LOW DIGIT
        pop   acc     ; recall byte
        swap  a       ; swap digits
        anl   a, #0fh ; isolate low nibble
        lcall display ; get 7-segment pattern
        mov   p5, a   ; display on HIGH DIGIT
        ret

; ====================================================
; subroutine Xlate
;  Converts the analog-to-digital output (count)
;  to the temperature.  The analog count [0..FF] is
;  given in the accumulator.  A table look up is used
;  to find the temperature.  The temperature is
;  returned in the accumulator in Celsius in the
;  binary-coded-decimal (BCD) format.
;
;  input    : analog count the accumulator
;  output   : BCD temperature in accumulator
;  destroys : a
; ----------------------------------------------------
Xlate:
        inc  a
        movc a, @a+pc
        ret
        db  99h, 99h, 99h, 99h, 99h ;   0-  4
        db  99h, 99h, 99h, 99h, 99h ;   5-  9
        db  99h, 99h, 99h, 99h, 99h ;  10- 14
        db  99h, 99h, 99h, 99h, 99h ;  14- 19
        db  99h, 99h, 99h, 99h, 98h ;  20- 24
        db  97h, 95h, 94h, 93h, 92h ;  25- 29
        db  91h, 90h, 87h, 85h, 84h ;  30- 34
        db  82h, 80h, 79h, 78h, 77h ;  35- 39
        db  76h, 75h, 75h, 74h, 74h ;  40- 44
        db  73h, 72h, 71h, 70h, 69h ;  45- 49
        db  68h, 67h, 66h, 65h, 64h ;  50- 54
        db  63h, 62h, 61h, 60h, 60h ;  55- 59
        db  59h, 58h, 57h, 57h, 56h ;  60- 64
        db  55h, 55h, 54h, 53h, 53h ;  65- 69
        db  53h, 52h, 51h, 51h, 50h ;  70- 74
        db  50h, 49h, 48h, 48h, 47h ;  75- 79
        db  47h, 46h, 46h, 45h, 45h ;  80- 84
        db  44h, 44h, 43h, 43h, 42h ;  85- 89
        db  42h, 41h, 41h, 40h, 40h ;  90- 94
        db  39h, 39h, 38h, 38h, 37h ;  95- 99
        db  37h, 36h, 36h, 35h, 35h ; 100-104
        db  34h, 34h, 33h, 33h, 32h ; 105-109
        db  32h, 32h, 31h, 31h, 30h ; 110-114
        db  30h, 30h, 29h, 29h, 28h ; 115-119
        db  28h, 28h, 27h, 27h, 27h ; 120-124
        db  26h, 26h, 25h, 25h, 25h ; 125-129
        db  25h, 24h, 24h, 23h, 22h ; 130-134
        db  22h, 21h, 21h, 20h, 20h ; 135-139
        db  19h, 19h, 19h, 18h, 18h ; 140-144
        db  18h, 17h, 17h, 17h, 16h ; 145-149
        db  16h, 15h, 15h, 14h, 14h ; 150-154
        db  13h, 13h, 12h, 12h, 11h ; 155-159
        db  11h, 11h, 10h, 10h, 10h ; 160-164
        db  09h, 08h, 08h, 07h, 07h ; 165-169
        db  07h, 06h, 06h, 06h, 05h ; 170-174
        db  05h, 05h, 05h, 04h, 04h ; 175-179
        db  03h, 03h, 02h, 02h, 01h ; 180-184
        db  01h, 00h, 00h, 00h, 00h ; 185-189
        db  00h, 00h, 00h, 00h, 00h ; 190-194
        db  00h, 00h, 00h, 00h, 00h ; 195-199
        db  00h, 00h, 00h, 00h, 00h ; 200-204
        db  00h, 00h, 00h, 00h, 00h ; 205-209
        db  00h, 00h, 00h, 00h, 00h ; 210-214
        db  00h, 00h, 00h, 00h, 00h ; 215-219
        db  00h, 00h, 00h, 00h, 00h ; 220-224
        db  00h, 00h, 00h, 00h, 00h ; 225-229
        db  00h, 00h, 00h, 00h, 00h ; 230-234
        db  00h, 00h, 00h, 00h, 00h ; 235-239
        db  00h, 00h, 00h, 00h, 00h ; 240-244
        db  00h, 00h, 00h, 00h, 00h ; 245-249
        db  00h, 00h, 00h, 00h, 00h ; 250-254
; ====================================================
; end of program
