; ----------------------------------------------------
; EXP-TMP2.ASM          Last Update:10/27/91
;
; Temperature Measurement
;
;  Shows the temperature in Celsius on the 2
;  digital displays.
;
; Use the constant current source given in Figure 2.
;
; The analog value from 0 (0 volts) to FF (5 volts)
; is displayed on the two seven-segment displays.
; This value needs to be converted to its Celsius
; equivalent.
; ====================================================
; UIOD connections
;  LOW DIGIT to Port 4 and HIGH DIGIT to Port 5.
;  Analog input channel 0 (AN0) to the thermistor-
;  resistor network.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; internal memory use
; bytes:
   analog0 equ 07Fh    ; 0-th adc input
;
;  constants
   WAIT equ    100     ; delay duration
; ----------------------------------------------------
;
; jumpers jp2 and jp3 must be installed.
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 os_return equ  0130h
 setintvec equ  0145h  ; a, dptr
; ----------------------------------------------------

        org 8000h

        lcall initadc
        setb  eal
begin:
        mov   a, analog0 ; read most recent analog value
        lcall shwbcd     ; display it
        mov   a, #WAIT
        lcall delay      ; show for a while

        lcall inkey      ; see if CONTROL-C pending
        cjne  a, #3h, begin ; if not repeat
        clr   eal        ; if so disable all interrupts
        ljmp  os_return  ; back to the monitor

; ====================================================
;  subroutine initadc - initialize adc
;  input    : none
;  output   : initializes adc interrupt
;             to continuously read analog 0
;  destroys : a, dptr
; ----------------------------------------------------
initadc:
        clr   eadc        ; disable adc interrupts
        clr   iadc        ; reset adc interrupt flag
        mov   a, #6       ; sixth interrupt source
        mov   dptr, #adcisr ; setup interrupt vector
        lcall setintvec
        setb  eadc        ; adc interrupt mask bit

        mov   a, adcon    ; read adc control register
        anl   a, #0d0h    ; select channel 0, single shot
        mov   adcon, a
        mov   dapr, #0    ; start conversion
        ret

; ====================================================
;  subroutine adcisr
;  input    : adc interrupt
;  output   : continuously obtains and stores analog 0
;             value in internal ram.
;  destroys : nothing - uses a, r0, and r1
; ----------------------------------------------------
adcisr:
        push  acc

        mov   a,   addat     ; read analog value
        mov   analog0, a     ; store analog value
        mov   a, adcon       ; adc control register
        anl   a, #0d0h
        mov   adcon, a       ; next select channel 0
        clr   iadc           ; reset adc interrupt flag
        mov   dapr, #0       ; start next conversion
done:
        pop   acc
        reti

; ====================================================
; subroutine shwbcd - shows a bcd number on the
;                        2 digital displays
;  input    : bcd number in the accumulator
;  output   : ports 4 and 5, connected to low digit
;             and high digit respectively.
;  destroys : a
; ----------------------------------------------------
shwbcd:
        push  acc     ; temporarily store byte
        anl   a, #0fh ; isolate low digit
        lcall display ; get 7-segment pattern
        mov   p4, a   ; send to LOW DIGIT
        pop   acc     ; recall byte
        swap  a       ; swap digits
        anl   a, #0fh ; isolate low nibble
        lcall display ; get 7-segment pattern
        mov   p5, a   ; display on HIGH DIGIT
        ret
; ====================================================
; end of program
