; ----------------------------------------------------
; EXP-TON2.ASM          Last Update:10/27/91
;
;  Simple Tone Generator Extension
;  Push Buttons PB0 to PB3 are connected to P1.0 to
;  P1.3; each button turns on a different tone.
;
; ====================================================
; UIOD connections
;
;  Connect P4.0 to the speaker.  The speaker must be
;    connected between P4.0 and Vcc (5 volts) via a
;    10uF to 100uF capacitor.
;  Connect P1.0 to Push Button 0.
;  Connect P1.1 to Push Button 1.
;  Connect P1.2 to Push Button 2.
;  Connect P1.3 to Push Button 3.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; --- port definitions -------------------------------
; input ports
  PB0       equ  90h   ; port bit P1.0
  PB1       equ  91h   ; port bit P1.1
  PB2       equ  92h   ; port bit P1.2
  PB3       equ  93h   ; port bit P1.3
; output ports
  Speaker   equ 0E8h   ; port bit P4.0
; constants
  Period0   equ  20h   ; various tones...
  Period1   equ  40h   ; ... try other values !
  Period2   equ  60h
  Period3   equ 0FFh
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 crlf      equ  0115h  ; a
 inkey     equ  012ah  ; a
 os_return equ  0130h
 print     equ  0136h  ; a, dptr
 prthex    equ  013fh  ; a, r2
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
   org   8000h
; --- program starts here ----------------------------
    lcall print
    db "program begins...", 10,13,0
    mov   P1, #0FFh      ; prepare Push Buttons for input

loop:
    lcall inkey
    cjne  a, #3, continue
    lcall print          ; print message
    db    10,13, "CTL-C received - program terminates", 10,13,0
    ljmp  os_return      ; return to OS

continue:
    jb    P1.0, L1
    mov   a, #Period0
    lcall Tone
L1:
    jb    P1.1, L2
    mov   a, #Period1
    lcall Tone
L2:
    jb    P1.2, L3
    mov   a, #Period2
    lcall Tone
L3:
    jb    P1.3, L4
    mov   a, #Period3
    lcall Tone
L4:
    sjmp  loop           ; repeat

; ::::::::::::::::::::::::::::::::::::::::::::::::::::
; --- subroutines ------------------------------------
; ====================================================
;  subroutine Tone
;   drives the speaker while Button is pressed
;  input    : tone period in accumulator
;             this is not the true period! - just a
;             count value
;  output   : speaker connected to P1.0
;  destroys : a, r0
; ----------------------------------------------------
Tone:
        mov   r0, a      ; save period in r0
        setb  Speaker    ; speaker voltage high
S_on:   djnz  r0, S_on   ; wait a little

        mov   r0, a      ; refresh period
        clr   Speaker    ; speaker voltage low
S_off:  djnz  r0, S_off  ; wait a little

        push  acc        ; save period
        mov   a, P1      ; read Push Buttons
        cpl   a          ; complement accumulator
        jz    Tdone      ; repeat while any Button pressed
        pop   acc        ; recall period
        sjmp  Tone
Tdone:
        pop   acc        ; discard byto on stack
        ret              ; return when all Buttons released

; ====================================================
; end of program
