; ----------------------------------------------------
; Int1.asm      Last Update:10/27/91
;
; Simple Interrupt Example
; External interrupt 0 is enabled and set up to
; respond to 1-to-0 transitions.  Each time an
; interrupt is received, LED0 is toggled.  The
; state of LED0 is stored in bit OUTFLAG.  The
; foregraund task displays digits 0 to F on
; LOW DIGIT.  Two nested loops are used to delay
; between updates of LOW DIGIT.  Note that there
; is no explicit calls to the interrupt service
; routine ISR0.  ISR0 is invoked upon  external
; signal.
;
; ----------------------------------------------
; system calls              registers used
; ----------------------------------------------
 break     equ  010ch  ; a, (reads accumulator)
 crlf      equ  0115h  ; a
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 print     equ  0136h  ; a, dptr
 prthex    equ  013fh  ; a, r2
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ====================================================
;       m a i n    p r o g r a m
; UIOD connections
;
; LOW DIGIT    P4     : output (segment a to P4.0, ...
;                               segment g to P4.6)
; L0           P5.0   ; output bit
; ----------------------------------------------------
; internal memory
; bits
 outflag   equ 20h    ; output indicator state
; ----------------------------------------------------

      org   8000h         ; run using the monitor
                          ;  G8000 command
      setb  IT0           ; set external interrupt
                          ; to be transition sensitive
      mov   a, #0         ; INT0 is the 0-th interrupt
                          ; source
      mov   dptr, #ISR0   ; set up dptr for a call to
                          ; setintvec
      lcall setintvec     ; install the ISR for INT0
      setb  EX0           ; enable external interrupt 0
      setb  EAL           ; master interrupt enable flag

      mov   b, #0         ; current count
      setb  outflag       ; start with indicator off
      setb  P5.0
      mov   r6, #0        ; initialize delay loop...
      mov   r7, #0        ; ... parameters
loop:
      djnz  r6, loop      ; waste a little time
      djnz  r7, loop

      mov   a, b          ; get current count
      lcall display       ; get display pattern
      mov   P4, a         ; display on LOW DIGIT
      inc   b             ; update current count
      anl   b, #7         ; modulus 8
      lcall inkey         ; check serial port for character
      lcall break         ; see if character is CTL-C
                          ; terminate program if CTL-C

      sjmp loop           ; repeat forever...

; ====================================================
;  subroutine ISR0
;  interrupt service routine for external interrupt 0
;  input    : external interrupt 0
;  output   : toggles the state of outflag.  the state
;             of outflag is copied to P5.0 which is
;             connected to LED0.
;             thus, each time an external interrupt 0
;             is received, LED0 is toggled.
;  destroys : nothing
; ----------------------------------------------------
ISR0:
    jb    outflag, OFset ; see if outflag is set
    setb  outflag        ; if not set outflag
    setb  P5.0           ; also set LED0
    reti                 ; return
OFset:
    clr   outflag        ; if outflag=1, clear it
    clr   P5.0           ; also clear LED0
    reti
; ====================================================
; end of program
