; ----------------------------------------------------
; Int2.asm      Last Update:10/27/91
;
; Simple Interrupt Example
; External interrupts 0 and 1 are enabled and
; set up to respond to 1-to-0 transitions.  Each
; time an interrupt is received, LEDs 0 and 1
; are toggled.  The state of the LEDs are stored
; in bits OUTFLAG0 and OUTFLAG1.  The
; foregraund task displays digits 0 to F on
; LOW DIGIT.  Two nested loops are used to delay
; between updates of LOW DIGIT.  Note that there
; is no explicit calls to the interrupt service
; routines ISR0 or ISR1.  the interrupt service
; routines are invoked upon  external signals
;
; ----------------------------------------------
; system calls              registers used
; ----------------------------------------------
 break     equ  010ch  ; a, (reads accumulator)
 crlf      equ  0115h  ; a
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 print     equ  0136h  ; a, dptr
 prthex    equ  013fh  ; a, r2
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ====================================================
;       m a i n    p r o g r a m
; UIOD connections
;
; LOW DIGIT    P4     : output (segment a to P4.0, ...
;                               segment g to P4.6)
; L0           P5.0   ; output bit 0
; L1           P5.1   ; output bit 1
; ----------------------------------------------------
; internal memory
; bits
 outflag0  equ 20h    ; output indicator 0 state
 outflag1  equ 21h    ; output indicator 1 state
; ----------------------------------------------------
      org   8000h         ; run using the monitor
                          ;  G8000 command
      setb  IT0           ; set external interrupt 0
                          ; to be transition sensitive
      mov   a, #0         ; INT0 is the 0-th interrupt
                          ; source
      mov   dptr, #ISR0   ; set up dptr for a call to
                          ; setintvec
      lcall setintvec     ; install the ISR for INT0
      setb  EX0           ; enable external interrupt 0

      setb  IT1           ; set external interrupt 1
                          ; to be transition sensitive
      mov   a, #2         ; INT1 is the 2-nd interrupt
                          ; source
      mov   dptr, #ISR1   ; set up dptr for a call to
                          ; setintvec
      lcall setintvec     ; install the ISR for INT1
      setb  EX1           ; enable external interrupt 1
      setb  EAL           ; master interrupt enable flag

      mov   b, #0         ; current count
      setb  outflag0      ; start with indicators off
      setb  P5.0
      setb  outflag1      ; start with indicators off
      setb  P5.1
      mov   r6, #0        ; initialize delay loop...
      mov   r7, #0        ; ... parameters
loop:
      djnz  r6, loop      ; waste a little time
      djnz  r7, loop

      mov   a, b          ; get current count
      lcall display       ; get display pattern
      mov   P4, a         ; display on LOW DIGIT
      inc   b             ; update current count
      anl   b, #7         ; modulus 8
      lcall inkey         ; check serial port for character
      lcall break         ; see if character is CTL-C
                          ; terminate program if CTL-C

      sjmp loop           ; repeat forever...

; ====================================================
;  subroutine ISR0
;  interrupt service routine for external interrupt 0
;  input    : external interrupt 0
;  output   : toggles the state of outflag0.  the state
;             of outflag is copied to P5.0 which is
;             connected to LED0.
;             thus, each time an external interrupt 0
;             is received, LED0 is toggled.
;  destroys : nothing
; ----------------------------------------------------
ISR0:
    jb    outflag0, OFset0 ; see if outflag0 is set
    setb  outflag0         ; if not set outflag0
    setb  P5.0             ; also set LED0
    reti                   ; return
OFset0:
    clr   outflag0         ; if outflag0=1, clear it
    clr   P5.0             ; also clear LED0
    reti

; ====================================================
;  subroutine ISR1
;  interrupt service routine for external interrupt 1
;  input    : external interrupt 1
;  output   : toggles the state of outflag1.  the state
;             of outflag is copied to P5.1 which is
;             connected to LED1.
;             thus, each time an external interrupt 1
;             is received, LED1 is toggled.
;  destroys : nothing
; ----------------------------------------------------
ISR1:
    jb    outflag1, OFset1 ; see if outflag1 is set
    setb  outflag1         ; if not set outflag1
    setb  P5.1             ; also set LED0
    reti                   ; return
OFset1:
    clr   outflag1         ; if outflag1=1, clear it
    clr   P5.1             ; also clear LED0
    reti
; ====================================================
; end of program
