; ----------------------------------------------------
; KeyLess1.asm      Last Update:10/27/90
;
; Keyless Entry System Example
;
; Courtesy Rigel Corporation
;
;
; ====================================================
;       m a i n    p r o g r a m
; internal data use
; bytes
  i_left equ  7eh ; number of incorrect buttons pressed
  count  equ  7fh ; count register for waiting
; ----------------------------------------------------
; UIOD connections
;
; P4.0   PB0 : key 0
; P4.1   PB1 : key 1
; P4.2   PB2 : key 2
; P4.3   PB3 : key 3
; P4.4   L0  : unlock
; P4.5   L1  : alarm
; P4.6   PB7 : reset lock
; ----------------------------------------------------
; key sequence 0-2-3-1 unlocks
; L0 indicates unlock
; L1 indicates alarm
;
; 10 incorrect key presses allowed before the alarm
; PB7 resets the lock
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------

 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a

; ----------------------------------------------
     org  8000h  ; run using the monitor G8000
                 ;   command
initialize:

      mov  P4, #0ffh    ; prepare port 4 for input...
                        ; and turn off LEDs

      mov  i_left, #10  ; set i_left to 10 trials

      lcall chkbrk      ; check user break

begin:

; ===== wait for button 0 ============================

; ----- repeat until a button is pressed -------------

press0:
      mov  a, P4        ; read the state of PB0-PB3
      mov  b, a         ; save port 4 status in b
      cpl  a            ; complement 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jz   press0       ; if no button pressed try again

; ----- wait until button is released ----------------

wait0:
      mov  a, P4        ; read the state of PB0-PB3
      cpl  a            ; complement: 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jnz  wait0        ; if button pressed try again

; ----- wait a bit more while the button bounces -----
      mov  count, #0ffh
delay0:
      djnz count, delay0 ; wait for debouncing

; ----- is the button pressed PB0 ? ------------------
      jnb  b.0, PB0_pressed
                         ; decrement number of incorrect
                         ; trials left.  if more trials
                         ; left then retry.
      djnz i_left, begin
            ; otherwise
            ; set off alarm
      ljmp alarm

PB0_pressed:

; ===== wait for button 2 ============================

; ----- repeat until a button is pressed -------------

press2:
      mov  a, P4        ; read the state of PB0-PB3
      mov  b, a         ; save port 4 status in b
      cpl  a            ; complement: 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jz   press2       ; if no button pressed try again

; ----- wait until button is released ----------------
wait2:
      mov  a, P4        ; read the state of PB0-PB3
      cpl  a            ; complement: 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jnz  wait2        ; if button pressed try again

; ----- wait a bit more while the button bounces -----
      mov  count, #0ffh
delay2:
      djnz count, delay2 ; wait for debouncing

; ----- is the button pressed PB2 ? ------------------
      jnb  b.2, PB2_pressed
            ; decrement number of incorrect
            ; trials left.  if more trials
            ; left then retry.
      djnz i_left, begin
            ; otherwise
            ; set off alarm
      ljmp alarm

PB2_pressed:

; ===== wait for button 3 ============================

; ----- repeat until a button is pressed -------------

press3:

      mov  a, P4        ; read the state of PB0-PB3
      mov  b, a         ; save port 4 status in b
      cpl  a            ; complement: 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jz   press3       ; if no button pressed try again

; ----- wait until button is released ----------------

wait3:
      mov  a, P4        ; read the state of PB0-PB3
      cpl  a            ; complement: 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jnz  wait3        ; if button pressed try again

; ----- wait a bit more while the button bounces -----
      mov  count, #0ffh
delay3:
      djnz count, delay3 ; wait for debouncing

; ----- is the button pressed PB3 ? ------------------
      jnb  b.3, PB3_pressed
                         ; decrement number of incorrect
                         ; trials left.  if more trials
                         ; left then retry.
      djnz i_left, begin
                         ; otherwise
                         ; set off alarm
      ljmp alarm

PB3_pressed:

; ===== wait for button 1 ============================

; ----- repeat until a button is pressed -------------

press1:

      mov  a, P4        ; read the state of PB0-PB3
      mov  b, a         ; save port 4 status in b
      cpl  a            ; complement: 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jz   press1       ; if no button pressed try again

; ----- wait until button is released ----------------
wait1:
      mov  a, P4        ; read the state of PB0-PB3
      cpl  a            ; complement: 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jnz  wait1        ; if button pressed try again

; ----- wait a bit more while the button bounces -----
      mov  count, #0ffh
delay1:
      djnz count, delay1 ; wait for debouncing

; ----- is the button pressed PB1 ? ------------------
      jnb  b.1, PB1_pressed
                         ; decrement number of incorrect
                         ; trials left.  if more trials
                         ; left then retry.
      djnz i_left, begin
            ; otherwise
            ; set off alarm
      ljmp alarm

PB1_pressed:

; ----- all 4 buttons correctly pressed --------------

ok:
      clr  P4.4        ; activate unlock - led 0
      mov  a, P4
      jb   P4.6, ok    ; wait until alarm reset
      ljmp initialize  ; ...then start over

; ----- 10 unsuccessful trials - set off alarm -------
alarm:
      clr  P4.5        ; activate alarm led 1
      mov  a, P4
      jb   P4.6, alarm ; wait until alarm reset
      ljmp initialize  ; ...then start over

; ====================================================
;
; end of program
