; ----------------------------------------------------
; KeyLess2.asm      Last Update:10/27/90
;
; Keyless Entry System Example - Uses Subroutines
;
; Courtesy Rigel Corporation
;
;
; ====================================================
;       m a i n    p r o g r a m
; internal data use
; bytes
  i_left equ  7eh ; number of incorrect buttons pressed
  count  equ  7fh ; count register for waiting
; ----------------------------------------------------
; UIOD connections
;
; P4.0   PB0 : key 0
; P4.1   PB1 : key 1
; P4.2   PB2 : key 2
; P4.3   PB3 : key 3
; P4.4   L0  : unlock
; P4.5   L1  : alarm
; P4.6   PB7 : reset lock
; ----------------------------------------------------
; key sequence 0-2-3-1 unlocks
; L0 indicates unlock
; L1 indicates alarm
;
; 10 incorrect key presses allowed before the alarm
; PB7 resets the lock
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------

      org  8000h        ; run using the monitor G8000
                        ;   command
initialize:

      mov  P4, #0ffh    ; prepare port 4 for input...
                        ; and turn off LEDs

      mov  i_left, #10  ; set i_left to 10 trials

      lcall chkbrk      ; check user break

begin:

      lcall w_press     ; wait for button pressed
      lcall w_rel       ; wait for button released
      mov   c, b.0      ; button status to carry flag
      lcall b_press     ; is button pressed ?

      lcall w_press     ; wait for button pressed
      lcall w_rel       ; wait for button released
      mov   c, b.2      ; button status to carry flag
      lcall b_press     ; is button pressed ?

      lcall w_press     ; wait for button pressed
      lcall w_rel       ; wait for button released
      mov   c, b.3      ; button status to carry flag
      lcall b_press     ; is button pressed ?

      lcall w_press     ; wait for button pressed
      lcall w_rel       ; wait for button released
      mov   c, b.1      ; button status to carry flag
      lcall b_press     ; is button pressed ?

; ----- all 4 buttons correctly pressed --------------

ok:
      clr  P4.4        ; activate unlock - led 0
      mov  a, P4
      jb   P4.6, ok    ; wait until alarm reset
      ljmp initialize  ; ...then start over

; ----- 10 unsuccessful trials - set off alarm -------
alarm:
      clr  P4.5        ; activate alarm led 1
      mov  a, P4
      jb   P4.6, alarm ; wait until alarm reset
      ljmp initialize  ; ...then start over


;=====================================================
; subroutine w_press
; waits until one of the bits P4.0 to P4.3 is pressed
;
; input  : none
; output : P4 status is returned in the b register
; uses   : a and b registers
;-----------------------------------------------------

w_press:
      lcall chkbrk      ; watch for user breaks

      mov  a, P4        ; read the state of PB0-PB3
      mov  b, a         ; save port 4 status in b
      cpl  a            ; complement 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jz   w_press      ; if no button pressed try again

      ret


;=====================================================
; subroutine w_rel
; waits until all of the bits P4.0 to P4.3 are
;  released
;
; input  : none
; output : none
; uses   : a (accumulator)
;-----------------------------------------------------

w_rel:
      mov  a, P4        ; read the state of PB0-PB3
      cpl  a            ; complement: 1 means PB pressed
      anl  a, #0fh      ; isolate low nibble
      jnz  w_rel        ; if button pressed try again

; ----- wait a bit more while the button bounces -----
      mov  a, #0ffh
w0:   djnz acc, w0      ; wait for debouncing

      ret

;=====================================================
; subroutine b_press
; a special purpose subroutine that checks if the
; carry flag is cleared.  the carry flag holds the
; status of the desired button.  if button is pressed
; then b_press returns.  if it is not, then i_left is
; updated.  if i_left=0 then the routine jumps (not
; return!) to alarm, else it jumps to begin.
;
; input  : button status in carry flag
; output : none
; uses   : a (accumulator)
;-----------------------------------------------------
b_press:
      jnc  bp_ret
      pop  acc          ; flush return address
      pop  acc          ; flush return address
      djnz i_left, xbegin

      ljmp alarm        ; no trials left
xbegin:
      ljmp begin

bp_ret:
      ret

;=====================================================
;
; end of program
