; ----------------------------------------------------
; KeyLess3.asm      Last Update:10/27/90
;
; Keyless Entry System Example
; rogrammable Unlock Sequence
;
; Courtesy Rigel Corporation 
;
;
; ====================================================
;       m a i n    p r o g r a m
; internal data use
; bytes
first equ   70h ; first  key to press
second equ  71h ; second key to press
third equ   72h ; third  key to press
fourth equ  73h ; fourh  key to press
i_left equ  74h ; number of incorrect buttons pressed
; ----------------------------------------------------
; port use
; p4.0  connected to pb0 - key 0
; p4.1  connected to pb1 - key 1
; p4.2  connected to pb2 - key 2
; p4.3  connected to pb3 - key 3
; p4.4  connected to l0  - unlock
; p4.5  connected to l1  - alarm
; p4.6  connected to pb7 - reset lock
; ----------------------------------------------------
; sequence 3-0-2-1 unlocks
; 10 incorrect key presses sets off alarm
; pb7 resets the lock
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h  
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------
     org  8000h       ; run using the monitor G8000
                      ;   command
initialize:

      mov  p4, #0ffh    ; prepare port 4 for input...
                        ; and turn off leds

      mov  i_left, #10  ; set i_left to 10 trials
      mov  first,  #3   ; set key sequence to unlock
      mov  second, #0
      mov  third,  #2
      mov  fourth, #1

begin:
      lcall w_press     ; wait for button pressed
      lcall w_rel       ; wait for button released
      mov   r0, first   ; load the acc with correct key
      lcall getbutton   ; get button status of correct...
                        ; next key and move to carry flag
      lcall b_press     ; is right button pressed ?


      lcall w_press     ; wait for button pressed
      lcall w_rel       ; wait for button released
      mov   r0, second  ; load the acc with correct key
      lcall getbutton   ; get button status of correct...
                        ; next key and move to carry flag
      lcall b_press     ; is right button pressed ?

      lcall w_press     ; wait for button pressed
      lcall w_rel       ; wait for button released
      mov   r0, third   ; load the acc with correct key
      lcall getbutton   ; get button status of correct...
                        ; next key and move to carry flag
      lcall b_press     ; is right button pressed ?

      lcall w_press     ; wait for button pressed
      lcall w_rel       ; wait for button released
      mov   r0, fourth  ; load the acc with correct key
      lcall getbutton   ; get button status of correct...
                        ; next key and move to carry flag
      lcall b_press     ; is right button pressed ?

; ----- all 4 buttons correctly pressed --------------
ok:
      clr   p4.4       ; activate unlock - led 0
      mov   a, p4
      jb    p4.6, ok   ; wait until alarm reset
      mov   a, #0ffh   ; delay a bit for...
dly:  djnz  acc, dly   ; ...for debouncing
      ljmp initialize  ; ...then start over

; ----- 10 unsuccessful trials - set off alarm -------
alarm:
      clr  p4.5        ; activate alarm led 1
      mov  a, p4
      jb   p4.6, alarm ; wait until alarm reset
      ljmp initialize  ; ...then start over
;=====================================================
; subroutine w_press
; waits until one of the bits p4.0 to p4.3 is pressed
;
; input  : none
; output : p4 status is returned in the b register
; uses   : a and b registers
;-----------------------------------------------------
w_press:
      mov   a, p4       ; read the state of pb0-pb3
      mov   b, a        ; save port 4 status in b
      cpl   a           ; complement 1 means pb pressed
      anl   a, #0fh     ; isolate low nibble
      jz    w_press     ; if no button pressed try again
      ret
;=====================================================
; subroutine w_rel
; waits until all of the bits p4.0 to p4.3 are
;  cleared
;
; input  : none
; output : none
; uses   : a (accumulator)
;-----------------------------------------------------
w_rel:
      mov   a, p4       ; read the state of pb0-pb3
      cpl   a           ; complement: 1 means pb pressed
      anl   a, #0fh     ; isolate low nibble
      jnz   w_rel       ; if button pressed try again
; ----- wait a bit more while the button bounces -----
      mov   a, #0ffh    ; delay a little...
wr1:  djnz  acc, wr1    ; ...for debouncing
      ret
;=====================================================
; subroutine getbutton
;
; input  : correct key number (n) in register 0,
;            port status in b register.
;
; output : carry flag = bit n of b (port 4)
; uses   : a (accumulator), carry flag, b register,
;           and register 0.
;-----------------------------------------------------
getbutton:
      mov   a, b      ; save port status in a
      inc   r0        ; need to rotate once for bit 0
gb1:
      rrc   a         ; rotate port status right
      djnz  r0, gb1   ; is n the next key?
                      ; if not repeat...
gb_ret:
      ret
;=====================================================
; subroutine b_press
; a special purpose subroutine that checks if the
; carry flag is cleared.  the carry flag holds the
; status of the desired button.  if button is pressed
; then b_press returns.  if it is not, then i_left is
; updated.  if i_left=0 then the routine jumps (not
; return!) to alarm, else it jumps to begin.
;
; input  : button status in carry flag
; output : none
; uses   : a (accumulator)
;-----------------------------------------------------
b_press:
      jnc  bp_ret
      pop  acc          ; flush return address
      pop  acc          ; flush return address
      djnz i_left, xbegin

      ljmp alarm        ; no trials left
xbegin:
      ljmp begin
bp_ret:
      ret
;=====================================================
;
; end of program

