; ----------------------------------------------------
; LCD4.asm      Last Update:8/23/91
;
; LCD Display Demonstration Program
;
; Demonstrates 4-bit data bus operation.  the bits
; connected to the LCD may be any 4 bits of the
; microcontroller.  this exapmle uses port bits
; P1.4 to p1.7 as LCD data bits DB4 to DB7
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD connections
;
; constants
 LCD_DATA  equ 090h ; port 1 is used for data
 LCD_DB4   equ 094h ; high nibble of port 1 is used for data
 LCD_DB5   equ 095h ; high nibble of port 1 is used for data
 LCD_DB6   equ 096h ; high nibble of port 1 is used for data
 LCD_DB7   equ 097h ; high nibble of port 1 is used for data
 LCD_RS    equ 0e8h ; p4.0 LCD Register Select line
 LCD_RW    equ 0e9h ; p4.1 LCD Read / Write line
 LCD_E     equ 0eah ; p4.2 LCD Enable line

; system instructions
 Config    equ 28h  ; 4 bit data, 2 lines, 5 by 7 character matrix
 entryMode equ 6    ; increment cursor, do not shift display

; cursor control instructions
 offCur    equ  0Ch
 lineCur   equ  0Eh
 blinkCur  equ  0Dh
 combnCur  equ  0Fh
 homeCur   equ  02h
 shLfCur   equ  10h
 shRtCur   equ  14h

; display control instructions
 clrDsp    equ  01h
 offDsp    equ  0Ah
 onDsp     equ  0Eh
 shLfDsp   equ  18h
 shRtDsp   equ  1Ch

; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------
    org 8000h

; initialize the LCD module --------------------------
    lcall resetLCD4    ; duplicate power on reset
    lcall print
    db    "resetLCD4 completed",0
    lcall crlf
    lcall getchr

    lcall initLCD4     ; initialize fonts, #lines, Entry Mode
    lcall print
    db    "initLCD completed",0
    lcall crlf
    lcall getchr

; print a message ------------------------------------
    lcall prtLCD4
    db    "Hello World !", 0

    lcall waitKey      ; wait for a key press

; blank (quench) the display - retain contents -------
    mov   r0, #offDsp
    lcall wrLCDcom4

    lcall waitKey      ; wait for a key press

; turn display back on -------------------------------
    mov   r0, #onDsp
    lcall wrLCDcom4

    lcall waitKey      ; wait for a key press

; ----------------------------------------------------
; place cursor on line 2, position 7 and print a message
    mov   a, #2        ; line 2
    mov   b, #7        ; position 7
    lcall placeCur4

    lcall prtLCD4      ; print message
    db "line 2", 0

    lcall waitKey      ; wait for a key press

; clear display - contents are erased ----------------
    mov   r0, #clrDsp
    lcall wrLCDcom4

    lcall waitKey      ; wait for a key press

; now let us define some custom characters -----------
; first set the CGRAM address, then download the
; font patterns.  characters 0 and 1 are defined
; successively.  character 0 is a Box, and character
; 1 is a Diamond.  for a blank font row, the
; bit pattern 20h is sent.  note that the last 5
; bits of 20h are 0.  this is to avoid sending 0,
; which would signal the end of the data string.

    mov   a, #0     ; select character 0
    mov   b, #0     ; select font row 0
    lcall setCGRAM4
    lcall prtLCD4
    db    1Fh, 11h, 11h, 11h, 11h, 11h, 1Fh, 20h    ; Box
    db    20h, 04h, 0Eh, 1Fh, 0Eh, 04h, 20h, 20h, 0 ; Diamond

; clearing the display selects the DDRAM location 0
; (homes the cursor) in preperation to display
; messages.
    mov   r0, #clrDsp
    lcall wrLCDcom4
; print message.  since 0 implies end of string,
; use character 8.  note that characters 0..7
; are the same as characters 8..Fh.
    lcall prtLCD4             ; print message
    db    "     Box : ", 8, 0
; move cursor to line 2
    mov   a, #2               ; line 2
    mov   b, #0               ; position 0
    lcall placeCur4

    lcall prtLCD4             ; print message
    db    " Diamond : ", 1, 0

    lcall waitKey      ; wait for a key press

; now let us shift the display - first let us write----
; a long message

    mov   r0, #clrDsp  ; clear display
    lcall wrLCDcom4

; now write a long message ---------------------------
; the following message is 40 characters long,
; the capacity of each line.  note that, since
; the cursor is incremented after each character,
; the cursor overflows onto the next line
    lcall prtLCD4      ; write long message
    db    "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcd",0

    lcall waitKey      ; wait for a key press

; bring the cursor to the beginning of the top line --
    mov   r0, #homeCur
    lcall wrLCDcom4

    lcall waitKey      ; wait for a key press

; let us shift the display ---------------------------
    mov   a, #1        ; shift 1 notch
    lcall dspShRt4     ; shift

    lcall waitKey      ; wait for a key press

    mov   a, #9        ; shift 9 more notches
    lcall dspShRt4     ; shift

    lcall waitKey      ; wait for a key press

; now let us shift the display to the left past origin
    mov   a, #20       ; shift 20 notches
    lcall dspShLf4     ; shift past origin

    lcall waitKey      ; wait for a key press

; finally shift the display back to the origin
    mov   a, #10       ; shift 10 notches
    lcall dspShRt4     ; shift to origin

    lcall waitKey      ; wait for a key press

; shift the cursor to the right
    mov   a, #1        ; shift 1 notch
    lcall crShRt4      ; shift right

    lcall waitKey      ; wait for a key press

; shift the cursor to the left 2 notches, past the origin.
; thus, the cursor will be out of the viewing range.
    mov   a, #2        ; shift 2 notches
    lcall crShLf4      ; shift past origin

    lcall waitKey      ; wait for a key press

; bring the cursor to the beginning of the top line --
    mov   r0, #homeCur
    lcall wrLCDcom4

;    lcall waitKey      ; wait for a key press
;    lcall initLCD4     ; clear display

    lcall crlf
    lcall print
    db    "program terminates... press ALT-F4", 0

    ljmp  os_return
; end of main program

; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; subroutines
; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

; waitKey is a utility that prints the message
; "press a key to continue" and then waits until
; a character is received.
waitKey:
    lcall print
    db    "press a key to continue", 0
    lcall crlf
    lcall getchr       ; wait for a key press
    ret

; ==========================================================
; subroutine crShLf4
; this routine shifts the cursor to the left.  the number of
; positions to be shifted is is placed in the accumulator.
; input  : accumulator indicates number of notches to shift
; output : none
; ==========================================================
;
crShLf4:
         jz    ret_scl
         mov   r0, #shLfCur ; control word for left shift
         lcall wrLCDcom4
         dec   acc
         sjmp  crShLf4
ret_scl: ret

; ==========================================================
; subroutine crShRt4
; this routine shifts the cursor to the right.  the number of
; positions to be shifted is is placed in the accumulator.
; input  : accumulator indicates number of notches to shift
; output : none
; ==========================================================
;
crShRt4:
         jz    ret_scr
         mov   r0, #shRtCur ; control word for right shift
         lcall wrLCDcom4
         dec   acc
         sjmp  crShRt4
ret_scr: ret

; ==========================================================
; subroutine placeCur4
; this routine sets the cursor position.  the cursor position
; is placed in the b register.  the cursor position starts
; at 0.  the accumulator gives the line number.
; input  : accumulator indicates line number (1, 2)
;        : b register holds the cursor position
; output : none
; ==========================================================
;
placeCur4:
         dec  acc          ; acc=0 for line=1
         jnz  line2        ; if acc=0 then first line
         mov  a, b
         add  a, #080h     ; construct control word for line 1
         sjmp setcur
line2:
         mov  a, b
         add  a, #0C0h     ; construct control word for line 2
setcur:
         mov  r0, a        ; place control word
         lcall wrLCDcom4    ; issue command
         ret

; ==========================================================
; subroutine setCGRAM4
; this routine sets the Character Generator RAM address.  the
; character [0..7] is placed in the accumulator.  the font row
; is placed in the b register.  the instructions 40h to 7Fh
; selects the CGRAM addresses 0 to 3Fh (0 to 63 decimal).
;
; the 5x7 character font is assumed.  that is, each character
; location contains a matrix of 40 dots arranged in 8 rows of
; 5 dots each.  the 7 rows are for the characters and the 8-th
; row for the underline cursor.  CGRAM address 0 corresponds
; to the first (font) row of character 0, address 7 corresponds
; to the 8-th (last) row of character 0, and address 3Fh
; corresponds to the 8-th row of character 7.
;
; input  : accumulator indicates character code [0..7]
;        : b register holds font row [0..7]
; output : none
; ==========================================================
;
setCGRAM4:
         push  b
         mov   b, #8
         mul   ab           ; multiply character number by 8
         pop   b            ; b holds row number
         add   a, b         ; a holds CGRAM address
         add   a, #40h      ; convert to instruction

         mov   r0, a        ; place instruction
         lcall wrLCDcom4    ; issue command
         ret

; ==========================================================
; subroutine dspShLf4
; this routine shifts the LCD contents to the left.  the
; number of characters to be shifted is is placed in the
; accumulator.
; input  : accumulator indicates number of notches to shift
; output : none
; ==========================================================
;
dspShLf4:
         jz    ret_sdl
         mov   r0, #shLfDsp ; control word for left shift
         lcall wrLCDcom4
         dec   a
         sjmp  dspShLf4
ret_sdl: ret

; ==========================================================
; subroutine dspShRt4
; this routine shifts the LCD contents to the right.  the
; number of characters to be shifted is is placed in the
; accumulator.
; input  : accumulator indicates number of notches to shift
; output : none
; ==========================================================
;
dspShRt4:
         jz    ret_sdr
         mov   r0, #shRtDsp ; control word for right shift
         lcall wrLCDcom4
         dec   a
         sjmp  dspShRt4
ret_sdr: ret

; ====================================================
; subroutine wrLCDdata4
; write a data word to the LCD
;
; data must be placed in r0 by calling program
; ----------------------------------------------------
wrLCDdata4:
    clr   LCD_E
    setb  LCD_RS        ; select send Data
    clr   LCD_RW        ; select write operation
    push  acc           ; save accumulator
    mov   a, r0         ; put data byte in acc

    mov   c, acc.4      ; load high nibble on data bus
    mov   LCD_DB4, c    ; one bit at a time using...
    mov   c, acc.5      ; bit move operatoins
    mov   LCD_DB5, c
    mov   c, acc.6
    mov   LCD_DB6, c
    mov   c, acc.7
    mov   LCD_DB7, c

    setb  LCD_E         ; pulse the Enable line
    clr   LCD_E

    mov   c, acc.0      ; similarly, load low nibble
    mov   LCD_DB4, c
    mov   c, acc.1
    mov   LCD_DB5, c
    mov   c, acc.2
    mov   LCD_DB6, c
    mov   c, acc.3
    mov   LCD_DB7, c

    lcall pulseEwait4   ; pulse the Enable line...
                        ; and wait for Busy Flag to clear
    pop   acc
    ret

; ====================================================
; subroutine wrLCDcom4
; write a command word to the LCD
;
; command must be placed in r0 by calling program
; ----------------------------------------------------
wrLCDcom4:
    clr   LCD_E
    clr   LCD_RS        ; select send Command
    clr   LCD_RW        ; select Write operation
    push  acc           ; save accumulator
    mov   a, r0         ; put data byte in acc

    mov   c, acc.4      ; load high nibble on data bus
    mov   LCD_DB4, c    ; one bit at a time using...
    mov   c, acc.5      ; bit move operatoins
    mov   LCD_DB5, c
    mov   c, acc.6
    mov   LCD_DB6, c
    mov   c, acc.7
    mov   LCD_DB7, c

    setb  LCD_E         ; pulse the Enable line
    clr   LCD_E

    mov   c, acc.0      ; similarly, load low nibble
    mov   LCD_DB4, c
    mov   c, acc.1
    mov   LCD_DB5, c
    mov   c, acc.2
    mov   LCD_DB6, c
    mov   c, acc.3
    mov   LCD_DB7, c

    lcall pulseEwait4   ; pulse the Enable line...
                        ; and wait for Busy Flag to clear
    pop   acc
    ret

; ====================================================
; subroutine rdLCDstat4
; reads the status byte (Busy Flag and Address Couter)
; the status byte is returned in the accumulator
; input  : none
; output : status byte in accumulator
; ----------------------------------------------------
rdLCDstat4:
    push  b
    mov   LCD_DATA, #0ffh ; prepare port for input
    setb  LCD_RW          ; prepare R/W for read operation

    setb  LCD_E           ; start the Enable pulse
    mov   a, LCD_DATA     ; read status high nibble
    clr   LCD_E           ; end Enable pulse
    anl   a, #0f0h
    mov   b, a
    setb  LCD_E           ; start the Enable pulse
    mov   a, LCD_DATA     ; read status low nibble
    clr   LCD_E           ; end Enable pulse
    swap  a
    anl   a, #0fh
    add   a, b
    pop   b

    ret

; ====================================================
; subroutine pulseEwait4
; generates a positive pulse on the LCD enable line.
; waits for the Busy Flag to clear before returning.
; input    : none
; output   : none
; destroys : LCD_RW, LCD_RS, LCD_DATA
;
; ----------------------------------------------------
pulseEwait4:
    clr   LCD_E
    setb  LCD_E           ; pulse the Enable line
    clr   LCD_E

    mov   LCD_DATA, #0ffh ; prepare port for input
    setb  LCD_RW          ; prepare R/W for read operation

    push  acc             ; save accumulator contents
pEw:
    setb  LCD_E           ; start the Enable pulse
    mov   a, LCD_DATA     ; read status nibble
    clr   LCD_E           ; end Enable pulse
    setb  LCD_E           ; pretend reading status low nibble
    clr   LCD_E
    jb    acc.7, pEw      ; loop while Busy Flag is set
    pop   acc             ; restore accumulator
    ret

; ====================================================
; subroutine initLCD4 - initialize the LCD
;
; ----------------------------------------------------
initLCD4:
    clr LCD_RS      ; LCD Register Select line
    clr LCD_RW      ; Read / Write line
    clr LCD_E       ; Enable line

    mov   r0, #Config ; Function Set - data bits, lines, fonts
    lcall wrLCDcom4

    mov   r0, #onDsp     ; display ON
    lcall wrLCDcom4

    mov   r0, #entryMode ; set Entry Mode
    lcall wrLCDcom4 ; increment cursor to right, no display shift

    mov   r0, #clrDsp  ; clear display, home cursor
    lcall wrLCDcom4

    ret

; ====================================================
; subroutine resetLCD4 - reset the LCD
; software version of the power on reset operation
;
; ----------------------------------------------------
resetLCD4:
    clr   LCD_RS          ; LCD Register Select line
    clr   LCD_RW          ; Read / Write line
    clr   LCD_E           ; Enable line

    clr   LCD_DB7         ; set bit pattern for...
    clr   LCD_DB6         ; ... power-on-reset
    setb  LCD_DB5
    setb  LCD_DB4
                          ; step 1
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #4           ; delay 4 milliseconds
    lcall mdelay
                          ; step 2
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #1           ; delay 1 millisecond
    lcall mdelay
                          ; step 3
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #1           ; delay 1 millisecond
    lcall mdelay

    clr   LCD_DB4         ; specify 4-bit operation
                          ; step 4
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #1           ; delay 1 millisecond
    lcall mdelay

    mov   r0, #Config     ; FUNCTION SET
    lcall wrLCDcom4

    mov   r0, #08h        ; display off
    lcall wrLCDcom4

    mov   r0, #1          ; clear display, home cursor
    lcall wrLCDcom4

    mov   r0, #entryMode  ; set ENTRY MODE
    lcall wrLCDcom4

    ret

; ==========================================================
; subroutine prtLCD4
; takes the string immediately following the call and
; displays on the LCD.  the string is read with the
;   mov a, @a+dptr
; instruction.  thus, the string is in data memory.  since
; the string immediately follows code, the microcontroller
; is used with overlapping code and data memories (the von
; Newman architecture).  string must be terminated with a
; null (0).  upon termination this routine jumps to the
; instruction immediately following the string.
;
; input    : none
; output   : none
; destroys : acc, dptr
; ==========================================================
;
prtLCD4:
         pop   dph          ; pop return address into dptr
         pop   dpl
prtNext:
         clr  a             ; set offset = 0
         movc a,  @a+dptr   ; get chr from code memory
         cjne a, #0, chrOK  ; if chr = 0 then return
         sjmp retPrtLCD
chrOK:
         mov   r0, a
         lcall wrLCDdata4   ; send character
         inc   dptr         ; point at next character
         ljmp  prtNext      ; loop till end of string
retPrtLCD:
         mov   a,  #1h      ; point to instruction after string
         jmp   @a+dptr      ; return with a jump instruction

; ====================================================
; end of program
