; ----------------------------------------------------
; LCD4V2.asm      Last Update:8/23/91
;
; LCD Display Demonstration Program
; implements a Voltmeter.
;
; Demonstrates 4-bit data bus operation.  the bits
; connected to the LCD may be any 4 bits of the
; microcontroller.  this exapmle uses port bits
; P1.4 to p1.7 as LCD data bits DB4 to DB7
;
; the voltage seen at the microcontroller pin is
; displayed in decimal on the LCD.  the full range,
; i.e., the value displayed when an0=5 volts must be
; an integer constant.  this constant, named 'range'
; is defined below by an EQU psudo-op.  with range=5,
; the voltmeter measures the true voltage on input an0.
;
; jumpers jp2 and jp3 must be installed !

; notice that since the analog-to-digital conversion
; is interrupt driven (in the back ground), in order
; to preserve the timing while sending commands and
; data to the LCD, the master interrupt flag eal
; (enable all) is cleared.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD connections:
;  80535 port an0 to the potentiometer an0 of the uiod
;  through a 1 kilo ohm (any value between 1k and 10k
;  will work equally well) resistor.
;
; internal memory use
; bytes:
   analog0 equ 0c0h    ; 0-th adc input
;
; global constants
   range equ   5       ; voltage for 256 count
; ----------------------------------------------------
;
; LCD constants
 LCD_DATA  equ 090h ; port 1 is used for data
 LCD_DB4   equ 094h ; high nibble of port 1 is used for data
 LCD_DB5   equ 095h ; high nibble of port 1 is used for data
 LCD_DB6   equ 096h ; high nibble of port 1 is used for data
 LCD_DB7   equ 097h ; high nibble of port 1 is used for data
 LCD_RS    equ 0e8h ; p4.0 LCD Register Select line
 LCD_RW    equ 0e9h ; p4.1 LCD Read / Write line
 LCD_E     equ 0eah ; p4.2 LCD Enable line
 LCDwidth  equ 20   ; assume 20 characters per line

; LCD system instructions
 Config    equ 28h  ; 4 bit data, 2 lines, 5 by 7 character matrix
 entryMode equ 6    ; increment cursor, do not shift display

; cursor control instructions
 offCur    equ  0Ch
 lineCur   equ  0Eh
 blinkCur  equ  0Dh
 combnCur  equ  0Fh
 homeCur   equ  02h
 shLfCur   equ  10h
 shRtCur   equ  14h

; display control instructions
 clrDsp    equ  01h
 offDsp    equ  0Ah
 onDsp     equ  0Eh
 shLfDsp   equ  18h
 shRtDsp   equ  1Ch

; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; system flags ---------------------------------------
 errorf    equ 3       ; error flag
; ----------------------------------------------------

; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; Main Program
; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    org 8000h

; initialize the LCD module --------------------------
    lcall resetLCD4    ; duplicate power on reset
    lcall print
    db    "resetLCD4 completed",0
    lcall crlf
    lcall initLCD4     ; initialize fonts, #lines, Entry Mode
    lcall print
    db    "initLCD completed",0
    lcall crlf

; print messages  ------------------------------------
    mov   r0, #homeCur   ; place cursor at line 1, position 0
    lcall wrLCDcom4
    lcall prtLCD4        ; print promt message
    db    "AN0 : ", 0
    mov   a, #1          ; row 1
    mov   b, #12         ; location 12
    lcall placeCur4      ; place cursor at line 1, position 12
    lcall prtLCD4
    db    "Volts", 0     ; print units message
    mov   r0, #offCur    ; supress cursor while showing Volts
    lcall wrLCDcom4

    lcall setBarChars

    mov   ip0, #0        ;  all interrupts low priority
    mov   ip1, #0
    lcall initadc        ; initialize ADC interrupt service routine
    setb  eal            ; enable interrupts

; --- main loop starts here ----------------------------
begin:
    mov   a, #1      ; row 1
    mov   b, #6      ; location 6
    clr   eal        ; hold interrupts while talking to LCD
    lcall placeCur4  ; place cursor at line 1, position 13
    setb  eal        ; enable interrupts again

    mov   r0, #analog0
    mov   a, @r0     ; get analog value
    mov   b, #range
    mul   ab         ; b holds integer
                     ; a holds fraction
    mov   r4, b      ; save integer in r4
    mov   r5, a      ; save fraction in r5

    mov   a, r4      ; mov integer part
    lcall binbcd     ; convert to bcd
    clr   eal        ; hold interrupts while talking to LCD
    lcall shwLCDbcd  ; display it
    setb  eal        ; enable interrupts again
    mov   r0, #'.'   ; print the radix
    clr   eal        ; hold interrupts while talking to LCD
    lcall wrLCDdata4 ; display radix
    setb  eal        ; enable interrupts again
    mov   a, r5      ; mov fraction part
    lcall percent    ; convert to decimal fraction
    clr   eal        ; hold interrupts while talking to LCD
    lcall shwLCDbcd  ; display it
    setb  eal        ; enable interrupts again

    mov   r0, #analog0
    mov   a, @r0
    lcall shwLCDbar  ; show the bar

    lcall inkey      ; check for control-c (break)
    cjne  a, #3h, begin

    clr   eal        ; disable all interrupts

    lcall initLCD4   ; initialize fonts, #lines, Entry Mode
    lcall print
    db    "initLCD completed",0
    lcall crlf
    lcall waitKey    ; wait for a key press

    ljmp  os_return  ; back to monitor

; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; Analg-to-Digital Conversion subroutines
; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; ====================================================
;  subroutine initadc - initialize adc
;  input    : none
;  output   : initializes adc interrupt
;             to continuously read analog 0
;  destroys : a, dptr
; ----------------------------------------------------
initadc:
        clr   eadc        ; disable adc interrupts
        clr   iadc        ; reset adc interrupt flag
        mov   a, #6       ; sixth interrupt source
        mov   dptr, #adcisr ; setup interrupt vector
        lcall setintvec
        setb  eadc        ; adc interrupt mask bit

        mov   a, adcon    ; read adc control register
        anl   a, #0d0h    ; select channel 0, single shot
        mov   adcon, a
        mov   dapr, #0    ; start conversion
        ret
; ====================================================
;  subroutine adcisr
;  input    : adc interrupt
;  output   : continuously obtains and stores analog 0
;             value in internal ram.
;  destroys : nothing - uses a, r0, and r1
; ----------------------------------------------------
adcisr:
        push  psw
        mov   psw, #0        ; select register bank 0
        push  acc
        push  0              ; push register 0

        mov   r0,  #analog0  ; analog 0 store address
        mov   a,   addat     ; read analog value
        mov   @r0, a         ; store analog value
        mov   a, adcon       ; adc control register
        anl   a, #0d0h
        mov   adcon, a       ; select next channel
        clr   iadc           ; reset adc interrupt flag
        mov   dapr, #0       ; start next conversion

        pop   0
        pop   acc
        pop   psw
        reti

; ====================================================
; subroutine shwLCDbcd - shows a bcd number on the LCD
;  input    : bcd number in the accumulator
;  output   : none
;  destroys : a, r0
; ----------------------------------------------------
shwLCDbcd:
        push  acc        ; save the bcd number
        swap  a
        anl   a, #0fh    ; mask off high digit
        add   a, #30h    ; convert to ASCII
        mov   r0, a
        lcall wrLCDdata4 ; display it

        pop   acc        ; repeat for low digit
        anl   a, #0fh
        add   a, #30h
        mov   r0, a
        lcall wrLCDdata4
        ret

; ====================================================
; subroutine setBarChrc - sets the user-defined
; character N for N=1 to 5 to be the character
; shouing only the first N columns.
;
; this routine must be called before using shwLCDbar
;  input    : none
;  output   : none
;  destroys : a, b
; ----------------------------------------------------
setBarChars:
; first set the CGRAM address, then download the
; font patterns.  characters 1 to 5 are defined
; successively.

    mov   a, #1     ; select character 0
    mov   b, #0     ; select font row 0
    lcall setCGRAM4
    lcall prtLCD4
    db    10h, 10h, 10h, 10h, 10h, 10h, 10h, 20h    ; Char 1
    db    18h, 18h, 18h, 18h, 18h, 18h, 18h, 20h    ; Char 2
    db    1Ch, 1Ch, 1Ch, 1Ch, 1Ch, 1Ch, 1Ch, 20h    ; Char 3
    db    1Eh, 1Eh, 1Eh, 1Eh, 1Eh, 1EH, 1Eh, 20h    ; Char 4
    db    1Fh, 1Fh, 1Fh, 1Fh, 1Fh, 1Fh, 1Fh, 20h, 0 ; Char 5
    ret


; ====================================================
; subroutine shwLCDbar - displays a bar on line 2 of
; the LCD.  the length of the bar is proportional to
; the value of the accumulator.
;
; the user-defined characters 1 to 5 must first be
; programmed by setBarChrs
;
; LCDwidth must be defined by a EQU pseudo op
;
;  input    : binary number in the accumulator
;  output   : none
;  destroys : a, b, r4, r5, r6
; ----------------------------------------------------
shwLCDbar:
    push  acc        ; save binary value
    mov   a, #2      ; row 2
    mov   b, #0      ; location 0
    clr   eal        ; hold interrupts while talking to LCD
    lcall placeCur4  ; place cursor at line 1, position 13
    setb  eal        ; enable interrupts again

    pop   acc        ; recall binary value
    mov   b, #LCDwidth ; full scale in characters
    mul   ab         ; b holds integer
                     ; a holds fraction
    mov   r4, b      ; save integer in r4
    mov   r6, b      ; character count
    mov   r5, a      ; save fraction in r5

    mov   a, r4      ; mov integer part
    mov   r0, #5     ; full char with 5 columns showing
sBar1:
    jz    sBar2      ; no full character if r4=0
    push  acc        ; save count
    clr   eal        ; hold interrupts
    lcall wrLCDdata4 ; display character
    setb  eal        ; enable interrupts
    pop   acc        ; recall count
    dec   acc        ; update count
    sjmp  sBar1      ; repeat
sBar2:
    mov   a, r5      ; the fraction part
    lcall percent    ; convert to decimal fraction
    ; there are 5 columns in the font matrix.  the most
    ; significant digit of the fraction is devided by 2
    ; to compute the number of columns to display
    anl   a, #0F0h   ; mask Less Significant Nibble
    swap  a          ; swap nibbles
    clr   c          ; clear carry bit
    rrc   a          ; divide by 2
    ; the accumulator now holds the number of columns
    ; to display.  character N for N=1,2,3,4,5 must be defined
    ; to be the characters with the first N columns showing.

    jz    sBar3      ; skip if # of columns to display is 0
    inc   r6         ; update # of chars written
    mov   r0, a
    clr   eal        ; hold interrupts while talking to LCD
    lcall wrLCDdata4 ; display it
    setb  eal        ; enable interrupts again

sBar3:
 ; now let us fill the rest of the line with spaces
    mov   a, #LCDwidth
    clr   c          ; clear carry flag
    subb  a, r6      ; spaces to write
    jz    sBar5      ; skip if 0

    mov   r6, a      ; save in a
    mov   r0, #' '   ; the space character
sBar4:
    clr   eal        ; hold interrupts while talking to LCD
    lcall wrLCDdata4 ; display it
    setb  eal        ; enable interrupts again
    djnz  r6, sBar4  ; repeat
sBar5:
    ret

; ====================================================
; subroutine binbcd - converts the binary number in
;                     the accumulator to bcd.  the
;                     errorf is set if number is > 99
;                        using the 2 digital displays
;  input    : hexadecimal number in the accumulator
;  output   : bcd number in accumulator
;             errorf=1 if number>99
;  uses     : a, b
;  destroys : a
; ----------------------------------------------------
binbcd:
        push  b
        clr   errorf
        mov   b, #100 ; to divide n by 100
        div   ab
        jz    binbcd1 ; if n>99...
        setb  errorf  ; ...set errorf
binbcd1:
        mov   a, #10  ; to divide n by 10
        xch   a, b
        div   ab
        swap  a       ; move 10s into high nibble
        add   a, b    ; move 1s  into low nibble
        pop   b
        ret

; ====================================================
;

; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; LCD subroutines
; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

; waitKey is a utility that prints the message
; "press a key to continue" and then waits until
; a character is received.
waitKey:
    lcall print
    db    "press a key to continue", 0
    lcall crlf
    lcall getchr       ; wait for a key press
    ret

; ==========================================================
; subroutine placeCur4
; this routine sets the cursor position.  the cursor position
; is placed in the b register.  the cursor position starts
; at 0.  the accumulator gives the line number.
; input  : accumulator indicates line number (1, 2)
;        : b register holds the cursor position
; output : none
; ==========================================================
;
placeCur4:
         dec  acc          ; acc=0 for line=1
         jnz  line2        ; if acc=0 then first line
         mov  a, b
         add  a, #080h     ; construct control word for line 1
         sjmp setcur
line2:
         mov  a, b
         add  a, #0C0h     ; construct control word for line 2
setcur:
         mov  r0, a        ; place control word
         lcall wrLCDcom4   ; issue command
         ret

; ==========================================================
; subroutine setCGRAM4
; this routine sets the Character Generator RAM address.  the
; character [0..7] is placed in the accumulator.  the font row
; is placed in the b register.  the instructions 40h to 7Fh
; selects the CGRAM addresses 0 to 3Fh (0 to 63 decimal).
;
; the 5x7 character font is assumed.  that is, each character
; location contains a matrix of 40 dots arranged in 8 rows of
; 5 dots each.  the 7 rows are for the characters and the 8-th
; row for the underline cursor.  CGRAM address 0 corresponds
; to the first (font) row of character 0, address 7 corresponds
; to the 8-th (last) row of character 0, and address 3Fh
; corresponds to the 8-th row of character 7.
;
; input  : accumulator indicates character code [0..7]
;        : b register holds font row [0..7]
; output : none
; ==========================================================
;
setCGRAM4:
         push  b
         mov   b, #8
         mul   ab           ; multiply character number by 8
         pop   b            ; b holds row number
         add   a, b         ; a holds CGRAM address
         add   a, #40h      ; convert to instruction

         mov   r0, a        ; place instruction
         lcall wrLCDcom4    ; issue command
         ret

; ====================================================
; subroutine wrLCDdata4
; write a data word to the LCD
;
; data must be placed in r0 by calling program
; ----------------------------------------------------
wrLCDdata4:
    clr   LCD_E
    setb  LCD_RS        ; select send Data
    clr   LCD_RW        ; select write operation
    push  acc           ; save accumulator
    mov   a, r0         ; put data byte in acc

    mov   c, acc.4      ; load high nibble on data bus
    mov   LCD_DB4, c    ; one bit at a time using...
    mov   c, acc.5      ; bit move operatoins
    mov   LCD_DB5, c
    mov   c, acc.6
    mov   LCD_DB6, c
    mov   c, acc.7
    mov   LCD_DB7, c

    setb  LCD_E         ; pulse the Enable line
    clr   LCD_E

    mov   c, acc.0      ; similarly, load low nibble
    mov   LCD_DB4, c
    mov   c, acc.1
    mov   LCD_DB5, c
    mov   c, acc.2
    mov   LCD_DB6, c
    mov   c, acc.3
    mov   LCD_DB7, c

    lcall pulseEwait4   ; pulse the Enable line...
                        ; and wait for Busy Flag to clear
    pop   acc
    ret

; ====================================================
; subroutine wrLCDcom4
; write a command word to the LCD
;
; command must be placed in r0 by calling program
; ----------------------------------------------------
wrLCDcom4:
    clr   LCD_E
    clr   LCD_RS        ; select send Command
    clr   LCD_RW        ; select Write operation
    push  acc           ; save accumulator
    mov   a, r0         ; put data byte in acc

    mov   c, acc.4      ; load high nibble on data bus
    mov   LCD_DB4, c    ; one bit at a time using...
    mov   c, acc.5      ; bit move operatoins
    mov   LCD_DB5, c
    mov   c, acc.6
    mov   LCD_DB6, c
    mov   c, acc.7
    mov   LCD_DB7, c

    setb  LCD_E         ; pulse the Enable line
    clr   LCD_E

    mov   c, acc.0      ; similarly, load low nibble
    mov   LCD_DB4, c
    mov   c, acc.1
    mov   LCD_DB5, c
    mov   c, acc.2
    mov   LCD_DB6, c
    mov   c, acc.3
    mov   LCD_DB7, c

    lcall pulseEwait4   ; pulse the Enable line...
                        ; and wait for Busy Flag to clear
    pop   acc
    ret

; ====================================================
; subroutine pulseEwait4
; generates a positive pulse on the LCD enable line.
; waits for the Busy Flag to clear before returning.
; input    : none
; output   : none
; destroys : LCD_RW, LCD_RS, LCD_DATA
;
; ----------------------------------------------------
pulseEwait4:
    clr   LCD_E
    setb  LCD_E           ; pulse the Enable line
    clr   LCD_E

    mov   LCD_DATA, #0ffh ; prepare port for input
    setb  LCD_RW          ; prepare R/W for read operation

    push  acc             ; save accumulator contents
pEw:
    setb  LCD_E           ; start the Enable pulse
    mov   a, LCD_DATA     ; read status nibble
    clr   LCD_E           ; end Enable pulse
    setb  LCD_E           ; pretend reading status low nibble
    clr   LCD_E
    jb    acc.7, pEw      ; loop while Busy Flag is set
    pop   acc             ; restore accumulator
    ret

; ====================================================
; subroutine initLCD4 - initialize the LCD
;
; ----------------------------------------------------
initLCD4:
    clr LCD_RS      ; LCD Register Select line
    clr LCD_RW      ; Read / Write line
    clr LCD_E       ; Enable line

    mov   r0, #Config ; Function Set - data bits, lines, fonts
    lcall wrLCDcom4

    mov   r0, #onDsp     ; display ON
    lcall wrLCDcom4

    mov   r0, #entryMode ; set Entry Mode
    lcall wrLCDcom4 ; increment cursor to right, no display shift

    mov   r0, #clrDsp  ; clear display, home cursor
    lcall wrLCDcom4

    ret

; ====================================================
; subroutine resetLCD4 - reset the LCD
; software version of the power on reset operation
;
; ----------------------------------------------------
resetLCD4:
    clr   LCD_RS          ; LCD Register Select line
    clr   LCD_RW          ; Read / Write line
    clr   LCD_E           ; Enable line

    clr   LCD_DB7         ; set bit pattern for...
    clr   LCD_DB6         ; ... power-on-reset
    setb  LCD_DB5
    setb  LCD_DB4
                          ; step 1
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #4           ; delay 4 milliseconds
    lcall mdelay
                          ; step 2
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #1           ; delay 1 millisecond
    lcall mdelay
                          ; step 3
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #1           ; delay 1 millisecond
    lcall mdelay

    clr   LCD_DB4         ; specify 4-bit operation
                          ; step 4
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #1           ; delay 1 millisecond
    lcall mdelay

    mov   r0, #Config     ; FUNCTION SET
    lcall wrLCDcom4

    mov   r0, #08h        ; display off
    lcall wrLCDcom4

    mov   r0, #1          ; clear display, home cursor
    lcall wrLCDcom4

    mov   r0, #entryMode  ; set ENTRY MODE
    lcall wrLCDcom4

    ret

; ==========================================================
; subroutine prtLCD4
; takes the string immediately following the call and
; displays on the LCD.  the string is read with the
;   mov a, @a+dptr
; instruction.  thus, the string is in data memory.  since
; the string immediately follows code, the microcontroller
; is used with overlapping code and data memories (the von
; Newman architecture).  string must be terminated with a
; null (0).  upon termination this routine jumps to the
; instruction immediately following the string.
;
; input    : none
; output   : none
; destroys : acc, dptr
; ==========================================================
;
prtLCD4:
         pop   dph          ; pop return address into dptr
         pop   dpl
prtNext:
         clr  a             ; set offset = 0
         movc a,  @a+dptr   ; get chr from code memory
         cjne a, #0, chrOK  ; if chr = 0 then return
         sjmp retPrtLCD
chrOK:
         mov   r0, a
         lcall wrLCDdata4   ; send character
         inc   dptr         ; point at next character
         ljmp  prtNext      ; loop till end of string
retPrtLCD:
         mov   a,  #1h      ; point to instruction after string
         jmp   @a+dptr      ; return with a jump instruction

; ====================================================
; end of program
