; ----------------------------------------------------
; LCD8.asm      Last Update:8/23/91
;
; LCD Display Demonstration Program
;
; 8-bit operation
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD connections
;
; constants
 LCD_DATA  equ 090h ; Port 1 is used for data
                    ; connect P1.0-P1.7 to LCD DB0-DB7
 LCD_RS    equ 0e8h ; p4.0 LCD RS (Register Select) line
 LCD_RW    equ 0e9h ; p4.1 LCD R/W (Read/Write) line
 LCD_E     equ 0eah ; p4.2 LCD E (Enable) line

; system instructions
 Config    equ 38h  ; 2 lines, 5 by 7 character matrix
 entryMode equ 6    ; increment cursor, do not shift display

; cursor control instructions
 offCur    equ  0Ch
 lineCur   equ  0Eh
 blinkCur  equ  0Dh
 combnCur  equ  0Fh
 homeCur   equ  02h
 shLfCur   equ  10h
 shRtCur   equ  14h

; display control instructions
 clrDsp    equ  01h
 offDsp    equ  0Ah
 onDsp     equ  0Eh
 shLfDsp   equ  18h
 shRtDsp   equ  1Ch

; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------
    org 8000h

; initialize the LCD module --------------------------
    lcall resetLCD     ; duplicate power on reset
    lcall initLCD      ; initialize fonts, #lines, Entry Mode

    lcall waitKey      ; wait for a key press

; print a message ------------------------------------
    lcall prtLCD
    db "Hello World !", 0

    lcall waitKey      ; wait for a key press

; blank (quench) the display - retain contents -------
    mov   r0, #offDsp
    lcall wrLCDcom

    lcall waitKey      ; wait for a key press

; turn display back on -------------------------------
    mov   r0, #onDsp
    lcall wrLCDcom

    lcall waitKey      ; wait for a key press

; ----------------------------------------------------
; place cursor on line 2, position 7 and print a message
    mov   a, #2        ; line 2
    mov   b, #7        ; position 7
    lcall placeCur

    lcall prtLCD       ; print message
    db "line 2", 0

    lcall waitKey      ; wait for a key press

; clear display - contents are erased ----------------
    mov   r0, #clrDsp
    lcall wrLCDcom

    lcall waitKey      ; wait for a key press

; now let us define some custom characters -----------
; first set the CGRAM address, then download the
; font patterns.  characters 0 and 1 are defined
; successively.  character 0 is a Box, and character
; 1 is a Diamond.  when the font row is blank the
; pattern mask 20h is sent.  note that the last 5
; bits of 20h are 0.  this is to avoid sending 0,
; which would signal the end of the data string.

    mov   a, #0     ; select character 0
    mov   b, #0     ; select font row 0
    lcall setCGRAM
    lcall prtLCD
    db    1Fh, 11h, 11h, 11h, 11h, 11h, 1Fh, 20h    ; Box
    db    20h, 04h, 0Eh, 1Fh, 0Eh, 04h, 20h, 20h, 0 ; Diamond

; clearing the display selects the DDRAM location 0
; (homes the cursor) in preperation to display
; messages.
    mov   r0, #clrDsp
    lcall wrLCDcom
; print message.  since 0 implies end of string,
; use character 8.  note that characters 0..7
; are the same as characters 8..Fh.
    lcall prtLCD              ; print message
    db    "     Box : ", 8, 0
; move cursor to line 2
    mov   a, #2               ; line 2
    mov   b, #0               ; position 0
    lcall placeCur

    lcall prtLCD              ; print message
    db    " Diamond : ", 1, 0

    lcall waitKey      ; wait for a key press

; now let us shift the display - first let us write
; a long message

    mov   r0, #clrDsp  ; clear display
    lcall wrLCDcom

; now write a long message ---------------------------
; the following message is 40 characters long,
; the capacity of each line.  note that, since
; the cursor is incremented after each character,
; the cursor overflows onto the next line
    lcall prtLCD       ; write long message
    db    "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcd",0

    lcall waitKey      ; wait for a key press

; bring the cursor to the beginning of the top line --
    mov   r0, #homeCur
    lcall wrLCDcom

    lcall waitKey      ; wait for a key press

; let us shift the display ---------------------------
    mov   a, #1        ; shift 1 notch
    lcall dspShRt      ; shift

    lcall waitKey      ; wait for a key press

    mov   a, #9        ; shift 9 more notches
    lcall dspShRt      ; shift

    lcall waitKey      ; wait for a key press

; now let us shift the display to the left past origin
    mov   a, #20       ; shift 20 notches
    lcall dspShLf      ; shift past origin

    lcall waitKey      ; wait for a key press

; finally shift the display back to the origin
    mov   a, #10       ; shift 10 notches
    lcall dspShRt      ; shift to origin

    lcall waitKey      ; wait for a key press

; shift the cursor to the right
    mov   a, #1        ; shift 1 notch
    lcall crShRt       ; shift right

    lcall waitKey      ; wait for a key press

; shift the cursor to the left 2 notches, past the origin.
; thus, the cursor will be out of the viewing range.
    mov   a, #2        ; shift 2 notches
    lcall crShLf       ; shift past origin

    lcall waitKey      ; wait for a key press

; bring the cursor to the beginning of the top line --
    mov   r0, #homeCur
    lcall wrLCDcom


    lcall crlf
    lcall print
    db    "program terminates... press ALT-F4", 0

    ljmp  os_return
; end of main program

; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; subroutines
; ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

; waitKey is a utility that prints the message
; "press a key to continue" and then waits until
; a character is received.
waitKey:
    lcall print
    db    "press a key to continue", 0
    lcall crlf
    lcall getchr       ; wait for a key press
    ret

; ==========================================================
; subroutine crShLf
; this routine shifts the cursor to the left.  the number of
; positions to be shifted is is placed in the accumulator.
; input  : accumulator indicates number of notches to shift
; output : none
; ==========================================================
;
crShLf:
         jz    ret_scl
         mov   r0, #shLfCur ; control word for left shift
         lcall wrLCDcom
         dec   acc
         sjmp  crShLf
ret_scl: ret

; ==========================================================
; subroutine crShRt
; this routine shifts the cursor to the right.  the number of
; positions to be shifted is is placed in the accumulator.
; input  : accumulator indicates number of notches to shift
; output : none
; ==========================================================
;
crShRt:
         jz    ret_scr
         mov   r0, #shRtCur ; control word for right shift
         lcall wrLCDcom
         dec   acc
         sjmp  crShRt
ret_scr: ret

; ==========================================================
; subroutine placeCur
; this routine sets the cursor position.  the cursor position
; is placed in the b register.  the cursor position starts
; at 0.  the accumulator gives the line number.
; input  : accumulator indicates line number (1, 2)
;        : b register holds the cursor position
; output : none
; ==========================================================
;
placeCur:
         dec  acc          ; acc=0 for line=1
         jnz  line2        ; if acc=0 then first line
         mov  a, b
         add  a, #080h     ; construct control word for line 1
         sjmp setcur
line2:
         mov  a, b
         add  a, #0C0h     ; construct control word for line 2
setcur:
         mov  r0, a        ; place control word
         lcall wrLCDcom    ; issue command
         ret

; ==========================================================
; subroutine setCGRAM
; this routine sets the Character Generator RAM address.  the
; character [0..7] is placed in the accumulator.  the font row
; is placed in the b register.  the instructions 40h to 7Fh
; selects the CGRAM addresses 0 to 3Fh (0 to 63 decimal).
;
; the 5x7 character font is assumed.  that is, each character
; location contains a matrix of 40 dots arranged in 8 rows of
; 5 dots each.  the 7 rows are for the characters and the 8-th
; row for the underline cursor.  CGRAM address 0 corresponds
; to the first (font) row of character 0, address 7 corresponds
; to the 8-th (last) row of character 0, and address 3Fh
; corresponds to the 8-th row of character 7.
;
; input  : accumulator indicates character code [0..7]
;        : b register holds font row [0..7]
; output : none
; ==========================================================
;
setCGRAM:
         push b
         mov  b, #8
         mul  ab           ; multiply character number by 8
         pop  b            ; b holds row number
         add  a, b         ; a holds CGRAM address
         add  a, #40h      ; convert to instruction

         mov  r0, a        ; place instruction
         lcall wrLCDcom    ; issue command
         ret

; ==========================================================
; subroutine dspShLf
; this routine shifts the LCD contents to the left.  the
; number of characters to be shifted is is placed in the
; accumulator.
; input  : accumulator indicates number of notches to shift
; output : none
; ==========================================================
;
dspShLf:
         jz    ret_sdl
         mov   r0, #shLfDsp ; control word for left shift
         lcall wrLCDcom
         dec   a
         sjmp  dspShLf
ret_sdl: ret

; ==========================================================
; subroutine dspShRt
; this routine shifts the LCD contents to the right.  the
; number of characters to be shifted is is placed in the
; accumulator.
; input  : accumulator indicates number of notches to shift
; output : none
; ==========================================================
;
dspShRt:
         jz    ret_sdr
         mov   r0, #shRtDsp ; control word for right shift
         lcall wrLCDcom
         dec   a
         sjmp  dspShRt
ret_sdr: ret

; ====================================================
; subroutine wrLCDdata
; write a data word to the LCD
;
; data must be placed in r0 by calling program
; ----------------------------------------------------
wrLCDdata:
    clr   LCD_E
    setb  LCD_RS        ; select send Data
    clr   LCD_RW        ; select write operation
    mov   LCD_DATA, r0  ; load command into port
    lcall pulseEwait    ; pulse the Enable line
    ret

; ====================================================
; subroutine wrLCDcom
; write a command word to the LCD
;
; LCD_RS, LCD_RW, and r0 must be set by
;  the calling program
; ----------------------------------------------------
wrLCDcom:
    clr   LCD_E
    clr   LCD_RS        ; select send Command
    clr   LCD_RW        ; select Write operation
    mov   LCD_DATA, r0  ; load command into port
    lcall pulseEwait    ; pulse the Enable line
    ret

; ====================================================
; subroutine rdLCDstat
; reads the status byte (Busy Flag and Address Couter)
; the status byte is returned in the accumulator
; input  : none
; output : status byte in accumulator
; ----------------------------------------------------
rdLCDstat:
    mov   r0, #0ffh ; prepare port for input
    clr   LCD_E
    clr   LCD_RS
    setb  LCD_RW
    setb  LCD_E           ; pulse the Enable line
    mov   a, r0
    clr   LCD_E
    ret

; ====================================================
; subroutine pulseEwait
; generates a positive pulse on the LCD enable line.
; waits for the Busy Flag to clear before returning.
; input    : none
; output   : none
; destroys : LCD_RW, LCD_RS, LCD_DATA
;
; ----------------------------------------------------
pulseEwait:
    clr   LCD_E
    setb  LCD_E           ; pulse the Enable line
    clr   LCD_E

    mov   LCD_DATA, #0ffh ; prepare port for input
    setb  LCD_RW          ; prepare R/W for read operation

    push  acc             ; save accumulator contents
pEw:
    inc   b
    setb  LCD_E           ; start the Enable pulse
    mov   a, LCD_DATA     ; read status
    clr   LCD_E           ; end Enable pulse
    jb    acc.7, pEw      ; loop while Busy Flag is set
    pop   acc             ; restore accumulator
    ret

; ====================================================
; subroutine initLCD - initialize the LCD
;
; ----------------------------------------------------
initLCD:
    clr LCD_RS      ; LCD Register Select line
    clr LCD_RW      ; Read / Write line
    clr LCD_E       ; Enable line

    mov   r0, #38h  ; Function Set - 2 lines 5x7 fonts
    lcall wrLCDcom

    mov   r0, #0Eh  ; display ON
    lcall wrLCDcom

    mov   r0, #06h  ; set Entry Mode
    lcall wrLCDcom  ; increment cursor to right, no display shift

    mov   r0, #01h  ; clear display, home cursor
    lcall wrLCDcom

    ret

; ====================================================
; subroutine resetLCD - reset the LCD
; software version of the power on reset operation
;
; ----------------------------------------------------
resetLCD:
    clr LCD_RS            ; LCD Register Select line
    clr LCD_RW            ; Read / Write line
    clr LCD_E             ; Enable line

    mov   r0, #30h        ; step 1
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #4           ; delay 4 milliseconds
    lcall delay

    mov   LCD_DATA, #30h  ; step 2
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #1           ; delay 1 millisecond
    lcall delay

    mov   LCD_DATA, #30h  ; step 3
    setb  LCD_E           ; start Enable pulse
    clr   LCD_E           ; end Enable pulse
    mov   a, #1           ; delay 1 millisecond
    lcall delay

    mov   r0, #Config     ; FUNCTION SET
    lcall wrLCDcom

    mov   r0, #08h        ; display off
    lcall wrLCDcom

    mov   r0, #01h        ; clear display, home cursor
    lcall wrLCDcom

    mov   r0, #entryMode  ; set ENTRY MODE
    lcall wrLCDcom        ; increment cursor, do not shift display

    ret

; ==========================================================
; subroutine prtLCD
; takes the string immediately following the call and
; displays on the LCD.  the string is read with the
;   mov a, @a+dptr
; instruction.  thus, the string is in data memory.  since
; the string immediately follows code, the microcontroller
; is used with overlapping code and data memories (the von
; Newman architecture).  string must be terminated with a
; null (0).  upon termination this routine jumps to the
; instruction immediately following the string.
;
; input    : none
; output   : none
; destroys : acc, dptr
; ==========================================================
;
prtLCD:
         pop   dph          ; pop return address into dptr
         pop   dpl
prtNext:
         clr  a             ; set offset = 0
         movc a,  @a+dptr   ; get chr from code memory
         cjne a, #0, chrOK  ; if chr = 0 then return
         sjmp retPrtLCD
chrOK:
         mov   r0, a
         lcall wrLCDdata    ; send character
         inc   dptr         ; point at next character
         ljmp  prtNext      ; loop till end of string
retPrtLCD:
         mov   a,  #1h      ; point to instruction after string
         jmp   @a+dptr      ; return with a jump instruction

; ====================================================
; end of program
