
; all macros arguments are destination,source with high byte first

isgt	equ	1
islt	equ	2
isge	equ	3
isle	equ	4
iseq	equ	5

lc1	set	100
lc2	set	200
lc3	set	300

dsc_byte	equ	1
dsc_word	equ	2
dsc_long	equ	4
dsc_quad	equ	8
dsc_string	equ	16
dsc_warray	equ	32
dsc_larray	equ	64
; words are low longs so they line up for math with longs
word1	data	06h	; always reg base 0
word2	data	26h
word3	data	2Ah
word4	data	2Eh

w5h	equ	04h
w5l	equ	05h
w1h	equ	06h
w1l	equ	07h
w6h	equ	24h
w6l	equ	25h
w2h	equ	26h
w2l	equ	27h
w7h	equ	28h
w7l	equ	29h
w3h	equ	2Ah
w3l	equ	2Bh
w8h	equ	2Ch
w8l	equ	2Dh
w4h	equ	2Eh
w4l	equ	2Fh

long1	data	04h
long2	data	24h
long3	data	28h
long4	data	2ch

preamble	equ	4
word	macro	arg
oadr	set $
	db	dsc_word	;; type
	db	0		;; class
	dw	2		;; size
arg:
	cseg at (oadr + preamble + dsc_word)
	endm

long	macro	arg
oadr	set $
	db	dsc_long	;; type
	db			;; class
	dw	4		;; size
arg:
	cseg at (oadr + preamble + dsc_long)
	endm

string	macro	arg,text
oadr	set $
	db	dsc_string	;; type
	db	0		;; class
	dw	slen		;; size
arg:
	db	text
	db	0
slen	set	$ - arg
	cseg at (oadr + preamble + slen)
	endm

warray	macro	arg,size
oadr	set $
alen	set size * dsc_word
	db	dsc_warray	;; type
	db	dsc_word	;; class
	dw	alen		;; size
arg:
	cseg at	(oadr + preamble + alen)
	endm

larray	macro	arg,size
oadr	set $
alen	set size * dsc_long
	db	dsc_larray	;; type
	db	dsc_long	;; class
	dw	alen		;; size
arg:
	cseg at	(oadr + preamble + alen)
	endm
;================================================================	
; add internal immediate
; increment internal word with constant
adi	macro	idst,constant
	mov	dptr,#constant
	mov	r0,#idst
	call	rtadi
	endm
;
; subtract internal immediate
; decrement internal word with constant
sdi	macro	idst,constant
	mov	dptr,#constant
	mov	r0,#idst
	call	rtsdi
	endm
;
; add external immediate
; increment/decrement external value with constant
axi	macro	xdst,constant
	ldx	word1,xdst
	ldi	word2,constant
	mov	dptr,#xdst-preamble
	call	rtaxi
	lxd	xdst,word2
	endm
;
; load data immediate
; load internal word with constant
ldi	macro	adst,constant
	mov	r0,#adst
	mov	dptr,#constant
	call	rtldi
	endm
;
; load external data
; move the contents of internal arg2 to external arg1		*
lxd	MACRO	ARG1,ARG2
	mov	r0,#arg2
	mov	dptr,#arg1-preamble
	call	rtlxd
	endm
;
; load external immediate
; load external word with constant				*	
lxi	macro	arg1,constant
	ldi	word1,constant		; load constant into internal
	lxd	arg1,word1		; load external with internal
	endm
;
; load data external
; move the contents of external ASRC to internal ADST
ldx	macro	adst,asrc
	mov	dptr,#asrc-preamble
	mov	r0,#adst
	call	rtldx
	endm
;
; duplicate the contents of internal src1 to internal dst1
ldd0	set 0
ldd	macro	dst1,src1
	mov	r0,#src1
	mov	r1,#dst1
	mov	r2,#4
	call	rtldd
	endm
;
; less than or equal	
ble	macro	arg1,arg2,addr
	ldx	word1,arg1
	ldx	word2,arg2
	call	rtcmp
	jb	Z,$+5		; is equal
	jnc	$+5		; is greater
	ljmp	addr
	endm
;
; less than
blt	macro	arg1,arg2,addr
	ldx	word1,arg1
	ldx	word2,arg2
	call	rtcmp
	jb	Z,$+8		; is equal
	jnc	$+5		; is greater
	ljmp	addr
	endm
;
; equal
beq	macro	arg1,arg2,addr
	ldx	word1,arg1
	ldx	word2,arg2
	call	rtcmp
	jnb	Z,$+6		; not equal
	ljmp	addr
	endm
;
; equal
bne	macro	arg1,arg2,addr
	ldx	word1,arg1
	ldx	word2,arg2
	call	rtcmp
	jb	Z,$+6		; is equal
	ljmp	addr
	endm
;
; greater	
bgt	macro	arg1,arg2,addr
	ldx	word1,arg1
	ldx	word2,arg2
	call	rtcmp
	jb	Z,$+9		; is equal
	jb	N,$+6		; is negative
	ljmp	addr
	endm
;
; greater or equal
bge	macro	arg1,arg2,addr
	ldx	word1,arg1
	ldx	word2,arg2
	call	rtcmp
	jc	$+8		; is less
	jb	N,$+6		; is negative
	ljmp	addr
	endm

; aname is base, aindex is index				*
array	macro	aname,aindex
	ldi	word1,aname
	ldx	word2,aindex
	endm
;
; load array immediate
; load array ANAME with CONSTANT
lai	macro	aname,aindex,constant
	ldi	word3,constant
	ldx	word2,aindex
	mov	dptr,#aname-preamble
	call	indexer			;; dptr has addr of indexed array
	mov	r0,#word3
	call	rtlxd2
	endm
;
; load array data
; load array ANAME with internal ARG1
lad	macro	aname,aindex,arg1
	pushl	arg1			;; save data
	ldx	word2,aindex
	mov	dptr,#aname-preamble
	call	indexer			;; dptr has addr of indexed array
	mov	b,r2
	popl	arg1
	mov	r0,#arg1
	mov	r2,b
	call	rtlxd2
	endm
;
; load data array
; load internal ARG1 from array ANAME
lda	macro	arg1,aname,aindex
	ldx	word2,aindex
	mov	dptr,#aname-preamble
	call	indexer			;; dptr has addr of indexed array
	mov	r0,#arg1
	call	rtldx2
	endm
;
for	macro	var1,argu,var2,con1
	sysfor	var1,argu,var2,con1, %lc1, %lc2
	endm
;
next	macro
	call	donext
	endm
;
sysfor	macro	var1,argu,var2,con1,lin1,lin2
	jmp	l&lin2
;; This is where 'next' comes to. No need to worry about
;; who comes first in nested for loops. The inner most
;; loop is first on stack. A lifo you might say. The 'next'
;; routine saves the address of where to go if condition fails.
l&lin1:
	iadd	var1,con1		;; bump step counter
	lxd	var1,word2

	if(argu=1)			;; greater than
	bgt	var1,var2,l&lin2
	endif
	if(argu=2)			;; less than
	blt	var1,var2,l&lin2
	endif
	if(argu=3)			;; greater than or equal
	bge	var1,var2,l&lin2
	endif
	if(argu=4)			;; less than or equal
	ble	var1,var2,l&lin2
	endif
;; We're here because the condition failed. The 'next'
;; macro called the 'next' function which places a return
;; address of where we need to go.
	ldx	word2,retadr		; get address after 'next' call
	push	w2l
	push	w2h
	ret				; for done, continue
;; We come here in the beginning and for each iteration
;; of the 'for' loop. We place a return address for the 'next'
;; function.
l&lin2:
	mov	a,#low l&lin1	; setup to return back here
	push	acc
	mov	a,#high l&lin1
	push	acc
;; internal counters for labels. The whole idea is not to have
;; to know where to go on in the 'for' loop.
lc1 set lc1 + 1
lc2 set lc2 + 1
	endm

; 	
pushb	macro	arg
	mov	r0,#arg
	mov	r2,#1
	call	rtpush
	endm
pushw	macro	arg
	mov	r0,#arg
	mov	r2,#2
	call	rtpush
	endm
pushl	macro	arg
	mov	r0,#arg
	mov	r2,#4
	call	rtpush
	endm
;
popb	macro	arg
	mov	r0,#arg
	mov	r2,#1
	call	rtpop
	endm
popw	macro	arg
	mov	r0,#arg
	mov	r2,#2
	call	rtpop
	endm
popl	macro	arg
	mov	r0,#arg
	mov	r2,#4
	call	rtpop
	endm
;
; integer subtraction
isub	macro	arg1,arg2
	ldx	word1,arg2
	ldx	word2,arg1
	call	rtisub
	endm
; 
; integer addition						*
iadd	macro	arg1,arg2
	ldx	word1,arg1
	ldx	word2,arg2
	call	rtiadd
	endm
;
sqr	macro	larg,warg
	ldd	word1,warg	;; multiplier
	ldd	word3,warg	;; multiplicand = multiplier
	mov	r0,#word2
	call	mbclr		;; init product low
	call	mbmul
	wtl	larg,word1,word2	;; convert to long
	endm
;
; integer multiplication					*
imul	macro	arg1,arg2
	mov	r0,#word2	; product_lo
	mov	r2,#2
	call	mbclr
	ldx	word3,arg1	; multiplicand
	ldx	word1,arg2	; multiplier
	mov	r2,#2
	call	mbmul
	endm
;
; integer division						*
idiv	macro	arg1,arg2
	mov	r0,#word2	; dividend_hi
	mov	r2,#2
	call	mbclr
	ldx	word3,arg1	; dividend_lo
	ldx	word1,arg2	; divisor
	mov	r2,#2
	call	mbdiv
	endm
;
iand	macro	arg1,arg2
	ldx	word1,arg2
	ldx	word2,arg1
	call	rtand
	endm
;
; 2 words high and low to 1 long
wtl	macro	larg,w1,w2
	mov	r2,#w1+1	; high word
	mov	r1,#w2+1	; low word
	mov	r0,#larg+3
	call	wtl_expand
	endm
;
; 1 long to 2 words high and low
ltw	macro	w1,w2,larg
	mov	r2,#w1
	mov	r1,#w2
	mov	r0,#larg
	call	wtl_reduce
	endm
;
argstk	macro	abas,aidx,isize
	if(isize=2)
	lda	word2,abas,aidx,isize
	pushw	word2
	endif
	if(isize=4)
	lda	long2,abas,aidx,isize
	pushl	long2
	endif
	endm
;
; compare external arg1 to external arg2
cmp	macro	arg1,arg2
	ldx	word1,arg1
	ldx	word2,arg2
	call	rtcmp
	endm
;
fixmpy	macro	var,aptr,aidx
	lda	word2,aptr,aidx,#2
	ldx	word1,var
	call	fix_mpy		; word1 = fix_mpy(vr,fr[j])
	endm
;
	
