        titl    'XT keyboard read and LCD write test'
        page    60
;
; Test program to read from a PC XT keyboard and write to an LCD
; display
;
; Written by Dennis S DeLorme     yuslord@millcomm.com
;
KeyClk  bit     P3.2            ;extrn interrupt 0
KeyData bit     P3.4            ;keyboard data
Nbits   equ     30h             ;number of bits processed
Nbytes  equ     31h             ;number of bytes processed
KeyTemp equ     32h             ;temp byte to build scan code
KeyRtn  equ     33h             ;char to return
shift   bit     0               ;shift flag
c_flags equ     21h         ;cursor flag byte
blink_on bit    (c_flags-20h)*8+0 ;blink state
cursor_on bit   (c_flags-20h)*8+1 ;cursor state
bitsave equ     22h             ;save area for data bit
;
;
        org     0
        ajmp    start           ;reset
        org     3
        ajmp    KeyBoard        ;extrn interrupt 0
        org     0bH
        ajmp    start           ;timer 0
        org     013h
        ajmp    start           ;extrn interrupt 1
        org     01bh
        ajmp    start           ;timer 1
        org     023h
        ajmp    start           ;Serial I/O
;
        org     40h
start
        mov     r0,#0ffh
        mov     r1,#0ffh
;
delay                           ;wait for settling time
        djnz    r0,$
        djnz    r1,delay
reset
        mov     P0,0ffh
        mov     P1,0ffh
        mov     P2,0ffh
        mov     P3,0ffh
;
        setb    IT0             ;ext int edge triggered
        clr     shift           ;clear shift flag
        mov     c_flags,#0
        mov     bitsave,#0
        mov     Nbits,#0
        mov     Nbytes,#0
        mov     KeyTemp,#0
        mov     KeyRtn,#0
;
        acall   LCD_Init        ;init LCD
        mov     dptr,#ok_msg    ;startup msg for LCD
        acall   LCD_PutSc
        setb    EX0             ;enable external interrrupt 0
        setb    EA              ;enable interrupts
loopit
        mov     r1,Nbytes       ;get byte count
        cjne    r1,#1,loopit    ;if no char keep looking
        mov     Nbytes,#0       ;reset count
        mov     a,KeyRtn        ;get char
        cjne    a,#08h,chkctrl
;
;   handle back space
        mov     R0,#LCD_READ_STS
        movx    a,@R0           ;get status and dd ram addr
        clr     acc.7           ;turn off busy flag
        jz      at_start        ;if not at start of display
        dec     a               ;back up 1 pos
at_start
        mov     R2,a            ;save in R2
        setb    acc.7           ;make set dd addr cmd
        acall   LCD_CmdWrite    ;set  dd ram addr
        mov     a,#20h          ;get a blank
        acall   LCD_DataWrite   ;erase current pos
        mov     a,R2            ;get dd ram addr
        setb    acc.7           ;set dd ram addr
        acall   LCD_CmdWrite    ;back up
        sjmp    loopit          ;get next char
chkctrl
        jc      loopit          ;control char
        cjne    a,#10h,not_home ;check for home
        mov     a,#2
        acall   LCD_CmdWrite
        sjmp    loopit
not_home
        cjne    a,#19h,lt_space ;rest of low control char
        sjmp    loopit
lt_space
        jc      loopit
        cjne    a,#80h,chk_FKeys        ;check function keys
chk_FKeys                       ;always come here code 80 not used
        jnc     FKeys           ;handle function keys
        acall   LCD_DataWrite   ;write char
        sjmp    loopit          ;go for next one
;
; function keys are encoded with the high bit on
;
FKeys
        cjne    a,#82h,FK1      ;handle F1
        cpl     cursor_on       ;toggle cursor bit
        sjmp    do_cursor
FK1
        jnc     loopit          ;only F1 and F2 handled
        cpl     blink_on        ;toggle blink bit
do_cursor
        mov     a,#0ch          ;display on
        orl     a,c_flags       ;set cursor state
        acall   LCD_CmdWrite
        sjmp    loopit
; *****************  end of main loop ****************
        titl    'LCD Display Routines'
        page    60

;
;*********** Routines to use an LCD display in memory mode ***********
;
; Since the LCD can't meet the timing for regular memory I/O a little
; kluge is used...
;
; Read and write are ORed together and used as the ENABLE
; A0 is used to select command or data 0=command,1=data
; A1 is used for write/read select 0=write, 1=read
;
; R0 is used for the I/O address
; single bye data is passed in the accumulator reg
; String data is passed as null terminated data pointed to by DPTR
; Up to 6 stack bytes may be used
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCD_READ_DATA   equ    11b
LCD_WRITE_DATA  equ    01b
LCD_READ_STS    equ    10b   ;status read is a comand operation
LCD_WRITE_CMD   equ    00b
; ;=============================================
;
;       Display the nul-terminated string stored in code ROM
;       on the LCD, starting at the current DD adress.
;       The string is pointed to by dptr
;
;
LCD_PutSc
    clr     a
    movc    a, @a+dptr                ; fetch the byte
    jz      LCD_PutScExit             ; exit on End Of String, Nul
    inc     dptr
    acall   LCD_DataWrite               ; display the byte
    sjmp    LCD_PutSc
LCD_PutScExit
    ret
;
;=============================================
;
;       Write the nul-terminated command string stored in code ROM
;       to the LCD.
;       The string is pointed to by dptr
;
LCD_PutCmdStr
    clr     a                           ;
    movc    a, @a+dptr                  ; fetch the byte
    jz      LCD_PutCmdExit              ; exit on End Of String, Nul
    inc     dptr                        ;
    acall   LCD_CmdWrite                ; Write the Command
    sjmp    LCD_PutCmdStr               ;
LCD_PutCmdExit:
    ret
;
;==============================================================================
;
;       Write the byte in ACC to the LCD Data port.  Before
;       writing, make sure the LCD is not busy.
;
LCD_DataWrite
    acall   LCD_BusyWait                ; wait for end of previous cycle
    mov     R0,#LCD_WRITE_DATA          ; invoke Write mode
    movx    @R0,a                       ; send display byte to data port
    ret
;
;==============================================================================
;
;       Write the byte in ACC to the LCD Command port.  Before
;       writing, make sure the LCD is not busy.
;
;
LCD_CmdWrite
    acall   LCD_BusyWait                ; wait for end of previous cycle
    mov     R0,#LCD_WRITE_CMD           ; invoke Write mode
    movx    @R0,a                       ; send command to display
    ret
;
;==============================================================================
;
;
;       Poll the LCD until BF goes False
;
LCD_BusyWait
    push    acc                         ;save acc
    mov     R0,#LCD_READ_STS            ; goto Read status mode
LCD_BusyLoop
    movx    a,@R0                       ; read data
    jb      ACC.7,LCD_BusyLoop         ; test Busy bit...
    pop     acc                         ;restore acc
    ret
;
;==============================================================================
;
;       Initialize the LCD panel:
;           8 - bit mode, single line, 5X10 char, display on,
;           cursor off, increment address
;
;
;
LCD_Init
    mov     R0,#LCD_WRITE_CMD          ; goto Command mode
    mov     a, #034h                   ; set 8-bit interface mode
    movx    @R0,a                      ; Write command
    acall   LCD_BusyWait
    mov     R0,#LCD_WRITE_CMD          ; goto Command mode
    mov     a, #034h                   ; set 8-bit interface mode
    movx    @R0,a                      ; Write command
    acall   LCD_BusyWait
    mov     R0,#LCD_WRITE_CMD          ; goto Command mode
    mov     a, #01h                    ; Clear display
    movx    @R0,a                      ; Write command
    acall   LCD_BusyWait
    mov     R0,#LCD_WRITE_CMD          ; goto Command mode
    mov     a, #06h                    ; entry mode - incr, no shift
    movx    @R0,a                      ; Write command
    acall   LCD_BusyWait
    mov     R0,#LCD_WRITE_CMD          ; goto Command mode
    mov     a, #0ch                    ; display on, cursor off
    movx    @R0,a                      ; Write command
    ret
; *************** end of LCD routines ****************
;
ok_msg  db      'OK So Far',0
;
        titl    'Keyboard interrupt handler'
        page    60
;
; ***************** Keyboard Routine **********************
;
; Read serial data from IBM XT keyboard
; Data read is returned in KeyRtn and Nbytes is set to 1
; when a data byte is available
;
;; Note - real IBM XT keyboards have 2 neg clocks for the start or
;        sync bit. The clock goes low before the sync bit goes
;        high followed by a second clock low while the sync is
;        high
;
;
;     ____     __    __    __    __    __    __    __    __    ___   _______
;clock    |___|  |__|  |__|  |__|  |__|  |__|  |__|  |__|  |__|  |__|
;
;           ________
;  data____|        |XXXXX|XXXXX|XXXXX|XXXXX|XXXXX|XXXXX|XXXXX|XXXXX|______
;           start      0     1     2     3     4     5     6     7
;                     lsb
;
; note that start data bit goes high after the clock goes low
; XX is the data bit and is valid when the clock is low, data changes
;    around the time the clock goes high.
; bit 7 = 0 when key is pressed, = 1 when key is released
;
;
; Some clones raise the sync bit before the clock and only have one neg
; clock for the sync bit. For example one clone keeps the data hi between
; keystrokes and has the following pattern. Note that there are only 9 clock
; cycles.
;     ________       __    __    __    __    __    __    __    ___   _______
;clock        |_____|  |__|  |__|  |__|  |__|  |__|  |__|  |__|  |__|
;
;      _____________                                                 _______
;  data             |XXXXX|XXXXX|XXXXX|XXXXX|XXXXX|XXXXX|XXXXX|XXXXX|
;           start      0     1     2     3     4     5     6     7
;                     lsb

;
; Either will work with the XT hardware shift register interface
;
; This software interface is less forgiving!
;
; You should be sure which type you have! If it is like the clone above 
; adjust the following code to not skip the second clock and only count
; 9 bits. Note the count starts a zero!
;
;*******************************************************
KeyBoard
        push    psw             ; save psw so we can use carry bit
        mov     c,KeyData       ; get data bit
;
; We now have the data bit and have 10s of usec to process it.
; By getting the data as soon as possible we have until the next clock
; low to finish processing the data bit. This can be important if the 
; keyboard clock has a short low state and/or a processor clock slower 
; than 10MHz.
;
        mov     bitsave.7,c     ;save for later
        push    acc             ; now save acc so we can use it
        mov     a,Nbits
        cjne    a,#1,bit_1      ;if this is second time
        sjmp    nextb           ;ignore
bit_1
        jc      nextb           ;also ignore if first
;
; The real bits
;
        mov     a,KeyTemp       ;get key in progress
        rr      a               ;move old bits right
        orl     a,bitsave       ;or in next bit
        mov     KeyTemp,a       ;store key so far
        mov     a,Nbits
        cjne    a,#9,nextb      ;do we have the whole byte ?
;
; Now have all 8 bits of the scan code - figure out what we have
;
        mov     a,KeyTemp       ;get scan code
        cjne    a,#2ah,notlson  ;is it make left shift?
        setb    shift           ;yes - set shift
        sjmp    cleanup
notlson
        cjne    a,#0aah,notlsoff  ;is it break left shift?
        clr     shift           ;yes clear shift
        sjmp    cleanup
;
; process key
;
notlsoff
        jb      acc.7,cleanup   ;break key code - ignore
        push    dph
        push    dpl
        jb      shift,shiftlook ;check shift state
        mov     dptr,#unshifted ;get unshifted table
        sjmp    skip
shiftlook
        mov     dptr,#shifted   ;get shifted table
skip
        movc    a,@a+dptr       ;translate scan code
        pop     dpl
        pop     dph
        cjne    a,#0,not0       ;check for unsupported key
        sjmp    cleanup         ;skip if unsupported
not0
        mov     KeyRtn,a        ;return char
        mov     Nbytes,#1       ;char to process
cleanup
        mov     Nbits,#0        ;clear # bits
        mov     KeyTemp,#0      ;clear temp byte
        sjmp    intdone
nextb
        inc     Nbits
intdone
        pop     acc
        pop     psw
        reti
unshifted
        db      0               ;scan code 0 not used
        db      0               ;scan code 1 - esc
        db      '1'             ;scan code 2
        db      '2'             ;scan code 3
        db      '3'             ;scan code 4
        db      '4'             ;scan code 5
        db      '5'             ;scan code 6
        db      '6'             ;scan code 7
        db      '7'             ;scan code 8
        db      '8'             ;scan code 9
        db      '9'             ;scan code 10
        db      '0'             ;scan code 11
        db      '-'             ;scan code 12
        db      '='             ;scan code 13
        db      08h             ;scan code 14 - back space
        db      09h             ;scan code 15 - tab
        db      'q'             ;scan code 16
        db      'w'             ;scan code 17
        db      'e'             ;scan code 18
        db      'r'             ;scan code 19
        db      't'             ;scan code 20
        db      'y'             ;scan code 21
        db      'u'             ;scan code 22
        db      'i'             ;scan code 23
        db      'o'             ;scan code 24
        db      'p'             ;scan code 25
        db      '['             ;scan code 26
        db      ']'             ;scan code 27
        db      0dh             ;scan code 28 - return
        db      0               ;scan code 29 - control
        db      'a'             ;scan code 30
        db      's'             ;scan code 31
        db      'd'             ;scan code 32
        db      'f'             ;scan code 33
        db      'g'             ;scan code 34
        db      'h'             ;scan code 35
        db      'j'             ;scan code 36
        db      'k'             ;scan code 37
        db      'l'             ;scan code 38
        db      ';'             ;scan code 39
        db      "'"             ;scan code 40
        db      '`'             ;scan code 41
        db      0               ;scan code 42 - left shift
        db      '\'             ;scan code 43
        db      'z'             ;scan code 44
        db      'x'             ;scan code 45
        db      'c'             ;scan code 46
        db      'v'             ;scan code 47
        db      'b'             ;scan code 48
        db      'n'             ;scan code 49
        db      'm'             ;scan code 50
        db      ','             ;scan code 56
        db      '.'             ;scan code 52
        db      '/'             ;scan code 53
        db      0               ;scan code 54 - right shift
        db      '*'             ;scan code 55 - prt scrn
        db      0               ;scan code 56 - alt
        db      ' '             ;scan code 57
        db      0               ;scan code 58 - cap lock
        db      81h             ;scan code 59 - F1
        db      82h             ;scan code 60 - F2
        db      83h             ;scan code 61 - F3
        db      84h             ;scan code 62 - F4
        db      85h             ;scan code 63 - F5
        db      86h             ;scan code 64 - F6
        db      87h             ;scan code 65 - F7
        db      88h             ;scan code 66 - F8
        db      89h             ;scan code 67 - F9
        db      8ah             ;scan code 68 - F10
        db      0               ;scan code 69 - num lock
        db      0               ;scan code 70 - scroll lock
        db      10h             ;scan code 71 - home
        db      11h             ;scan code 72 - up arrow
        db      12h             ;scan code 73 - page up
        db      '-'             ;scan code 74
        db      13h             ;scan code 75 - left arrow
        db      0               ;scan code 76 - key pad 5
        db      14h             ;scan code 77 - right arrow
        db      '+'             ;scan code 78
        db      15h             ;scan code 79 - end
        db      16h             ;scan code 80 - down arrow
        db      17h             ;scan code 81 - page down
        db      18h             ;scan code 82 - ins
        db      19h             ;scan code 83 - del
shifted
        db      0               ;scan code 0 not used
        db      0               ;scan code 1 - esc
        db      '!'             ;scan code 2
        db      '@'             ;scan code 3
        db      '#'             ;scan code 4
        db      '$'             ;scan code 5
        db      '%'             ;scan code 6
        db      '^'             ;scan code 7
        db      '&'             ;scan code 8
        db      '*'             ;scan code 9
        db      '('             ;scan code 10
        db      ')'             ;scan code 11
        db      '_'             ;scan code 12
        db      '+'             ;scan code 13
        db      08h             ;scan code 14 - back space
        db      09h             ;scan code 15 - tab
        db      'Q'             ;scan code 16
        db      'W'             ;scan code 17
        db      'E'             ;scan code 18
        db      'R'             ;scan code 19
        db      'T'             ;scan code 20
        db      'Y'             ;scan code 21
        db      'U'             ;scan code 22
        db      'I'             ;scan code 23
        db      'O'             ;scan code 24
        db      'P'             ;scan code 25
        db      '{'             ;scan code 26
        db      '}'             ;scan code 27
        db      0dh             ;scan code 28 - return
        db      0               ;scan code 29 - control
        db      'A'             ;scan code 30
        db      'S'             ;scan code 31
        db      'D'             ;scan code 32
        db      'F'             ;scan code 33
        db      'G'             ;scan code 34
        db      'H'             ;scan code 35
        db      'J'             ;scan code 36
        db      'K'             ;scan code 37
        db      'L'             ;scan code 38
        db      ':'             ;scan code 39
        db      '"'             ;scan code 40
        db      '~'             ;scan code 41
        db      0               ;scan code 42 - left shift
        db      '|'             ;scan code 43
        db      'Z'             ;scan code 44
        db      'X'             ;scan code 45
        db      'C'             ;scan code 46
        db      'V'             ;scan code 47
        db      'B'             ;scan code 48
        db      'N'             ;scan code 49
        db      'M'             ;scan code 50
        db      '<'             ;scan code 56
        db      '>'             ;scan code 52
        db      '?'             ;scan code 53
        db      0               ;scan code 54 - right shift
        db      0               ;scan code 55 - prt scrn
        db      0               ;scan code 56 - alt
        db      ' '             ;scan code 57
        db      0               ;scan code 58 - cap lock
        db      0               ;scan code 59 - F1
        db      0               ;scan code 60 - F2
        db      0               ;scan code 61 - F3
        db      0               ;scan code 62 - F4
        db      0               ;scan code 63 - F5
        db      0               ;scan code 64 - F6
        db      0               ;scan code 65 - F7
        db      0               ;scan code 66 - F8
        db      0               ;scan code 67 - F9
        db      0               ;scan code 68 - F10
        db      0               ;scan code 69 - num lock
        db      0               ;scan code 70 - scroll lock
        db      '7'             ;scan code 71 - home
        db      '8'             ;scan code 72 - up arrow
        db      '9'             ;scan code 73 - page up
        db      '-'             ;scan code 74
        db      '4'             ;scan code 75 - left arrow
        db      '5'             ;scan code 76 - key pad 5
        db      '6'             ;scan code 77 - right arrow
        db      '+'             ;scan code 78
        db      '1'             ;scan code 79 - end
        db      '2'             ;scan code 80 - down arrow
        db      '3'             ;scan code 81 - page down
        db      '0'             ;scan code 82 - ins
        db      '.'             ;scan code 83 - del
; ******************* end of keyboard routine and tables ***********
        end




