$nomod51
$symbols
$debug
$nolist
$include (\i\rg51fb.pdf)
$list
;
;This is a test of the PWM output of the 8051fx family of parts.
;
;It will generate a sine wave of arbitrary frequency by DDS techniques.
;with a 16.000 Mhz clock, the sample frequency, Fs, will be 15.625 Khz.
; output frequency will be: Fo = Fs * ( Phase_inc / 65536 )
;
;  Phase_inc = (65536 * Frequency)/Fs
;  phase_inc = (65536 / 15625) * Frequency  = 4.1943 * Freq.
; for standard DTMF tones the following table can be used:
;row	Phase_inc
;697 	2923 = 0b6bh	1 2 3  a
;770	3230 = 0c9eh	4 5 6  b
;852	3574 = 0df6h	7 8 9  c
;941	3947 = 0f6bh 	* 0 #  d
;col		     	| | |  |
;1209	5071 = 13cfh ---+ | |  |
;1336	5604 = 15e4h -----+ |  |
;1477	6195 = 1833h -------+  |
;1633   6849 = 1ac1h ----------+
;
; This technique allows synthesis of Sine waves from about 5khz to .24Hz with
; .25Hz frequency resolution and high spectral purity, after filtering.
;
data_seg segment data
code_seg segment code
;
cseg at reset
	ljmp	start
;reserve interrupt vectors
;
cseg at exti0
	reti
;
cseg at timer0
	reti
;
cseg at exti1
	reti
;
cseg at timer1
;	ljmp	DDS_isr
	reti
;
cseg at sint
	reti
;
; the following interrupt vector is for the 8052
	org	002bh
timer2:
; the following interrupt vector is for the 8051Fx
	org	0033h
pca:
;
cseg at timer2
;	reti
;
cseg at pca
PCA_I:	ljmp	DDS_isr
;
dseg	at 18h
;
Phase_acc:	ds	2	; big endian
Phase_inc:	ds	2	; msb,lsb
NewVal:	ds	1
;
cseg
;
DDS_isr:
	mov	ccap4h,newval	; put out new value, pronto
	clr	ccf3	; clear interrupt flag
	inc	ccap3h	; set up for next interrupt to occur 256 cycles later
	clr	ea	; disable interrupts
	push	acc
	push	psw
	push	dph
	push	dpl
;				; Phase_acc := Phase_acc + Phase_inc
	mov	a,phase_inc+1	;  LSB
	add	a,Phase_acc+1
	mov	phase_acc+1,a

	mov	a,phase_inc	;  MSB
	addc	a,Phase_acc
	mov	phase_acc,a
;
	mov	dptr,#SinTab	; PWM = sine(hi(Phase_Acc))
	movc	a,@a+dptr
	mov	newval,a

	pop	dpl
	pop	dph
	pop	psw
	pop	acc
	setb	ea
DDS_end:
	reti                	
;
;**********************
;
Start:	mov	ie,#0
	mov	sp,#20h

	; initialize PCA
	mov	cmod,#02h	;select clock = 1/4 Fosc
	mov	cl,#0
	mov	ch,#0

 	mov	ccapm4,#43h	; module 4 in PWM mode
	mov	ccap4l,#00h
	mov	ccap4h,#80h	; start with .50 duty cycle

	; Use PCA channel 3 as interrupt source.
	mov	ccapm3,#49h	; module 3= interrupt on compare
	mov	ccap3l,#0
	mov	ccap3h,#0
	setb	ec		; enable pca INT

	;initialize DDS variables
	orl	psw,#18h	; sel rb3
	mov	phase_inc,#0bh
	mov	phase_inc+1,#6bh
	anl	psw,# not 18h	; sel rb0

	setb	cr		; turn on PCA timer.
	setb	ea		; enable all interrupts
loop:
	jmp	loop		;  hang in there waiting for interrupts
;
	nop
;
sintab:
    DB  080H,083H,086H,089H,08CH,090H,093H,096H
    DB  099H,09CH,09FH,0A2H,0A5H,0A8H,0ABH,0AEH
    DB  0B1H,0B3H,0B6H,0B9H,0BCH,0BFH,0C1H,0C4H
    DB  0C7H,0C9H,0CCH,0CEH,0D1H,0D3H,0D5H,0D8H
    DB  0DAH,0DCH,0DEH,0E0H,0E2H,0E4H,0E6H,0E8H
    DB  0EAH,0EBH,0EDH,0EFH,0F0H,0F1H,0F3H,0F4H
    DB  0F5H,0F6H,0F8H,0F9H,0FAH,0FAH,0FBH,0FCH
    DB  0FDH,0FDH,0FEH,0FEH,0FEH,0FFH,0FFH,0FFH
    DB  0FFH,0FFH,0FFH,0FFH,0FEH,0FEH,0FEH,0FDH
    DB  0FDH,0FCH,0FBH,0FAH,0FAH,0F9H,0F8H,0F6H
    DB  0F5H,0F4H,0F3H,0F1H,0F0H,0EFH,0EDH,0EBH
    DB  0EAH,0E8H,0E6H,0E4H,0E2H,0E0H,0DEH,0DCH
    DB  0DAH,0D8H,0D5H,0D3H,0D1H,0CEH,0CCH,0C9H
    DB  0C7H,0C4H,0C1H,0BFH,0BCH,0B9H,0B6H,0B3H
    DB  0B1H,0AEH,0ABH,0A8H,0A5H,0A2H,09FH,09CH
    DB  099H,096H,093H,090H,08CH,089H,086H,083H
    DB  080H,07DH,07AH,077H,074H,070H,06DH,06AH
    DB  067H,064H,061H,05EH,05BH,058H,055H,052H
    DB  04FH,04DH,04AH,047H,044H,041H,03FH,03CH
    DB  039H,037H,034H,032H,02FH,02DH,02BH,028H
    DB  026H,024H,022H,020H,01EH,01CH,01AH,018H
    DB  016H,015H,013H,011H,010H,00FH,00DH,00CH
    DB  00BH,00AH,008H,007H,006H,006H,005H,004H
    DB  003H,003H,002H,002H,002H,001H,001H,001H
    DB  001H,001H,001H,001H,002H,002H,002H,003H
    DB  003H,004H,005H,006H,006H,007H,008H,00AH
    DB  00BH,00CH,00DH,00FH,010H,011H,013H,015H
    DB  016H,018H,01AH,01CH,01EH,020H,022H,024H
    DB  026H,028H,02BH,02DH,02FH,032H,034H,037H
    DB  039H,03CH,03FH,041H,044H,047H,04AH,04DH
    DB  04FH,052H,055H,058H,05BH,05EH,061H,064H
    DB  067H,06AH,06DH,070H,074H,077H,07AH,07DH


	end
