; ----------------------------------------------------
; R-NET.asm      Last Update:06/22/92
;
; RS-485 network demo program for the prototyping
; board
;
;
; Courtesy Rigel Corporation
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; constants
  HERE   equ 2     ; address of this node
  OKcmd  equ 0
; ----------------------------------------------------
; port bits

  Dir485  equ P1.3  ; Dir485=0 for input, 1 for output
  Data485 equ P1.2  ; Data input/output
; ----------------------------------------------------
; registers
  OutByte equ 7fh   ; Output byte copy
; ----------------------------------------------------

       org   8000h
ProgStart:                    ; program starts here
       clr   RI               ; flush serial input
       clr   Dir485           ; setup for input

loop0:
       jnb   RI, loop1
       lcall getchr
       cjne  a, #3, loop1
       ljmp  0
loop1:
       jb    Data485, loop0

       lcall Bin5             ; read message of 5 bytes

       mov   dptr, #InBuffer
       movx  a, @dptr         ; get source address
       inc   dptr
       movx  a, @dptr         ; get destination address
       cjne  a, #HERE, loop0  ; done if not this address

       inc   dptr
       movx  a, @dptr         ; get command
       push  acc
       clr   C
       subb  a, #5
       pop   acc
       jnc   loop0

       mov   dptr, #CmdTable  ; command table base
       clr   C                ; clear carry flag
       rlc   a                ; multiply acc by 2
       mov   b, a             ; save acc in b register
       inc   a                ; get ready to read high byte
       movc  a, @a+dptr       ; get low address
       push  acc
       mov   a, b             ; recall acc to...
       movc  a, @a+dptr       ; ... read high address
       push  acc
       ret                    ; ljump by a return

CmdTable:
       dw    SlaveSendDigital    ; Cmd 0
       dw    SlaveSendAnalog     ; Cmd 1
       dw    SlaveReceiveDigital ; Cmd 2
       dw    SlaveEchoDigital    ; Cmd 3
       dw    Acknowledge         ; Cmd 4


; --- Command Service Routines -----------------------
; ----------------------------------------------------
SlaveSendDigital:
       lcall print
       db   "Slave_Send_Digital message received", 0ah, 0dh, 0

       mov   a, #HERE ; source address
       lcall Sout     ; send message byte

       mov   a, #0    ; master's address
       lcall Sout     ; send message byte

       mov   a, #0    ; echo  Slave_Send_Digital command
       lcall Sout     ; send message byte

       mov   a, #0aah ; send data byte
       lcall Sout     ; send message byte

       clr   a        ; dummy byte is 0
       lcall Sout     ; send dummy byte

       ljmp  loop0
; ----------------------------------------------------
SlaveReceiveDigital:
       lcall print
       db   "Slave_Receive_Digital message received..."
       db   "data byte is ",0

       mov   dptr, #InBuffer
       inc   dptr
       inc   dptr
       inc   dptr
       movx  a, @dptr   ; get data byte
       mov   OutByte, a ; save data byte
       push  acc        ; save data byte for processing later
       lcall prthex
       lcall crlf

       pop   acc       ; recall data byte
       mov   C, acc.0  ; data bit 0 goes to P1.0
       cpl   C         ; complement C so that a 1 lights LED
       mov   P1.0, C
       mov   C, acc.1  ; data bit 1 goes to P1.1
       cpl   C         ; complement C so that a 1 lights LED
       mov   P1.1, C

       ljmp  loop0
; ----------------------------------------------------
SlaveEchoDigital:
       lcall print
       db   "Slave_Echo_Digital message received... data byte sent: ", 0

       mov   a, #HERE ; source address
       lcall Sout     ; send message byte
       mov   a, #0    ; master's address
       lcall Sout     ; send message byte
       mov   a, #3    ; send echo command
       lcall Sout     ; send message byte

       clr   a        ; set up acc for data byte
       mov   C, P1.4  ; place push button values
       cpl   C        ; send 1 if button pressed
       mov   acc.0, C
       mov   C, P1.5  ; now push button 1
       cpl   C
       mov   acc.1, C

       mov   b, OutByte ; recall output byte
       Mov   C, b.0     ; place LED 0 value
       mov   acc.2, C
       mov   C, b.1     ; now LED 1
       mov   acc.3, C
       push  acc
       lcall Sout     ; send data byte
       pop   acc
       lcall prthex
       lcall crlf

       clr   a        ; dummy byte is 0
       lcall Sout     ; send dummy byte

       ljmp  loop0
; ----------------------------------------------------
SlaveSendAnalog:
       lcall print
       db   "Slave_Send_Analog message received", 0ah, 0dh, 0
       ljmp  loop0
; ----------------------------------------------------
Acknowledge:
       lcall print
       db   "Acknowledge message received", 0ah, 0dh, 0
       mov   a, #0     ; master's address
       lcall Sout      ; send address
       mov   a, #HERE  ; this node's address
       lcall Sout      ; send address
       mov   a, #OKcmd ; OK command
       lcall Sout      ; send command
       clr   a        ; dummy data byte
       lcall Sout      ; send data byte
       ljmp  loop0
; ----------------------------------------------------

; ====================================================
; include files

#include r-util.inc
#include r-net.inc

; ====================================================
;
; end of program
