; -----------------------------------------------------------------------------
; R-SLAVE0.asm      Last Update:06/22/92
;
; RS-485 network demo program for the prototyping board
;
; This routine emulates a PC network slave node.
; This routine illustrates the skeletal network slave code.
; It will not work with NETDEMO.exe since its command service
; routines do not respond back to the network master node.
;
; Run R-MASTER on one board and R-SLAVEx on other node(s).
;
; Courtesy Rigel Corporation
;
; =============================================================================
;       m a i n    p r o g r a m
; -----------------------------------------------------------------------------
; constants
  HERE   equ 2     ; address of this node
  OKcmd  equ 0

; -----------------------------------------------------------------------------
; port bits

  Dir485  equ P1.3  ; Dir485=0 for input, 1 for output
  Data485 equ P1.2  ; Data input/output
; -----------------------------------------------------------------------------
; registers
  OutByte equ 7fh   ; Output byte copy
; -----------------------------------------------------------------------------

       org   8000h
ProgStart:                    ; program starts here
       clr   RI               ; flush serial input
       clr   Dir485           ; setup for input

loop0:
       jnb   RI, loop1        ; see if a key is pending (from the RS-232 host)
       lcall getchr           ; if so get char...
       cjne  a, #3, loop1     ; ... if not ctl-C branch to loop1...
       ljmp  0                ; ... else return to monitor via RESET
loop1:
       jb    Data485, loop0   ; see if a start bit is on the network
                              ; if not repeat (branch to loop0)

                              ; if start bit is detected on the network ...
       lcall Bin5             ; ...read message of 5 bytes.  Bin5 places ...
                              ; ... the 5 bytes in external memory ...
                              ; ... starting at address defined by InBuffer

       mov   dptr, #InBuffer  ; point to the start of message block
       movx  a, @dptr         ; read source address into acc

       inc   dptr             ; point to next message byte
       movx  a, @dptr         ; read destination address into acc
       cjne  a, #HERE, loop0  ; done if not this address, repeat

       inc   dptr             ; else point to next message byte
       movx  a, @dptr         ; read command into acc
       push  acc              ; save command
       clr   C                ; clear carry flag, setup for subtraction
       subb  a, #5            ; command-5
       pop   acc              ; recall command
       jnc   loop0            ; if no barrow command>4, (invalid) done, repeat

                              ; if command is valid (0<command<5) then...
                              ; ... jump to the corresponding command ...
                              ; ...service routine.  The service routine ...
                              ; ... addresses are stored in the table ...
                              ; ...labeled CmdTable.

       mov   dptr, #CmdTable  ; command table base
       rlc   a                ; multiply acc by 2
       mov   b, a             ; save acc in b register
       inc   a                ; get ready to read high byte
       movc  a, @a+dptr       ; get high byte - low address of service routine
       push  acc              ; push command service routine low addr on stack
       mov   a, b             ; recall acc to...
       movc  a, @a+dptr       ; ... read command service routine high address
       push  acc              ; push command service routine high addr on stack
       ret                    ; ljump to command service routine by a return
                              ; Note: return pops 2 bytes from stack into...
                              ; ... program counter, next instruction...
                              ;... fetched is thus the command service routine
CmdTable:
       dw    SlaveSendDigital    ; Cmd 0
       dw    SlaveSendAnalog     ; Cmd 1
       dw    SlaveReceiveDigital ; Cmd 2
       dw    SlaveEchoDigital    ; Cmd 3
       dw    Acknowledge         ; Cmd 4


; --- Command Service Routines ------------------------------------------------
; -----------------------------------------------------------------------------
SlaveSendDigital:
       lcall print
       db   "Slave_Send_Digital message received", 0ah, 0dh, 0
       ljmp  loop0
; -----------------------------------------------------------------------------
SlaveReceiveDigital:
       lcall print
       db   "Slave_Receive_Digital message received..."
       db   "data byte is ",0

       mov   dptr, #InBuffer
       inc   dptr
       inc   dptr
       inc   dptr
       movx  a, @dptr   ; get data byte
       mov   OutByte, a ; save data byte
       lcall prthex
       lcall crlf
       ljmp  loop0
; -----------------------------------------------------------------------------
SlaveEchoDigital:
       lcall print
       db   "Slave_Echo_Digital message received... ", 0ah, 0dh, 0
       ljmp  loop0
; -----------------------------------------------------------------------------
SlaveSendAnalog:
       lcall print
       db   "Slave_Send_Analog message received", 0ah, 0dh, 0
       ljmp  loop0
; -----------------------------------------------------------------------------
Acknowledge:
       lcall print
       db   "Acknowledge message received", 0ah, 0dh, 0
       ljmp  loop0
; -----------------------------------------------------------------------------

; =============================================================================
; include files

#include r-util.inc
#include r-net.inc

; =============================================================================
;
; end of program
