; ==========================================================
; subroutine binasc
; binasc takes the contents of the accumulator and converts it
; into two ascii hex numbers.  the result is returned in the
; accumulator and r2.
; ==========================================================
;
binasc:  mov   r2, a             ; save in r2
         anl   a,  #0fh         ; convert least sig digit.
         add   a,  #0f6h        ; adjust it
         jnc   noadj1            ; if a-f then readjust
         add   a,  #07h
noadj1:  add   a,  #3ah         ; make ascii

         xch   a,  r2            ; put result in reg 2
         swap  a                 ; convert most sig digit
         anl   a,  #0fh         ; look at least sig half of acc
         add   a,  #0f6h        ; adjust it
         jnc   noadj2            ; if a-f then re-adjust
         add   a,  #07h
noadj2:  add   a,  #3ah         ; make ascii
         ret

; ==========================================================
; subroutine crlf
; crlf sends a carriage return line feed out the serial port
; ==========================================================
;
crlf:    mov   a,  #0ah         ; print lf
         lcall sndchr
cret:    mov   a,  #0dh         ; print cr
         lcall sndchr
         ret

; ==========================================================
; subroutine getchr
; this routine reads in a chr from the serial port and saves it
; in the accumulator.
; ==========================================================
;
getchr:  jnb  ri, getchr         ; wait till character received
         mov  a,  sbuf           ; get character
         anl  a,  #7fh          ; mask off 8th bit
         clr  ri                 ; clear serial status bit
         ret

; ====================================================
;  subroutine inkey
;  input    : peeks at serial port - no wait
;  output   : if chr present
;                 then accumulator returns chr
;                 else accumulator is 00 (null)
;  destroys : a
; ----------------------------------------------------
inkey:  mov a,  #00h
        jnb ri, doneik
        mov a,  sbuf
        anl a, #7fh
        clr ri
doneik: ret

; ==========================================================
; subroutine print
; print takes the string immediately following the call and
; sends it out the serial port.  the string must be terminated
; with a null. this routine will ret to the instruction
; immediately following the string.
; ==========================================================
;
print:   pop   dph              ; put return address in dptr
         pop   dpl
         lcall prtstr           ; print string and update dptr
         mov   a,  #1h          ; point to instruction after string
         jmp   @a+dptr          ; return

; ==========================================================
; subroutine prtstr
; this routine takes the string pointed to by the data pointer
; and sends it out the serial port. the string must be
; terminated with a null.
; ==========================================================
;
prtstr:  clr  a                  ; set offset = 0
         movc a,  @a+dptr        ; get chr from code memory
         cjne a,  #0h, mchrok    ; if chr = ff then return
         ret
mchrok:  lcall sndchr            ; send character
         inc   dptr              ; point at next character
         ljmp  prtstr            ; loop till end of string

; ==========================================================
; subroutine prthex
; this routine takes the contents of the acc and prints it out
; as a 2 digit ascii hex number.
; ==========================================================
;
prthex:  lcall binasc            ; convert acc to ascii
         lcall sndchr            ; print first ascii hex digit
         mov   a,  r2            ; get second ascii hex digit
         lcall sndchr            ; print it
         ret

; ==========================================================
; subroutine prsphx
; this routine first prints a space then takes the contents of
; the acc and prints it out as a 2 digit ascii hex number.
; ==========================================================
;
prsphx:  mov   r2, a             ; save acc in r2
         mov   a,  #20h         ; print space
         lcall sndchr
         mov   a,  r2            ; recall acc
         lcall prthex            ; print it
         ret

; ==========================================================
; subroutine sndchr
; this routine takes the chr in the acc and sends it out the
; serial port.
; ==========================================================
;
sndchr:  clr  scon.1             ; clear the tx  buffer full flag.
         mov  sbuf,a             ; put chr in sbuf
txloop:  jnb  scon.1, txloop     ; wait till chr is sent
         ret

; ====================================================
;  subroutine beep
;  input    : none
;  output   : ^g (bell = 7) sent to serial port
;  destroys : a
; ----------------------------------------------------
beep:   push  acc
        mov a, #7h
        lcall sndchr
        pop   acc
        ret

; ==========================================================
; subroutine crlf
; crlf sends a carriage return line feed out the serial port
; ==========================================================
;
crlf:    mov   a,  #0ah         ; print lf
         lcall sndchr
cret:    mov   a,  #0dh         ; print cr
         lcall sndchr
         ret

; ====================================================
;  subroutine delay - millisecond delay
;   accumulator holds milliseconds to delay
;   - 2 microseconds are reserved for the call
;   to this routine.
;  input    : milliseconds to delay in accumulator
;  output   : none
;  destroys : nothing - uses a
; ----------------------------------------------------
; 100h-a6h=5ah=(90)decimal
; 90 * 11 = 990
; plus 10 gives 1 millisecond
;
;                        microseconds (cycles)
;                       -----------------------
delay:
        dec   a         ;    1

d_olp:
      push acc          ;    2            \
      mov   a, #0a6h    ;    1            |
                        ;                 |
d_ilp:  inc   a         ;    1  \         |
        nop             ;    1  |         |
        nop             ;    1  |         |
        nop             ;    1  |         |
        nop             ;    1  |         |
        nop             ;    1  |- 11     | (acc-1)
        nop             ;    1  | cycles  |- msec
        nop             ;    1  |         |
        nop             ;    1  |         |
        jnz   d_ilp     ;    2  /         |
                        ;                 |
        nop             ;    1            |
        nop             ;    1            |
        nop             ;    1            |
        pop   acc       ;    2            |
                        ;                 |
        djnz  acc,d_olp ;  2            /

; need to wait 998 microseconds more

        mov   a, #0a6h  ;    1

d_lp2:  inc   a         ;    1  \
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |- 11
        nop             ;    1  | cycles
        nop             ;    1  |
        nop             ;    1  |
        jnz   d_lp2     ;    2  /
        nop             ;    1
        nop             ;    1
        nop             ;    1
        nop             ;    1
        nop             ;    1

        ret             ;    2

; ====================================================
;  subroutine mdelay - millisecond delay
;   delays for 998 microseconds - 2 microseconds are
;   reserved for the call to this routine.
;  input    : none
;  output   : none
;  destroys : nothing - uses a
; ----------------------------------------------------
; 100h-a6h=5ah=(90)decimal
; 90 * 11 = 990
; plus 8 gives 998 microseconds
;
;                        microseconds (cycles)
;                       -----------------------
mdelay: push acc        ;    2
        mov   a, #0a6h  ;    1

md_olp: inc   a         ;    1  \
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |- 11  cycles
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        jnz   md_olp    ;    2  /

        nop             ;    1
        pop   acc       ;    2
        ret             ;    2

; ====================================================
;  subroutine sdelay - second delay
;   delays for 999998 microseconds - 2 microseconds
;   are reserved for the call to this routine.
;  input    : none
;  output   : none
;  destroys : nothing - uses a
; ----------------------------------------------------
; 100h-91h=6fh=(111)decimal
; 9008 * 111 = 999888
; plus 102 from second loop
; plus 8 gives 999998 microseconds
;
;                        microseconds (cycles)
;                       -----------------------
sdelay: push acc             ;   2
        mov  a, #91h        ;   1

sd_olp: inc   a      ; \
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |- loop takes 9008 microseconds
        lcall mdelay ; |
        lcall mdelay ; |
        nop          ; |
        nop          ; |
        nop          ; |
        nop          ; |
        nop          ; |
        jnz   sd_olp ; /

        mov   a, #33h       ;   1
sd_ilp: djnz  acc, sd_ilp    ; -loop takes 2*33h=66h=(102)dec

        pop   acc            ;   2
        ret                  ;   2

