; ----------------------------------------------------
; rTime1.asm      Last Update:10/24/90
;
; Reaction Timer
;
; Courtesy Rigel Corporation
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD connections
;
; pb0          p1.0 : press to stop count
; low  digit   p4
; high digit   p5
;
; internal memory use
; bytes:
   clk_0 equ   70h ; software clock 1s
   clk_1 equ   71h ; software clock 10s
   clk_2 equ   72h ; software clock 100s
   rndtim equ  73h ; random delay time
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------
    org 8000h

    mov   rndtim, #3    ; 3 seconds to fire
    mov   clk_0, #0     ; initialize clock ticks
    mov   clk_1, #0
    mov   clk_2, #0
    sjmp  lb0           ; jump to display ready
begin:
    mov   clk_0, #0     ; initialize clock ticks
    mov   clk_1, #0
    mov   clk_2, #0
    mov   p4, #0ffh     ; turn digits off
    mov   p5, #0ffh

    lcall randwait      ; wait for rndtim
    mov   p5, #0        ; flash signal
    mov   p4, #0
    lcall count         ; wait foe reaction
    lcall showtime      ; show reaction time
lb0:
    mov   p4, #0a1h     ; display the 'r' (ready)
    mov   p5, #0afh     ; display the 'd'
    lcall random        ; get next random time

lb1:
    jb    p1.0, lb1     ; wait for button pressed
lb2:
    jnb   p1.0, lb2     ; wait for button release

    sjmp  begin


; ====================================================
;  subroutine random
;
;  get a random number in the interval [2,5].  uses
;  the clk_0 (clock 1s) for a random number.
;
;  input    : clk_0
;  output   : rndtim
;  destroys : a
; ----------------------------------------------------
random:
    mov   a, clk_0
    anl   a, #3
    add   a, #2
    mov   rndtim, a
    ret

; ====================================================
;  subroutine randwait
;
;  waits for the number of seconds given in rndtim.
;
;  input    : rndtim
;  output   : none
;  destroys : a
; ----------------------------------------------------
randwait:
    mov   a, rndtim
    jz    rw2
rw1:
    lcall sdelay
    djnz  rndtim, rw1
rw2:
    ret

; ====================================================
;  subroutine count
;
;  increments the clock stored in clk_0, clk_1, clk_2
;  every millisecond until port bit p1.0 is cleared.
;  p1.0 is connected to pb0, so count measures the
;  time until pb0 is pressed.
;
;  input    : p1.0
;  output   : clk_0, clk_1, clk_2
;  destroys : a
; ----------------------------------------------------
count:
    jb   p1.0, cnt0
    ret
cnt0:
    lcall mdelay        ; wait for 1 millisecond
    mov   a, clk_0
    inc   a             ; increment clock
    cjne  a, #0ah, cnt1 ; if 10 milliseconds...
    mov   clk_0, #0     ; clear clk_0
    sjmp  cnt2          ; and check clk_1

cnt1:
    mov   clk_0, a      ; ...otherwise repeat
    sjmp  count
cnt2:
    mov   a, clk_1      ; increment clk_1
    inc   a
    cjne  a, #0ah, cnt3 ; if 100 milliseconds...
    mov   clk_1, #0     ; clear clk_1
    sjmp  cnt4          ; and check clk_2

cnt3:
    mov   clk_1, a      ; otherwise repeat
    sjmp  count
cnt4:
    mov   a, clk_2      ; increment clk_2
    inc   a
    cjne  a, #0ah, cnt5 ; if 1000 milliseconds...
    mov   clk_2, #0     ; clear clk_2
    ret                 ; and return
cnt5:
    mov   clk_2, a      ; otherwise repeat
    sjmp  count


; ====================================================
;  subroutine show reaction time
;
;  shows the values clk_2, clk_1, and clk_0 by
;  outputting the sevent-segment-display patterns to
;  ports 4 and 5.  p4 and p5 are connected to the low
;  and high digit displays, respectively.  the 3
;  digits clk_2, clk_1, and clk_0 are rotated through
;  the 2 displays.  showtime terminates when p1.0 is
;  cleared.  since p1.0 is connected to pb0, showtime
;  returns when pb0 is pressed.
;
;  input    : clk_0, clk_1, clk_2, p1.0
;  output   : p4, p5
;  destroys : a
; ----------------------------------------------------
showtime:

    mov   a, clk_0      ; compute clk_0+clk_1+clk_2...
    add   a, clk_1      ; if zero then error
    add   a, clk_2
    jnz   swt2
;----if error ----------------------------------------
swt0:
    mov   p4, #0ffh     ; turn digits off
    mov   p5, #0ffh
    clr   a             ; clear a for a delay of...
    lcall delay         ; 256 milliseconds
    jb    p1.0, swt1    ; check if PB0 pressed
    ljmp  swt12         ; if not...
swt1:
    mov   p4, #0bfh     ; turn segments g on
    mov   p5, #0bfh
    clr   a             ; again, delay for...
    lcall delay         ; 256 milliseconds
    jb    p1.0, swt0    ; check for PB0 pressed
    sjmp  swt12

;----if valid reaction time ---------------------------
swt2:
    mov   p5, #0ffh     ; clear displays
    mov   a, clk_2      ; get most significant digit
    lcall display       ; convert to 7-segment
    mov   p4, a         ; display on LOW DIGIT
    clr   a             ; delay for...
    lcall delay         ; 256 milliseconds
    jb    p1.0, swt3    ; check for PB0 pressed
    sjmp  swt12         ; if PB0 not pressed...
swt3:
    lcall delay         ; ...another 256 milliseconds
    jb    p1.0, swt4    ; check if PB0 pressed...
    sjmp  swt12         ; ...if not
swt4:
    mov   p5, p4        ; move most significant digit
                        ; to HIGH DIGIT
    mov   a, clk_1      ; get next digit (10s)
    lcall display       ; convert to 7-segment
    mov   p4, a         ; display on LOW DIGIT
    clr   a             ; 256 millisecond...
    lcall delay         ; ...delay
    jb    p1.0, swt5    ; check if PB0 pressed
    sjmp  swt12         ; if PB0 not pressed...
swt5:
    lcall delay         ; ...another 256 milliseconds
    jb    p1.0, swt6    ; check if PB0 pressed...
    sjmp  swt12         ; ...if not
swt6:
    mov   p5, p4        ; move digit 10s to HIGH DIGIT
    mov   a, clk_0      ; get least significant digit
    lcall display       ; convert to 7-segment
    mov   p4, a         ; display on LOW DIGIT
    clr   a             ; 256 millisecond...
    lcall delay         ; ...delay
    jb    p1.0, swt7    ; check if PB0 pressed
    sjmp  swt12         ; if not...
swt7:
    lcall delay         ; another 256 milliseconds
    jb    p1.0, swt8    ; check if PB0 pressed
    sjmp  swt12         ; if not...
swt8:
    mov   p5, p4        ; move least significant digit
                        ; to HIGH DIGIT
    mov   p4, #0ffh     ; turn off LOW DIGIT
    clr   a             ; 256 millisecond delay
    lcall delay
    jb    p1.0, swt9    ; check if PB0 is pressed
    sjmp  swt12         ; if not..
swt9:
    lcall delay         ; ...another 256 milliseconds
    jb    p1.0, swt10   ; check again if PB0 is pressed
    sjmp  swt12         ; if not...
swt10:
    mov   p5, #0ffh     ; turn both digits off
    mov   p4, #0ffh
    clr   a             ; and delay for...
    lcall delay         ; 256 milliseconds
    jb    p1.0, swt11   ; check if PB0 is pressed
    sjmp  swt12         ; if not...
swt11:
    lcall delay         ; another 256 milliseconds
    jb    p1.0, swt2    ; check if PB0 is pressed
                        ; if PB0 is not pressed
                        ; then repeat all over

; ---- come here when PB0 is pressed -----------------
swt12:
    jnb   p1.0, swt12   ; wait until PB0 is released
    ret


; ====================================================
;
; end of program
