; these multi-byte precision math routines will do 1 to 4 bytes
; which equates from 8 to 32 bits. optimization could be achieved
; if a fixed precision is desired.
; with a 4 byte precision - you can divide 64 bits by 32 bits and
; multiply 32 bits by 32 bits with a 64 bit result 
; if you want any more bits, may i suggest floating point.
; included is a string to binary conversion for input and
; a display for any radix.
;
radbas		equ	10		; decimal display
;
; monitor routines
outt		equ	sndchr		; address of print screen routine
newline		equ	crlf		; send cr and lf to screen
;
; storage for multiplication
multiplier	equ	04h	; long1 or word1
product_hi	equ	04h	; long1 or word1
product_lo	equ	24h	; long2 or word2
multiplicand	equ	28h	; long3 or word3

; storage for division - redundant addressing for readability
divisor		equ	04h	; long1 or word1
remainder	equ	24h	; long2 or word2
dividend_hi	equ	24h
dividend_lo	equ	28h	; long3 or word3
quotient	equ	28h

buff		equ	30h		; ascii input buffer

counter		data	4bh		; bit counter for shifts
saver		equ	4ch		; temporary storage
altsav		equ	44h		; alternate saver

;
; calculate bit count from number of bytes in R2
getbits:
	mov	a,R2
	mov	b,#8
	mul	ab
	mov	counter,a
	ret
;
; move the address pointers to the least significant byte
; r2 has byte count
asetup:
	mov	a,r0
	add	a,r2
	dec	a
	mov	r0,a
	mov	a,r1
	add	a,r2
	dec	a
	mov	r1,a
	ret
;
; compare value in R0 with value in R1
; R2 has byte count
; C set if less
; Z set if equal
; N set if negative

mbcmp:	call	asetup
	setb	Z
	clr	N
	clr	c
cmpv2:	mov	a,@r0
	subb	a,@r1
	jz	$+4
	clr	Z		; low byte not equal
	dec	r0
	dec	r1
	djnz	r2,cmpv2
	jnb	acc.7,$+5
	setb	N
	ret
;
; and internal value in R0 with internal value in R1, result in R1
; r2 has byte count
mband:
ander1:	mov	a,@R0
	anl	a,@r1
	mov	@r1,a
	inc	r0
	inc	r1
	djnz	r2,ander1
	ret
;
; R0 has addr of value to negate, R2 has byte count
mbneg:
negate:	mov	a,r0
	add	a,r2
	dec	a
	mov	r0,a		; starting position
	setb	c		; 2's compliment
nv1:	mov	a,@r0
	cpl	a
	addc	a,#0
	mov	@r0,a
	dec	r0
	djnz	r2,nv1
	ret
;
; R0 has addr of value to shift left, R2 has byte count
mbsvl:	push	psw		; save carry
	mov	a,r0
	add	a,r2
	dec	a
	mov	r0,a
	pop	psw		; get carry
mbsvl2:	mov	a,@r0
	rlc	a
	mov	@r0,a
	dec	r0
	djnz	r2,mbsvl2
	ret
;
; R0 has addr of value to shift right, R2 has byte count
mbsvr:
mbsvr2:	mov	a,@r0
	rrc	a
	mov	@r0,a
	inc	r0
	djnz	r2,mbsvr2
	ret
;
; R0 has addr of value to clear, R2 has byte count
mbclr:
mbclr2:	mov	@r0,#0
	inc	r0
	djnz	r2,mbclr2
	ret
;
; R0 has addr of value to save, R2 has byte count
mbsav:
	mov	r1,#saver
mbsav2:	mov	a,@r0
	mov	@r1,a
	inc	r1
	inc	r0
	djnz	r2,mbsav2
	ret
;
; R0 has addr of value to restore, R2 has byte count
mbrst:
	mov	r1,#saver
mbrst2:	mov	a,@r1
	mov	@r0,a
	inc	r0
	inc	r1
	djnz	r2,mbrst2
	ret
;
; R0 has addr of value to swap, R2 has byte count
mbswp:
	mov	r1,#saver
mbswp2:	mov	a,@r0
	xch	a,@r1
	mov	@r0,a
	inc	r0
	inc	r1
	djnz	r2,mbswp2
	ret
;
; R0 has addr of value to add carry to, R2 has byte count
mbadc:	push	psw		; save carry
	mov	a,r0
	add	a,r2
	dec	a
	mov	r0,a
	pop	psw		; get carry
mbadc2:	mov	a,@r0
	addc	a,#0
	mov	@r0,a
	inc	r0
	djnz	r2,mbadc2
	ret
;
; multi-byte subtraction
; R0 has addr of source operand
; R1 has addr of destination operand
; R2 has byte count
; destination = destination - source
; I tried to simply negate and add but that
; messes up division when it tries to add back.
;
mbsub:	call	asetup
	clr	c
mbslup:	mov	a,@r1
	subb	a,@r0
	mov	@r1,a
	dec	r0
	dec	r1
	djnz	r2,mbslup
	ret
;
; multi-byte addition
; R0 has addr of source operand
; R1 has addr of destination operand
; R2 has byte count
; destination = source + destination
;
mbadd:	call	asetup
	clr	c
mbalup:	mov	a,@r0
	addc	a,@r1
	mov	@r1,a
	dec	r0
	dec	r1
	djnz	r2,mbalup
	ret
;
; move value pointed by R0 to addr pointed by R1, R2 has byte count
mbmov:
mbmv2:	mov	a,@r0
	mov	@r1,a
	inc	r0
	inc	r1
	djnz	r2,mbmv2
	ret
;
; multi-byte division

mbdiv:	call	getbits		; counter = # bits
	clr	c		; no Q bit yet
mbdiv0:	mov	r0,#dividend_lo
	call	mbsvl		; paper and pencil method
	mov	r0,#dividend_hi
	call	mbsvl
	mov	r0,#divisor	; do a goesinta
	mov	r1,#dividend_hi
	call	mbsub		; dividend = dividend - divisor
	jnc	mbdiv2		; go if positive
	mov	r0,#divisor	; overflow
	mov	r1,#dividend_hi	; add back divisor
	call	mbadd		; dividend = dividend + divisor
	setb	c		; set Q = 0
mbdiv2:	cpl	c		; set Q = 1
	mov	r0,#dividend_lo
	call	mbadc		; set/reset Q bit
	djnz	counter,mbdiv0
	ret			; oh yes!
;
; multi-byte multiplication
;
mbmul:	call	getbits			; counter  = # bits
	mov	r0,#product_lo		; clear product
	call	mbclr
mbmul0:	clr	c
	mov	r0,#product_lo		; shift low partial product
	call	mbsvl
	mov	r0,#multiplier		; through high multiplier to carry
	call	mbsvl
	jnc	mbmul1
	mov	r0,#multiplicand	; add multiplicand
	mov	r1,#product_lo		; to product
	call	mbadd
	jnc	mbmul1
	mov	r0,#product_hi		; extend result
	call	mbadc
mbmul1:	djnz	counter,mbmul0		; do bits
	ret				; ta da !
;

