; ----------------------------------------------------
; RunLgts1.asm      Last Update:10/24/90
;
; Rotating Lights Example
;  Uses a fixed delay of 40h milliseconds
;
; ====================================================
; UIOD connections
;
; L0-L7  to P4.0 to P4.7  : output
; L8-L15 to P5.0 to P5.7  : output
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; internal memory use
; bits
   rot_left equ  10h  ; rotating direction
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------
      org 8000h

      mov   p4, #0ffh
      mov   p5, #0ffh
      lcall print
      db "press any key to begin...", 0
      lcall crlf
      lcall getchr

      mov   p4, #0feh
      mov   p5, #0ffh
      setb  rot_left
loop:
      lcall rlights
      mov   a, #40h
      lcall delay
      jb    rot_left, light1
      mov   a, #'+'
      sjmp  light2

light1:
      mov   a, #'-'

light2:
      lcall sndchr

      mov   a, p5
      lcall prsphx
      mov   a, p4
      lcall prthex
      lcall crlf

      lcall inkey
      cjne  a, #3h, loop
      mov   p4, #0ffh
      mov   p5, #0ffh
      ljmp  os_return

; ====================================================
; subroutine rlights - rotate lights
;
; only one bit of p4 or p5 is 0 at any given time.
; the 0 bit is rotated left and right to create the
; effect of moving lights.
;
;  input    : none
;  output   : rotates p4 and p5
;  destroys : acc
; ----------------------------------------------------
rlights:
    mov  a, p4
    cjne a, #0feh, rot1
    setb rot_left      ; rotate left
    sjmp rotate
rot1:
    cjne a, #7fh, rot2
    jnb  rot_left, rotate
    mov  p4, #0ffh
    mov  p5, #0feh
    ret
rot2:
    mov  a, p5
    cjne a, #7fh, rot3
    clr  rot_left
    sjmp rotate
rot3:
    cjne a, #0feh, rot4
    jb   rot_left, rotate
    mov  p5, #0ffh
    mov  p4, #7fh
    ret
rot4:
rotate:
    jb   rot_left, left
    mov  a, p4
    rr   a
    mov  p4, a
    mov  a, p5
    rr   a
    mov  p5, a
    ret

left:
    mov  a, p4
    rl   a
    mov  p4, a
    mov  a, p5
    rl   a
    mov  p5, a
    ret

; ====================================================
;
; end of program
