; ----------------------------------------------------
; RunLgts2.asm      Last Update:10/24/90
;
; Rotating Lights Example
;  Uses Timer 0 for delays.  The timer issues an
;  interrupt.  The interrupt service routine
;  increments the internal ram location named clock.
;  When clock reaches 12 the lights are rotated.
;
; ====================================================
; UIOD connections
;
; L0-L7  to P4.0 to P4.7  : output
; L8-L15 to P5.0 to P5.7  : output
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; internal memory use
; bytes:
   clock equ  0f0h       ;  software clock
; bits
   dir_left equ  10h     ; rotating direction
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h  
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------
    org 8000h

    lcall print
    db "press any key to begin...", 0
    lcall crlf
    lcall getchr

    mov   ip0, #0        ;  all interrupts low priority
    mov   ip1, #0
    lcall itim0          ; initialize timer 0
    lcall print
    db "initialization done", 0
    lcall crlf
    setb  tr0
    setb  eal

    mov   p4, #0feh
    mov   p5, #0ffh
loop:
    lcall inkey
    cjne  a, #3h, loop

    mov   p4, #0ffh
    mov   p5, #0ffh
    clr   tr0
    clr   eal
    ljmp  os_return

; ====================================================
; subroutine rlights - rotate lights
;
; only one bit of p4 or p5 is 0 at any given time.
; the 0 bit is rotated left and right to create the
; effect of moving lights.
;
;  input    : none
;  output   : rotates p4 and p5
;  destroys : acc
; ----------------------------------------------------
rlights:
    mov  a, p4
    cjne a, #0feh, rot1
    setb dir_left      ; rotate left
    sjmp rotate
rot1:
    cjne a, #7fh, rot2
    jnb  dir_left, rotate
    mov  p4, #0ffh
    mov  p5, #0feh
    ret
rot2:
    mov  a, p5
    cjne a, #7fh, rot3
    clr  dir_left
    sjmp rotate
rot3:
    cjne a, #0feh, rot4
    jb   dir_left, rotate
    mov  p5, #0ffh
    mov  p4, #7fh
    ret
rot4:
rotate:
    jb   dir_left, rot_left
    mov  a, p4
    rr   a
    mov  p4, a
    mov  a, p5
    rr   a
    mov  p5, a
    ret

rot_left:
    mov  a, p4
    rl   a
    mov  p4, a
    mov  a, p5
    rl   a
    mov  p5, a
    ret

; ====================================================
;  subroutine initialize timer 0
;  input    : none
;  output   : none
;  destroys : a, r0, dptr
; ----------------------------------------------------
itim0:
    anl   tmod, #0f0h    ;  initialize timer 0
    orl   tmod, #00h     ;  mode 1 : 13-bit timer

    mov   dptr, #isrt0   ;  isr for timer 0
    mov   a, #1          ;  modify intt0 vector
    lcall setintvec
    mov   clock, #0      ;  initialize software clock
    setb  et0            ;  enable timer 0 int
    setb  tr0
    ret

; ====================================================
;  subroutine isrt0
;
;  timer 0 is initialized to run in mode 1.  this is a
;  13 bit timer, incremented every machine cycle.
;  thus, timer 0 overflows every 8192 microseconds.
;  12 such overflows occur in just under 100
;  miliseconds. isrt0 counts for 12 overflows and then
;  calls subroutine rlights to rotate the lights.
;
;  input    : timer 0 overflow
;  output   : calls rlights to rotate lights
;
;  destroys : nothing - uses a, r0, r1, r2
; ----------------------------------------------------
isrt0:
    push  psw
    mov   psw, #0     ; select register bank 0
    push  acc
    push  0
    push  1
    push  2

    mov   r0, #clock
    mov   a, @r0
    inc   a
    mov   @r0, a
    cjne  a, #12, isrt0dn
    clr   a
    mov   @r0, a

    lcall rlights

    jb    dir_left, light1
    mov   a, #'+'
    sjmp  light2

light1:
    mov   a, #'-'

light2:
    lcall sndchr

    mov   a, p5
    lcall prsphx
    mov   a, p4
    lcall prthex
    lcall crlf

isrt0dn:
    pop   2
    pop   1
    pop   0
    pop   acc
    pop   psw
    reti

; ====================================================
;
; end of program
