; ----------------------------------------------------
; Timer2.asm      Last Update:10/27/91
;
; Simple Timer Example
; Uses Counter/Timer 0 as a Timer in Mode 1
; register r7 is implemented as a software
; counter, decremented every time Timer 0
; overflows.  Timer 0 in mode 1 is a 16-bit
; timer, overflowing every 65536 microseconds.
; The software timer counts to 0Eh=15, thus
; the digit is updated every 65536*15=983040
; microseconds, or about a second, excluding
; the program overhead.
;
; ----------------------------------------------
; system calls              registers used
; ----------------------------------------------

 break     equ  010ch  ; a, (reads accumulator)
 crlf      equ  0115h  ; a
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 print     equ  0136h  ; a, dptr
 prthex    equ  013fh  ; a, r2
; --- constants --------------------------------------
 Scount    equ  0Eh    ; software counter reload value
; ====================================================
;       m a i n    p r o g r a m
; UIOD connections
;
; LOW DIGIT    P4     : output (segment a to P4.0, ...
;                               segment g to P4.6)
; ----------------------------------------------------

      org   8000h         ; run using the monitor
                          ;  G8000 command
      mov   TMOD, #81h    ; configure timer/counter 0
                          ;  timer in mode 1
      setb  TR0           ; turn timer/counter on
      mov   b, #0         ; current count
      mov   r7, #Scount   ; software counter

      mov   a, b          ; load a with current count
      lcall display       ; get display pattern
      mov   P4, a         ; display the digit
      mov   a, b          ; get current count
      lcall prthex        ; echo count to host
      lcall crlf          ; new line on host

loop:
      jnb   TF0, loop     ; repeat until Timer 0 times out
      clr   TF0           ; clear timer overflow flag
      djnz  r7, loop      ; decrement software counter
                          ; repeat until zero
      mov   r7, #Scount   ; reset software counter

      inc   b             ; when software counter overflows
                          ; increment current count
      anl   b, #0Fh       ; modulus 0Fh
      mov   a, b          ; load a with current count
      lcall display       ; get display pattern
      mov   P4, a         ; display the digit
      mov   a, b          ; get current count
      lcall prthex        ; echo count to host
      lcall crlf          ; new line on host

      lcall inkey         ; check serial port for character
      lcall break         ; see if character is CTL-C
                          ; terminate program if CTL-C

      sjmp loop           ; repeat forever...

; ====================================================
; end of program
