; ----------------------------------------------------
; Timer3.asm      Last Update:10/27/91
;
; Simple Timer Example
; Uses Counter/Timer 0 as a Timer in Mode 2
; registers r6 and r7 are implemented as a
; software counter, decremented every time
; Timer 0 overflows.  Timer 0 in mode 2 is
; a 8-bit timer.  Every time the low count
; register (TL0) overflows, it is reloaded
; with the value in the high count register
; (TH0). TL0 is incremented every machine cycle.
; A 2-byte software timer counts to 8192, thus
; with a reload value of 86h=134 in TH0, the
; digit is updated every (256-134)*8192=999424
; microseconds, or about once per second,
; excluding the program overhead.
;
; try different reload values in TH0.
;
; ----------------------------------------------
; system calls              registers used
; ----------------------------------------------

 break     equ  010ch  ; a, (reads accumulator)
 crlf      equ  0115h  ; a
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 print     equ  0136h  ; a, dptr
 prthex    equ  013fh  ; a, r2
; --- constants --------------------------------------
 ScountL    equ  0     ; software counter reload value
                       ; low byte
 ScountH    equ  20h   ; software counter reload value
                       ; high byte
 Reload     equ  86h   ; TL0 auto reloar value in TH0
; ====================================================
;       m a i n    p r o g r a m
; UIOD connections
;
; LOW DIGIT    P4     : output (segment a to P4.0, ...
;                               segment g to P4.6)
; ----------------------------------------------------

      org   8000h         ; run using the monitor
                          ;  G8000 command
      mov   TMOD, #82h    ; configure timer/counter 0
                          ;  timer in mode 1
      mov   TH0, #Reload  ; timer reload value
      setb  TR0           ; turn timer/counter on
      mov   b, #0         ; current count
      mov   r6, #ScountL  ; software counter low byte
      mov   r7, #ScountH  ; software counter high byte

      mov   a, b          ; load a with current count
      lcall display       ; get display pattern
      mov   P4, a         ; display the digit
      mov   a, b          ; get current count
      lcall prthex        ; echo count to host
      lcall crlf          ; new line on host

loop:
      jnb   TF0, loop     ; repeat until Timer 0 times out
      clr   TF0           ; clear timer overflow flag
      djnz  r6, loop      ; decrement software counter
                          ; low byte, repeat until zero
      mov   r0, #ScountL  ; reset software counter
      djnz  r7, loop      ; decrement software counter
                          ; high byte, repeat until zero
      mov   r7, #ScountH  ; reset software counter

      inc   b             ; when software counter overflows
                          ; increment current count
      anl   b, #0Fh       ; modulus 0Fh
      mov   a, b          ; load a with current count
      lcall display       ; get display pattern
      mov   P4, a         ; display the digit
      mov   a, b          ; get current count
      lcall prthex        ; echo count to host
      lcall crlf          ; new line on host

      lcall inkey         ; check serial port for character
      lcall break         ; see if character is CTL-C
                          ; terminate program if CTL-C

      sjmp loop           ; repeat forever...

; ====================================================
; end of program
