; program vmeter1.asm
;
; shows analogue input 0 (an0) in hex on the
;   2 digital displays
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; uiod connections:
;  low digit to port 4 and high digit to port 5.
;  80535 port an0 to a voltage divider network as
;  given in the manual tutorial section.
;
; internal memory use
; bytes:
   analog0 equ 0c0h    ;  0-th adc input
; ----------------------------------------------------
; the voltage seen at the microcontroller pin is
; displayed in hexadecimal (0..ff), 00=0 volts, and
; ff=5 volts.  the jumpers jp2 (vgnd) and jp3 (vref)
; should be installed.
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
; ----------------------------------------------------

        org 8000h

        lcall print
        db   "adc begins", 0
        lcall crlf
        mov   ip0, #0        ;  all interrupts low priority
        mov   ip1, #0
        lcall initadc
        lcall print
        db   "initialization done", 0
        lcall crlf
        setb  eal

begin:
    mov   r0, #analog0
    mov   a, @r0
    push  acc
    push  acc
    anl   a, #0fh
    lcall display
    mov   p4, a
    pop   acc
    swap  a
    anl   a, #0fh
    lcall display
    mov   p5, a

    lcall crlf
    pop   acc
    lcall prthex

    mov   a, #8h
dlp:    lcall mdelay
    djnz  acc, dlp

    lcall inkey
    cjne  a, #3h, begin
    clr   eal
    ljmp  os_return

; ====================================================
;  subroutine initadc - initialize adc
;  input    : none
;  output   : initializes adc interrupt
;             to continuously read analog 0
;  destroys : a, dptr
; ----------------------------------------------------
initadc:
        clr   eadc        ; disable adc interrupts
        clr   iadc        ; reset adc interrupt flag
        mov   a, #6       ; sixth interrupt source
        mov   dptr, #adcisr ; setup interrupt vector
        lcall setintvec
        setb  eadc        ; adc interrupt mask bit

        mov   a, adcon    ; read adc control register
        anl   a, #0d0h    ; select channel 0, single shot
        mov   adcon, a
        mov   dapr, #0    ; start conversion
        ret

; ====================================================
;  subroutine adcisr
;  input    : adc interrupt
;  output   : continuously obtains and stores analog 0
;             value in internal ram.
;  destroys : nothing - uses a, r0, and r1
; ----------------------------------------------------
adcisr:
    push  psw
    mov   psw, #0        ; select register bank 0
    push  acc
    push  1              ; push register 1

    mov   r0,  #analog0  ; analog 0 store address
    mov   a,   addat     ; read analog value
    mov   @r0, a         ; store analog value
    mov   a, adcon       ; adc control register
    anl   a, #0d0h
    mov   adcon, a       ; select next channel
    clr   iadc           ; reset adc interrupt flag
    mov   dapr, #0       ; start next conversion

done:
    pop   1
    pop   acc
    pop   psw
    reti

; ====================================================
;
; end of program
