; program vmeter2.asm
;
; shows analogue input 0 (an0) in hex on the
;   2 digital displays.  allows setting an integer
;   range.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; uiod connections:
;  low digit to port 4 and high digit to port 5.
;  80535 port an0 to the potentiometer an0 of the uiod
;  through a 1 kilo ohm (any value between 1k and 10k
;  will work equally well) resistor.
;
; internal memory use
; bytes:
   analog0 equ 0c0h    ; 0-th adc input
;
; global constants
   range equ   5       ; voltage for 256 count
   wait equ    100     ; delay duration
; ----------------------------------------------------
; the voltage seen at the microcontroller pin is
; displayed in decimal in two parts: first the integer
; voltage in the range (0..99), and then the fraction
; in the range (0..99).  the full range, i.e., the
; value displayed when an0=5 volts must be an integer
; constant.  this constant, named 'range' is defined
; above by the equ psudo-opt.  with range=5, the
; voltmeter measures the true voltage on input an0.
;
; jumpers jp2 and jp3 must be installed.
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 autoexec  equ  0103h
 beep      equ  0106h  ;  none
 binasc    equ  0109h  ; a
 break     equ  010ch  ; a, (reads accumulator)
 chkbrk    equ  010fh  ; a, (reads serial port)
 cret      equ  0112h  ; a
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getbyt    equ  011eh  ; a, b
 getchr    equ  0121h  ; a
 getchrx   equ  0124h  ; a
 init      equ  0127h
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 sdelay    equ  0142h  ; a
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
 errorf    equ     3   ; system error flag
; ----------------------------------------------------

        org 8000h

        mov   ip0, #0        ;  all interrupts low priority
        mov   ip1, #0
        lcall initadc
        setb  eal
begin:
        mov   r0, #analog0
        mov   a, @r0
        mov   b, #range
        mul   ab         ; b holds integer
                         ; a holds fraction
        mov   r4, b      ; save integer in r4
        mov   r5, a      ; save fraction in r5

        mov   a, r4      ; mov integer part
        lcall binbcd     ; convert to bcd
        lcall shwbcd     ; display it
        mov   a, #wait
        lcall delay      ; show for a while

        mov   a, r5      ; mov fraction part
        lcall percent    ; convert to decimal fraction
        lcall shwbcd     ; display it
        mov   a, #wait
        lcall delay      ; show for a while

        mov   p4, #0ffh  ; blank low digit
        mov   p5, #0ffh  ; blank high digit
        mov   a, #wait
        lcall delay      ; show for a while

        lcall inkey
        cjne  a, #3h, begin
        clr   eal
        ljmp  os_return

; ====================================================
;  subroutine initadc - initialize adc
;  input    : none
;  output   : initializes adc interrupt
;             to continuously read analog 0
;  destroys : a, dptr
; ----------------------------------------------------
initadc:
        clr   eadc        ; disable adc interrupts
        clr   iadc        ; reset adc interrupt flag
        mov   a, #6       ; sixth interrupt source
        mov   dptr, #adcisr ; setup interrupt vector
        lcall setintvec
        setb  eadc        ; adc interrupt mask bit

        mov   a, adcon    ; read adc control register
        anl   a, #0d0h    ; select channel 0, single shot
        mov   adcon, a
        mov   dapr, #0    ; start conversion
        ret

; ====================================================
;  subroutine adcisr
;  input    : adc interrupt
;  output   : continuously obtains and stores analog 0
;             value in internal ram.
;  destroys : nothing - uses a, r0, and r1
; ----------------------------------------------------
adcisr:
        push  psw
        mov   psw, #0        ; select register bank 0
        push  acc
        push  1              ; push register 1

        mov   r0,  #analog0  ; analog 0 store address
        mov   a,   addat     ; read analog value
        mov   @r0, a         ; store analog value
        mov   a, adcon       ; adc control register
        anl   a, #0d0h
        mov   adcon, a       ; select next channel
        clr   iadc           ; reset adc interrupt flag
        mov   dapr, #0       ; start next conversion

done:   pop   1
        pop   acc
        pop   psw
        reti

; ====================================================
; subroutine shwbcd - shows a bcd number on the
;                        2 digital displays
;  input    : bcd number in the accumulator
;  output   : ports 4 and 5, connected to low digit
;             and high digit respectively.
;  destroys : a
; ----------------------------------------------------
shwbcd:
        push  acc
        anl   a, #0fh
        lcall display
        mov   p4, a
        pop   acc
        swap  a
        anl   a, #0fh
        lcall display
        mov   p5, a
        ret

; ====================================================
; subroutine binbcd - converts the binary number in
;                     the accumulator to bcd.  the
;                     errorf is set if number is > 99
;                        using the 2 digital displays
;  input    : hexadecimal number in the accumulator
;  output   : bcd number in accumulator
;             errorf=1 if number>99
;  uses     : a, b
;  destroys : a
; ----------------------------------------------------
binbcd:
        push  b
        clr   errorf
        mov   b, #100 ; to divide n by 100
        div   ab
        jz    binbcd1 ; if n>99...
        setb  errorf  ; ...set errorf
binbcd1:
        mov   a, #10  ; to divide n by 10
        xch   a, b
        div   ab
        swap  a       ; move 10s into high nibble
        add   a, b    ; move 1s  into low nibble
        pop   b
        ret

; ====================================================
;
; end of program
